// BCGPEditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "BCGPDrawManager.h"
#include "BCGPEditCtrl.h"
#include "BCGPIntelliSenseWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const CString g_strEOL = _T ("\n");
static const TCHAR g_chEOL = _T ('\n');
static const CString g_strEOLExport = _T ("\r\n");

static CString strTipText;

static CLIPFORMAT g_cFormat = 0;

IMPLEMENT_DYNAMIC(CBCGPEditCtrl,CWnd)
IMPLEMENT_DYNAMIC(CBCGPEditMarker,CObject)
IMPLEMENT_DYNAMIC(CBCGPLineColorMarker,CBCGPEditMarker)

#define ID_TEXT_SCROLL				1
#define _MAX_COLOR_BLOCK_STR_LEN	4

BOOL CBCGPEditCtrl::m_bOvrMode = FALSE;

UINT BCGM_ON_EDITCHANGE	= ::RegisterWindowMessage (_T("BCGM_ON_EDITCHANGE"));

/////////////////////////////////////////////////////////////////////////////
// CBCGPEditCtrl

CBCGPEditCtrl::CBCGPEditCtrl() :
	m_clrBack ((COLORREF) -1),
	m_clrText ((COLORREF) -1),
	m_nLineHeight (0),
	m_nMaxCharWidth (0),
	m_hFont (NULL),
	m_nCurrOffset (0),
	m_nLeftMarginWidth (20),
	m_ptCaret (CPoint (m_nLeftMarginWidth, 0)),
	m_nTabSize (4),
	m_iStartSel (-1),
	m_iEndSel (-1),
	m_bEnableWholeTextCopy (FALSE),
	m_nScrollOffsetHorz (0),
	m_nScrollOffsetVert (0),
	m_nTopVisibleOffset (0),
	m_szTotalScroll (0, 0),
	m_nCurRow (0),
	m_nCurColumn (0),
	m_nLastSelOffset (-1),
	m_nLastMaxColumn (0),
	m_posUndoBuffer (NULL),
	m_nUndoBufferSize (300),
	m_bEOL (FALSE),
	m_bBOL (FALSE),
	m_bIntelliSenseSupport(FALSE),
	m_bIsLastActionUndo (TRUE),
	m_bUndoCharMode (FALSE),
	m_bDragTextMode (FALSE),
	m_nSavedOffset (0), 
	m_nDropOffset (-1),
	m_bDefaultIndent (TRUE),
	m_pIntelliSenseWnd (NULL),
	m_pIntelliSenseImgList (NULL),
	m_pIntelliSenseLBFont (NULL),
	m_bIntelliSenseMode (FALSE),
	m_bKeepTabs (TRUE), 
	m_nTabLogicalSize (0),
	m_bBlockSelectionMode (FALSE),
	m_bAltPressedForBlockSel (FALSE),
	m_nIndentSize (4),
	m_bReadOnly (FALSE),
	m_nLineVertSpacing (1),
	m_nMaxScrollWidth (2048),
	m_nTotalLines (0),
	m_bEnableToolTips (FALSE),
	m_bReplaceTabsAndEOLOnCopy (TRUE),
	m_bIntelliSenseUpdate(FALSE),
	m_bIsModified (FALSE),
	m_nScrollTimeOut (100),
	m_nScrollTimer (-1),
	m_bToolTipCleared (FALSE),
	m_posDocSavedUndoPos (NULL),
	m_dwLastUndoReason (g_dwUATUndefined)
{
	m_strWordDelimeters = _T(" \t\n,./?<>;:\"'{[}]~`%^&*()-+=!");
	m_strSpecialDelimiters = _T (";.{}\n");
	m_strNonSelectableChars = _T (" \n\t");
	m_strIndentChars = _T (" \t");
	m_strIntelliSenseChars = _T (".>:");
	m_strContinueStringSymbols = _T ("\\");
	m_rectText.SetRectEmpty ();
	m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);

	m_nScrollRightOffset = 15; // percents
	m_nScrollLeftOffset = 2;   // symbols

	m_nPrevKey = m_nLastKey = -1;

	m_hPrinterFont = NULL;
	m_hMirrorFont = NULL;

	m_nDlgCode = DLGC_WANTALLKEYS | DLGC_WANTMESSAGE;

	m_pSymbolsImgList = NULL;
	m_bEnableSymSupport = FALSE;
}

CBCGPEditCtrl::~CBCGPEditCtrl ()
{
	CleanUpMarkers ();
	::DestroyCaret ();
	EmptyUndoList ();
}

void CBCGPEditCtrl::EmptyUndoList ()
{
	while (!m_lstUndoActions.IsEmpty ())
	{
		BCGP_EDIT_UNDO_ACTION* pUndoAction = m_lstUndoActions.GetHead ();
		OnRemoveUndoAction (pUndoAction);
		delete m_lstUndoActions.RemoveHead ();
	}

	m_posUndoBuffer = NULL;
	m_bEOL = FALSE;
	m_bBOL = FALSE;
	m_bIsLastActionUndo = TRUE;
}

BEGIN_MESSAGE_MAP(CBCGPEditCtrl, CWnd)
	//{{AFX_MSG_MAP(CBCGPEditCtrl)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_SETCURSOR()
	ON_WM_KEYDOWN()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_CHAR()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_RBUTTONDOWN()
	ON_WM_TIMER()
	ON_WM_SYSCOLORCHANGE()
	ON_WM_GETDLGCODE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_SETFONT, OnSetFont)
	ON_MESSAGE(WM_GETFONT, OnGetFont)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXT, 0, 0xFFFF, OnTTNeedTipText)
	ON_MESSAGE(WM_SETTEXT, OnSetText)
	ON_MESSAGE(WM_GETTEXT, OnGetText)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBCGPEditCtrl message handlers

int CBCGPEditCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_DropTarget.Register (this);

	CRect rectClient;
	GetClientRect (&rectClient);

	m_wndToolTip.Create (this, TTS_ALWAYSTIP | TTS_NOPREFIX);
	m_wndToolTip.AddTool (this, LPSTR_TEXTCALLBACK, &rectClient, GetDlgCtrlID ());

	Initialize ();
	SetCaret (0);
	return 0;
}
//********************************************************************************
void CBCGPEditCtrl::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();
}
//********************************************************************************
void CBCGPEditCtrl::Initialize ()
{
	//-----------------
	// Initialize font:
	//-----------------
	OnChangeFont ();

	InitColors ();

	g_cFormat = (CLIPFORMAT) RegisterClipboardFormat (_T ("BCGP_TEXT"));

	CString strBuffer;
	ProcessTextBeforeInsert (strBuffer);
	InsertText (strBuffer, -1, FALSE, TRUE);
	m_strBuffer = strBuffer;

	ModifyStyle (0, WS_HSCROLL | WS_VSCROLL);
}
//********************************************************************************
BOOL CBCGPEditCtrl::OnEraseBkgnd(CDC* /*pDC*/)
{
	return TRUE;
}
//*******************************************************************************
void CBCGPEditCtrl::OnPaint() 
{
	CPaintDC dcPaint(this); // device context for painting

	CRect rectClient;
	GetClientRect (rectClient);

	CRect rectClip;
	dcPaint.GetClipBox (rectClip);

	CDC*		pDC = &dcPaint;
	BOOL		bMemDC = FALSE;
	CDC			dcMem;
	CBitmap		bmp;
	CBitmap*	pOldBmp = NULL;

	if (dcMem.CreateCompatibleDC (&dcPaint) &&
		bmp.CreateCompatibleBitmap (&dcPaint, rectClient.Width (),
								  rectClient.Height ()))
	{
		//-------------------------------------------------------------
		// Off-screen DC successfully created. Better paint to it then!
		//-------------------------------------------------------------
		bMemDC = TRUE;
		pOldBmp = dcMem.SelectObject (&bmp);
		pDC = &dcMem;
	}

	//-------------------------
	// Fill control background:
	//-------------------------
	OnFillBackground (pDC);

	//---------------------
	// Set text attributes:
	//---------------------
	int nOldBkMode = pDC->SetBkMode (TRANSPARENT);
	COLORREF clrOldText = pDC->SetTextColor (GetDefaultTextColor ());

	CFont* pOldFont = SelectFont (pDC);

	int nVertOffset = m_nScrollOffsetVert * m_nLineHeight;

	//---------------
	// Draw side bar:
	//---------------
	if (m_nLeftMarginWidth > 0 && rectClip.left < m_nLeftMarginWidth)
	{
		CRect rectLeft = rectClient;
		rectLeft.right = rectLeft.left + m_nLeftMarginWidth;
		rectLeft.top -= nVertOffset;

		OnDrawSideBar (pDC, rectLeft);
	}

	//-------------
	// Draw border:
	//-------------
	if (GetExStyle () & WS_EX_CLIENTEDGE)
	{
		pDC->DrawEdge (rectClient, BDR_SUNKENINNER, BF_RECT);
	}
	else if (GetExStyle () & WS_EX_WINDOWEDGE)
	{
		pDC->DrawEdge (rectClient, BDR_RAISEDINNER, BF_RECT);
	}
	else if (GetStyle () & WS_BORDER)
	{
		pDC->Draw3dRect (rectClient, ::GetSysColor (COLOR_WINDOWTEXT),
			::GetSysColor (COLOR_WINDOWTEXT));
	}

	//-----------
	// Draw text:
	//-----------
	if (!m_strBuffer.IsEmpty ())
	{
		rectClip.IntersectRect (rectClip, m_rectText);
		CRgn rgnClip;
		rgnClip.CreateRectRgnIndirect (&rectClip);

		pDC->SelectClipRgn (&rgnClip);
		OnDrawText (pDC);
		pDC->SelectClipRgn (NULL);	
		rectClip = rectClient;
	}
	
	pDC->SelectObject (pOldFont);
	pDC->SetTextColor (clrOldText);
	pDC->SetBkMode (nOldBkMode);

	if (bMemDC)
	{
		//--------------------------------------
		// Copy the results to the on-screen DC:
		//-------------------------------------- 
		dcPaint.BitBlt (rectClip.left, rectClip.top, rectClip.Width(), rectClip.Height(),
					   &dcMem, rectClip.left, rectClip.top, SRCCOPY);

		dcMem.SelectObject(pOldBmp);
	}
}
//*****************************************************************************
afx_msg LRESULT CBCGPEditCtrl::OnSetFont (WPARAM wParam, LPARAM lParam)
{
	m_hFont = (HFONT) wParam;
	OnChangeFont ();

	if ((BOOL)lParam)
	{
		RedrawWindow ();
	}

	return 0;
}
//*****************************************************************************
afx_msg LRESULT CBCGPEditCtrl::OnGetFont (WPARAM, LPARAM)
{
	return (LRESULT) m_hFont;
}
//*****************************************************************************
CFont* CBCGPEditCtrl::SelectFont (CDC* pDC)
{
	CFont* pOldFont = NULL;

	if (pDC->IsPrinting ())
	{
		if (m_hPrinterFont != NULL)
		{
			pOldFont = pDC->SelectObject (CFont::FromHandle(m_hPrinterFont));
		}
	}
	else
	{
		pOldFont = (m_hFont == NULL) ?
			(CFont*) pDC->SelectStockObject (DEFAULT_GUI_FONT) :
			pDC->SelectObject (CFont::FromHandle (m_hFont));
	}

	ASSERT (pOldFont != NULL);
	return pOldFont;
}
//*****************************************************************************
LRESULT CBCGPEditCtrl::OnSetText (WPARAM, LPARAM lp)
{
	LPCTSTR lpszText = (LPCTSTR) lp;

	m_strBuffer = _T("");
	m_nCurrOffset = 0;
	m_iEndSel = -1;
	m_iStartSel = -1;

	if (lpszText != NULL)
	{
		InsertText (lpszText, -1, FALSE, TRUE, FALSE);
	}
	
	m_posUndoBuffer = NULL;
	
	while (!m_lstUndoActions.IsEmpty ())
	{
		delete m_lstUndoActions.RemoveHead ();
	}

	SetCaret (0);
	return TRUE;
}
//*****************************************************************************
LRESULT CBCGPEditCtrl::OnGetText (WPARAM wp, LPARAM lp)
{
	int nMaxChars = min ((int) wp, m_strBuffer.GetLength ());

	LPTSTR lpszText = (LPTSTR) lp;
	ASSERT (lpszText != NULL);

	::lstrcpyn (lpszText, m_strBuffer, nMaxChars);

	return nMaxChars;
}
//*****************************************************************************
BOOL CBCGPEditCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CPoint ptCursor;
	::GetCursorPos (&ptCursor);
	ScreenToClient (&ptCursor);

	if (m_rectText.PtInRect (ptCursor))
	{
		if (IsPointInSelectionRect (ptCursor))
		{
			return CWnd::OnSetCursor(pWnd, nHitTest, message);
		}

		if (m_bReadOnly)
		{
			CPoint pt = ptCursor;
			HitTest (pt, TRUE, TRUE);
			
			if (abs (pt.x - ptCursor.x) > m_nMaxCharWidth ||
				abs (pt.y - ptCursor.y) > m_nLineHeight)
			{
				return CWnd::OnSetCursor(pWnd, nHitTest, message);
			}
		}

		::SetCursor (AfxGetApp ()->LoadStandardCursor (IDC_IBEAM));
		return TRUE;
	}

	return CWnd::OnSetCursor(pWnd, nHitTest, message);
}
//*****************************************************************************
void CBCGPEditCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	const BOOL bShift = ::GetAsyncKeyState (VK_SHIFT) & 0x8000;
	const BOOL bCtrl = ::GetAsyncKeyState (VK_CONTROL) & 0x8000;

	if (nChar > VK_ESCAPE && nChar < VK_NUMLOCK)
	{
		ToolTipPop ();
	}

	if (m_bReadOnly)
	{
		switch (nChar)
		{
		case VK_LEFT:
			ScrollUp (SB_HORZ, TRUE);
			break;

		case VK_RIGHT:
			ScrollDown (SB_HORZ, TRUE);
			break;

		case VK_HOME:
			StartOfText ();
			break;

		case VK_END:
			EndOfText ();
			break;

		case VK_UP:
			ScrollUp (SB_VERT, TRUE);
			break;

		case VK_DOWN:
			ScrollDown (SB_VERT, TRUE);
			break;

		case VK_PRIOR:
			PageUp ();
			break;

		case VK_NEXT:
			PageDown ();
			break;

		case VK_RETURN:
			break;

		case VK_TAB:
			bShift ? PageUp () : PageDown ();
			break;
																				   
		case VK_BACK:
			break;
													  
		case VK_DELETE:
			break;

		case VK_ESCAPE:
			break;

		case VK_INSERT:
			break;
		}
		CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
		return;
	}							 

	m_nPrevKey = m_nLastKey;
	m_nLastKey = nChar;

	if (nChar == VK_ESCAPE && m_iStartSel != -1 ||
		nChar == VK_DELETE || nChar == VK_UP || nChar == VK_DOWN ||
		nChar == VK_PRIOR || nChar == VK_NEXT)
    {																																																																																																																																																																									
		CPoint ptCurrOffset;
		OffsetToPoint (m_nCurrOffset, ptCurrOffset);

		if (!m_rectText.PtInRect (ptCurrOffset))
		{
			SetCaret (m_nCurrOffset, TRUE);
		}
    }
		
	switch (nChar)
	{
	case VK_LEFT:
		if (bCtrl)
		{
			if (bShift)
			{
				MakeSelection (ST_PREV_WORD);
			}
			else
			{
				RemoveSelection ();
				PrevWord ();
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_PREV_SYMBOL);
		}
		else
		{
			if (!RemoveSelection ())
			{
				Left ();
			}
		}
		break;

	case VK_RIGHT:
		if (bCtrl)
		{
			if (bShift)
			{
				MakeSelection (ST_NEXT_WORD);
			}
			else
			{
				RemoveSelection (FALSE);
				NextWord ();
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_NEXT_SYMBOL);
		}
		else
		{
			if (!RemoveSelection (FALSE))
			{
				Right ();
			}
		}
		break;

	case VK_HOME:
		if (bCtrl)
		{
			if (bShift)
			{
				MakeSelection (ST_START_OF_TEXT);
			}
			else
			{
				RemoveSelection ();
				StartOfText ();
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_HOME);
		}
		else
		{
			RemoveSelection ();
			Home ();
		}
		break;

	case VK_END:
		if (bCtrl)
		{
			if (bShift)
			{
				MakeSelection (ST_END_OF_TEXT);
			}
			else
			{
				RemoveSelection (FALSE);
				EndOfText ();
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_END);
		}
		else
		{
			RemoveSelection (FALSE);
			End ();
		}
		break;

	case VK_UP:
		if (bCtrl)
		{
			if (!bShift)
			{
				ScrollText (FALSE);
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_PREV_LINE);
		}
		else
		{
			BOOL bResult = RemoveSelection ();
			Up (!bResult);
		}
		break;

	case VK_DOWN:
		if (bCtrl)
		{
			if (!bShift)
			{
				ScrollText (TRUE);
			}
		}
		else if (bShift)
		{
			MakeSelection (ST_NEXT_LINE);
		}
		else
		{
			BOOL bResult = RemoveSelection (FALSE);
			Down (!bResult);
		}
		break;

	case VK_PRIOR:
		if (!bCtrl)
		{
			if (!bShift)
			{
				RemoveSelection ();
				PageUp ();
			}
			else
			{
				MakeSelection (ST_PREV_PAGE);
			}
		}
		break;

	case VK_NEXT:
		if (!bCtrl)
		{
			if (!bShift)
			{
				RemoveSelection ();
				PageDown ();
			}
			else
			{
				MakeSelection (ST_NEXT_PAGE);
			}
		}
		break;

	case VK_RETURN:
		DeleteSelectedText (TRUE, FALSE, TRUE);		
		if (GetOverrideMode ())
		{
			Down ();
			Home ();
		}
		else
		{
			SetLastUndoReason (g_dwUATEnter);
			InsertNewLine (TRUE);
		}
		m_nLastMaxColumn = GetCurColumn ();
		break;

	case VK_TAB:
		if (!bCtrl)
		{
			if (!bShift && m_iStartSel == -1)
			{
				SetLastUndoReason (g_dwUATTab);
				InsertTab (TRUE);
			}	
			else if (bShift && m_iStartSel == -1)
			{
				PrevIndent ();
			}
			else if (bShift && m_iStartSel != -1)
			{
				IndentSelection (FALSE);
			}
			else
			{
				if (!IndentSelection (TRUE))
				{
					DeleteSelectedText (FALSE, FALSE, TRUE);
					InsertTab (TRUE);
				}
			}
		}
		break;

	case VK_BACK:
		if (m_nCurrOffset > 0)
		{
			CPoint pt;
			OffsetToPoint (m_nCurrOffset - 1, pt);

			if (pt.y >= 0)
			{
				m_nCurrOffset--;
				SetLastUndoReason (g_dwUATBackspace);
				if (!OnDelete (TRUE))
				{
					m_nCurrOffset++;
				}
				SetCaret (m_nCurrOffset, TRUE, TRUE);
			}
			else
			{
				int nOldOffset = m_nCurrOffset;
				SetCaret (m_nCurrOffset - 1, TRUE, FALSE);
				SetLastUndoReason (g_dwUATBackspace	);
				if (!OnDelete (TRUE))
				{
					SetCaret (nOldOffset, TRUE, TRUE);
				}
			}
		}
		break;									  
												  
	case VK_DELETE:
		SetLastUndoReason (g_dwUATDelete);
		OnDelete (TRUE);
		break;

	case VK_ESCAPE:
		RemoveSelection (FALSE, TRUE);
		break;

	case VK_INSERT:
		OnSetOvrMode ();
		break;
	}
		  
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}
//*****************************************************************************
BOOL CBCGPEditCtrl::OnSetOvrMode ()
{
	if (m_bReadOnly)
	{
		OnFailedOperation (g_dwOpSetOvrMode | g_dwOpReasonReadOnly);
		return FALSE;
	}

	SetOverrideMode (!GetOverrideMode ());
	::DestroyCaret ();
	CSize sizeCaret = GetCaretSize ();
	::CreateCaret (GetSafeHwnd (), NULL, sizeCaret.cx, sizeCaret.cy);
	ShowCaret ();
	return TRUE;
}
//*****************************************************************************
CSize CBCGPEditCtrl::GetCaretSize () const
{
	return GetOverrideMode () ? CSize (m_nMaxCharWidth, m_nLineHeight) : 
						CSize (2, m_nLineHeight);
}
//*****************************************************************************
void CBCGPEditCtrl::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if ((isprint (nChar) || nChar >= 127) && !m_bReadOnly)
	{
		DeleteSelectedText (TRUE, FALSE, TRUE);
		SetLastUndoReason (g_dwUATTyping);
		InsertChar ((TCHAR) nChar, TRUE);

		CObList lstIntelliSenseData;
		CString strIntelliSense;
		int nCurrOffset = m_nCurrOffset;

		if (IsIntelliSenseEnabled() &&
			(nCurrOffset > 0) &&
			(nCurrOffset <= m_strBuffer.GetLength()) &&
			m_strIntelliSenseChars.Find (m_strBuffer.GetAt(nCurrOffset - 1)) != -1)
		{
			if (OnFillIntelliSenseList (nCurrOffset, lstIntelliSenseData))
			{
				CPoint pt (0, 0);

				OffsetToPoint (m_nCurrOffset = nCurrOffset, pt);
				ClientToScreen (&pt);

				InvokeIntelliSense (lstIntelliSenseData, pt);
			}
		}
	}

	CWnd::OnChar(nChar, nRepCnt, nFlags);
}
//*****************************************************************************
BOOL CBCGPEditCtrl::OnFillIntelliSenseList (int& nCurrOffset, CObList& lstIntelliSenseData) const
{
	CString strIntelliSence;

	if (!OnBeforeInvokeIntelliSense(m_strBuffer, nCurrOffset, strIntelliSence))
	{
		return FALSE;
	}

	VERIFY(FillIntelliSenseList(lstIntelliSenseData, strIntelliSence.GetBuffer(0)));

	return !lstIntelliSenseData.IsEmpty();
}
//*****************************************************************************
BOOL CBCGPEditCtrl::InvokeIntelliSense()
{
	CObList lstIntelliSenseData;
	int nCurrOffset = m_nCurrOffset;

	if (!OnFillIntelliSenseList(nCurrOffset, lstIntelliSenseData))
	{
		OnFailedOperation (g_dwOpInvokeIS);
		return FALSE;
	}

	CPoint pt;

	OffsetToPoint(m_nCurrOffset, pt);
	ClientToScreen (&pt);
	
	return InvokeIntelliSense (lstIntelliSenseData, pt);
}
//*****************************************************************************
BOOL CBCGPEditCtrl::InvokeIntelliSense (CObList& lstIntelliSenseData, 
										CPoint ptTopLeft)
{
	ASSERT (m_bIntelliSenseSupport);
	
	if (lstIntelliSenseData.IsEmpty())
	{
		return FALSE;
	}

	CBCGPIntelliSenseWnd* pIntelliSenseWnd = new CBCGPIntelliSenseWnd;
	return pIntelliSenseWnd->Create (lstIntelliSenseData, 
		WS_POPUP | WS_VISIBLE | MFS_SYNCACTIVE | MFS_4THICKFRAME, 
		ptTopLeft, this, m_pIntelliSenseLBFont, m_pIntelliSenseImgList);
}
//*****************************************************************************
BOOL CBCGPEditCtrl::InvokeSymList ()
{
	ASSERT (m_bEnableSymSupport);
	CObList lstIntelliSenseData;
	int nCurrOffset = m_nCurrOffset;

	if (!OnFillSymListData (nCurrOffset, lstIntelliSenseData))
	{
		OnFailedOperation (g_dwOpInvokeIS);
		return FALSE;
	}

	CPoint pt;

	OffsetToPoint(m_nCurrOffset, pt);
	ClientToScreen (&pt);

	return InvokeSymList (lstIntelliSenseData, pt);
}
//*****************************************************************************
BOOL CBCGPEditCtrl::OnFillSymListData (int& nCurrOffset, CObList& lstIntelliSenseData) const 
{
	ASSERT (m_bEnableSymSupport);

	for (POSITION pos = m_lstSymDefs.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_SYM_DEF symDef = m_lstSymDefs.GetNext (pos);
		CBCGPIntelliSenseData* pData = new CBCGPIntelliSenseData;
		pData->m_nImageListIndex = symDef.m_nIndex;
		pData->m_strItemName = symDef.m_strSymText;
		pData->m_dwData = symDef.m_chReplacement;
		lstIntelliSenseData.AddTail (pData);
	}

	return !lstIntelliSenseData.IsEmpty ();
}
//*****************************************************************************
BOOL CBCGPEditCtrl::InvokeSymList (CObList& lstIntelliSenseData, 
									CPoint ptTopLeft)
{
	ASSERT (m_bEnableSymSupport);
	if (lstIntelliSenseData.IsEmpty())
	{
		return FALSE;
	}

	CBCGPIntelliSenseWnd* pIntelliSenseWnd = new CBCGPIntelliSenseWnd;
	return pIntelliSenseWnd->Create (lstIntelliSenseData, 
		WS_POPUP | WS_VISIBLE | MFS_SYNCACTIVE | MFS_4THICKFRAME, 
		ptTopLeft, this, m_pIntelliSenseLBFont, m_pSymbolsImgList, 
		RUNTIME_CLASS (CBCGPSymImagesLB));
}
//**************************************************************************************
BOOL CBCGPEditCtrl::LookUpSymbol (LPCTSTR lpcszBuffer, int nOffset, int nCount, 
								  TCHAR chSymbol, BCGP_EDIT_SYM_DEF& symDef, 
								  int& nSymExtraLen, BOOL bForward)
{
	ASSERT (m_bEnableSymSupport);
	for (POSITION pos = m_lstSymDefs.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_SYM_DEF& nextSym = m_lstSymDefs.GetNext (pos);
		if (nextSym.m_chReplacement == chSymbol)
		{
			symDef = nextSym;
			nSymExtraLen = 0;
			return TRUE;
		}

		if (!nextSym.m_strSymSequence.IsEmpty ())
		{
			int nLen = nextSym.m_strSymSequence.GetLength ();
			if (bForward && nCount - nOffset < nLen ||
				!bForward && nOffset - nLen + 1 < 0)
			{
				continue;
			}
			LPCTSTR lpcszBuffToCompare = bForward ? lpcszBuffer + nOffset : 
										 lpcszBuffer + nOffset - nLen + 1;			
			if (_tcsncmp (lpcszBuffToCompare, nextSym.m_strSymSequence, nLen * sizeof (TCHAR)) == 0)
			{
				symDef = nextSym;
				nSymExtraLen = nLen - 1;
				return TRUE;
			}
		}
	}
	
	return FALSE;
}
//*****************************************************************************
int  CBCGPEditCtrl::GetSymbolExtraLen (CString& strBuffer, int nOffset, BOOL bForward)
{
	ASSERT (m_bEnableSymSupport);
	int nSymExtraLen = 0;
	BCGP_EDIT_SYM_DEF symDef;

	if (!bForward)
	{
		nOffset--;
	}

	if (nOffset < 0)
	{
		nOffset = 0;
	}

	LookUpSymbol (strBuffer, nOffset, strBuffer.GetLength (), strBuffer.GetAt (nOffset), 
						symDef, nSymExtraLen, bForward);
	return nSymExtraLen;
}
//*****************************************************************************
BOOL CBCGPEditCtrl::EnableIntelliSense(BOOL bFl /* = TRUE */)
{
	const BOOL bIntelliSense = m_bIntelliSenseSupport;
	m_bIntelliSenseSupport = bFl;

	return bIntelliSense;
}
//*****************************************************************************
void CBCGPEditCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	CRect rectClient;
	GetClientRect (rectClient);

	m_rectText = rectClient;
	m_rectText.left += m_nLeftMarginWidth;

	UpdateScrollBars ();

	SetCaret (m_nCurrOffset, FALSE);

	if (m_nTotalLines < m_rectText.Height () / m_nLineHeight &&
		m_nTopVisibleOffset > 0)
	{
		OnScroll (SB_VERT, SB_THUMBPOSITION, 0);
	}
	else
	{
		SetScrollPos (SB_VERT, m_nScrollOffsetVert, FALSE);
	}
	
	SetScrollPos (SB_HORZ, m_nScrollOffsetHorz, FALSE);

	CRect rect (0, 0, cx, cy);
	m_wndToolTip.SetToolRect (this, GetDlgCtrlID (), rect);
}
//*****************************************************************************
int CBCGPEditCtrl::HitTest (CPoint& pt, BOOL bNormalize, BOOL bIgnoreTextBounds)
{
	if (!m_rectText.PtInRect (pt) && !bIgnoreTextBounds)
	{
		return -1;
	}

	int x = -1;
	int y = m_rectText.top; // - m_nScrollOffsetVert * m_nLineHeight;
	int nOffset = 0;

	CClientDC dc (this);
	CFont* pOldFont = SelectFont (&dc);

	int nRow = max (0, (pt.y + m_nScrollOffsetVert * m_nLineHeight) / m_nLineHeight);
	int nColumn = 0;
	int iRowOffset = m_nTopVisibleOffset;
	
	int nStrartFrom = m_nScrollOffsetVert;
	if (pt.y < 0)
	{
		iRowOffset = GetRowStart (nRow);
		nStrartFrom = nRow;
	}


	for (int i = nStrartFrom; i <= nRow; i++)
	{
		int iNextRow = m_strBuffer.Find (_T('\n'), iRowOffset);

		if (i == nRow)
		{
			CString strRow;
			if (iNextRow < 0)
			{
				strRow = m_strBuffer.Mid (iRowOffset);
			}
			else
			{
				strRow = m_strBuffer.Mid (iRowOffset, iNextRow - iRowOffset);
			}

			int nXRight = m_rectText.left + GetStringExtent (&dc, strRow, strRow.GetLength ()).cx;

			if (pt.x >= nXRight)
			{
				x = nXRight;
				nOffset = (iNextRow < 0) ? m_strBuffer.GetLength () : iNextRow;
				nColumn = iNextRow - iRowOffset; 
			}
			else
			{
				int cxPrev = m_rectText.left;

				int nScrollOffsetPix = m_nScrollOffsetHorz * m_nMaxCharWidth; //GetHorzRowExtent (&dc, iRowOffset);
				int nOffsetInRowPix = 0;

				nOffset = -1;		

				int nStartOffsetToCheck = 0;	
				int nExtent = 0; //GetStringExtent (&dc, (LPCTSTR) strRow, nStartOffsetToCheck).cx;

				for (int iIdx = nStartOffsetToCheck; iIdx < strRow.GetLength (); iIdx++)
				{
					TCHAR chNext = strRow [iIdx];
					int nSymExtraLen = 0;
					int nCharWidth = 0;
					if (m_bEnableSymSupport)
					{
						BCGP_EDIT_SYM_DEF symDef;
						if (LookUpSymbol (strRow, iIdx, strRow.GetLength (), chNext, symDef, nSymExtraLen))
						{
							IMAGEINFO imgInfo;
							m_pSymbolsImgList->GetImageInfo (symDef.m_nIndex, &imgInfo);
							nCharWidth = imgInfo.rcImage.right - imgInfo.rcImage.left;
						}
					}
					if (nCharWidth == 0 && !m_mapCharWidth.Lookup (chNext, nCharWidth))
					{
						nCharWidth = GetStringExtent (&dc, CString (chNext), 1).cx;
						m_mapCharWidth.SetAt (chNext, nCharWidth);
					}

					if (chNext == _T ('\t'))
					{
						int nRestOfTab = m_nTabSize - nColumn % m_nTabSize;
						
						nColumn += nRestOfTab; 
						nCharWidth = nRestOfTab * m_nAveCharWidth;
					}
					else
					{
						nColumn++;
					}

					nExtent += nCharWidth;

					int cx = m_rectText.left + nExtent;
					int nAddition = iIdx == strRow.GetLength () - 1  ? 0 : nCharWidth / 2;

					if (cx > pt.x + nScrollOffsetPix + nAddition)
					{
						x = cxPrev;
						nOffset = iRowOffset + iIdx;
						nOffsetInRowPix = cx - m_rectText.left;
						break;
					}

					cxPrev = cx;
					iIdx += nSymExtraLen;
				}

				if (nOffset == -1)
				{
					nOffset = iRowOffset + i;
				}

				nColumn++;
			}

			break;
		}

		iRowOffset = iNextRow + 1;
		y += m_nLineHeight;

		if (iNextRow == -1 && x == -1)
		{
			x = m_rectText.left;
			y -= m_nLineHeight;
			nOffset = m_strBuffer.GetLength ();
			break;
		}
	}

	dc.SelectObject (pOldFont);

	if (bNormalize)
	{
		pt.x = x;
		pt.y = y;
	}

	OnSetCaret	();
	return nOffset;
}
//******************************************************************************
BOOL CBCGPEditCtrl::SetCaret (int nOffset, BOOL bScrollToCaret, BOOL bRedrawOnScroll)
{
	CPoint pt;
	CPoint ptRowColumn;

	if (!OffsetToPoint (nOffset, pt, &ptRowColumn))
	{
		return FALSE;
	}

	m_nCurrOffset = nOffset;
	m_ptCaret = pt;


	BOOL bScrollRequired = FALSE;
	BOOL bCaretSet = FALSE;
	
	if (bScrollToCaret)
	{
		int nDeltaX = 0;
		
		if (m_ptCaret.x < m_rectText.left + m_nMaxCharWidth)
		{
			nDeltaX = ((m_ptCaret.x - m_rectText.left) / m_nMaxCharWidth) - m_nScrollLeftOffset;
		}
		else if (m_ptCaret.x >= m_rectText.right - m_nMaxCharWidth)
		{
			int nScrollOffset = m_rectText.Width () / m_nMaxCharWidth * m_nScrollRightOffset / 100;
			if (nScrollOffset < 1)
			{
				nScrollOffset = 1;
			}
			nDeltaX = ((m_ptCaret.x - m_rectText.right) / m_nMaxCharWidth) + nScrollOffset;
		}

		if (nDeltaX != 0)
		{
			int nScrollValue = m_nScrollOffsetHorz + nDeltaX;
			if (nDeltaX < 0)
			{
				int nRowEndOffset = 0;
				int nNumColumnsInRow = GetNumOfColumnsInRowByOffset (nOffset, nRowEndOffset);
				if (nScrollValue > nNumColumnsInRow - m_nScrollLeftOffset)
				{
					nScrollValue = max (0, nNumColumnsInRow - m_nScrollLeftOffset);
				}
			}
			bScrollRequired = OnScroll (SB_HORZ, SB_THUMBPOSITION, nScrollValue);
			OffsetToPoint (nOffset, pt, &ptRowColumn);
			m_nCurrOffset = nOffset;
			m_ptCaret = pt;
			SetCaretPos (m_ptCaret);
			OnSetCaret	();
			bCaretSet = TRUE;
		}

		int nDeltaY = 0;
		if (m_ptCaret.y < m_rectText.top + m_nLineHeight)
		{
			nDeltaY = ((m_ptCaret.y - m_rectText.top) / m_nLineHeight);
		}
		else if (m_ptCaret.y >= m_rectText.bottom - m_nLineHeight * 2)
		{
			nDeltaY = ((m_ptCaret.y - m_rectText.bottom + m_nLineHeight * 2) / m_nLineHeight);
		}

		if (nDeltaY != 0)
		{
			bScrollRequired = OnScroll (SB_VERT, SB_THUMBPOSITION, m_nScrollOffsetVert + nDeltaY);
			OffsetToPoint (nOffset, pt, &ptRowColumn);
			m_nCurrOffset = nOffset;
			m_ptCaret = pt;
			SetCaretPos (m_ptCaret);
			OnSetCaret	();
			bCaretSet = TRUE;
		}
	}

	m_nCurRow	 = ptRowColumn.y;
	m_nCurColumn = GetColumnFromOffset (m_nCurrOffset);

	if (!bCaretSet && bScrollToCaret)
	{
		SetCaretPos (m_ptCaret);
		OnSetCaret	();
	}

	if (bRedrawOnScroll && bScrollRequired)
	{
		RedrawWindow ();
	}

	return TRUE;
}
//***************************************************************************************
int CBCGPEditCtrl::GetColumnFromOffset (int nOffset) const
{
	int nRowStart = GetRowStartByOffset (nOffset);
	
	if (m_bKeepTabs)
	{
		int nColumn = 0;
		int nNumChars = nOffset - nRowStart;
		LPCTSTR lpcszRowStart = ((LPCTSTR) m_strBuffer) + nRowStart;
		for (int i = 0; i < nNumChars; i++)
		{
			TCHAR chNext = lpcszRowStart [i];
			if (chNext == _T ('\t'))
			{
				int nRestOfTab = m_nTabSize - nColumn % m_nTabSize;
				nColumn += nRestOfTab; 
			}
			else
			{
				nColumn++;
			}
		}
		return nColumn;
	}
	
	return nOffset - nRowStart;
}
//******************************************************************************
void CBCGPEditCtrl::OnChangeFont (CDC* pDC)
{
	CClientDC dc (this);

	if (pDC == NULL)
	{
		pDC = &dc;
	}

	CFont* pOldFont = SelectFont (pDC);

	TEXTMETRIC tm;
	pDC->GetTextMetrics (&tm);

	m_nLineHeight = tm.tmHeight + m_nLineVertSpacing;
	m_nMaxCharWidth = tm.tmMaxCharWidth;
	m_nAveCharWidth = tm.tmAveCharWidth;

	pDC->SelectObject (pOldFont);

	//------------------
	// Initialize caret:
	//------------------
	::DestroyCaret ();

	CSize sizeCaret = GetCaretSize ();
	::CreateCaret (GetSafeHwnd (), NULL, sizeCaret.cx, sizeCaret.cy);

	UpdateScrollBars ();

	if (!m_bReadOnly)
	{
		SetCaretPos (m_ptCaret);
		ShowCaret ();
	}
}
//******************************************************************************
BOOL CBCGPEditCtrl::OffsetToPoint (int nOffset, CPoint& pt, LPPOINT ptRowColumn, LPINT pnScrollOffset)
{
	ASSERT_VALID (this);

	nOffset = min (max (nOffset, 0), m_strBuffer.GetLength ());

	CClientDC dc (this);
	CFont* pOldFont = SelectFont (&dc);

	int nRow = -1;
	int iRowOffset = 0;
	int nLowBound = 0;

	if (nOffset >= m_nTopVisibleOffset)
	{
		nLowBound = m_nTopVisibleOffset;
	}
	
	for (int i = nOffset - 1; i >= nLowBound; i--)
	{
		if (m_strBuffer [i] == _T('\n'))
		{
			if (nRow == -1)
			{
				nRow = 1;
			}
			else
			{
				nRow ++;
			}
		}
	}

	if (nRow == -1)
	{
		nRow = 0;
	}

	if (nOffset >= m_nTopVisibleOffset)
	{
		nRow += m_nScrollOffsetVert;
	}

	iRowOffset = GetRowStartByOffset (nOffset);
	int nScrolledExtent = m_nScrollOffsetHorz * m_nMaxCharWidth;

	pt.x = m_rectText.left + 
		(nOffset > iRowOffset ? 
			GetStringExtent (&dc, (LPCTSTR) m_strBuffer + iRowOffset, nOffset - iRowOffset).cx : 
			0) - nScrolledExtent - 1;

	pt.x = max (pt.x, m_rectText.left);

	if (pnScrollOffset != NULL)
	{
		*pnScrollOffset = 0;
	
		if (pt.x < m_rectText.left)
		{
			for (int i = nOffset - iRowOffset; 
				i < m_strBuffer.GetLength () && m_strBuffer [i] != _T('\n'); i++)
			{
				(*pnScrollOffset)++;
			}
		}
		else if (pt.x >= m_rectText.right)
		{
		}
	}

	pt.y = m_rectText.top + (nRow - m_nScrollOffsetVert) * m_nLineHeight;

	dc.SelectObject (pOldFont);

	if (ptRowColumn != NULL)
	{
		ptRowColumn->x = GetColumnFromOffset (nOffset);
		ptRowColumn->y = nRow;
	}

	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::Left (BOOL bRedrawOnScroll)
{
	if (m_nCurrOffset == 0)
	{
		OnFailedOperation (g_dwOpLeft);
		return FALSE;
	}

	int nCharCount = 1;
	if (m_bEnableSymSupport && m_nCurrOffset > 0)
	{
		nCharCount += GetSymbolExtraLen (m_strBuffer, m_nCurrOffset, FALSE);
	}

	if (!SetCaret (m_nCurrOffset - nCharCount, TRUE, bRedrawOnScroll))
	{
		OnFailedOperation (g_dwOpLeft);
		return FALSE;
	}

	m_nLastMaxColumn = GetCurColumn ();

	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::Right (BOOL bRedrawOnScroll)
{
	if (m_nCurrOffset == m_strBuffer.GetLength ())
	{
		OnFailedOperation (g_dwOpRight);
		return FALSE;
	}

	int nCharCount = 1;
	if (m_bEnableSymSupport)
	{
		nCharCount += GetSymbolExtraLen (m_strBuffer, m_nCurrOffset);
	}

	if (!SetCaret (m_nCurrOffset + nCharCount, TRUE, bRedrawOnScroll))
	{
		OnFailedOperation (g_dwOpRight);
		return FALSE;
	}

	m_nLastMaxColumn = GetCurColumn ();

	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::Up (BOOL bSetCaretToMaxColumn, BOOL bRedrawOnScroll)
{
	if (m_nCurRow == 0)
	{
		OnFailedOperation (g_dwOpUp);
		return FALSE;
	}

	if (m_strBuffer.IsEmpty ())
	{
		OnFailedOperation (g_dwOpUp);
		return FALSE;
	}


	CPoint point (m_ptCaret.x, m_ptCaret.y);

	int nCurrOldRow = m_nCurRow;

	if (m_ptCaret.y - m_nLineHeight < m_rectText.top )
	{
		ScrollUp (SB_VERT, bRedrawOnScroll);
	}
	else
	{
		point.y -= m_nLineHeight;
	}

	int nOffset = HitTest (point, TRUE);
	if (nOffset >= 0 && nCurrOldRow > 0)
	{
		if (bSetCaretToMaxColumn && SetCaretToLastMaxColumn (nOffset, bRedrawOnScroll))
		{
			return TRUE;
		}
		else
		{
			return SetCaret (nOffset, TRUE, bRedrawOnScroll);
		}
	}

	OnFailedOperation (g_dwOpUp);
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::Down (BOOL bSetCaretToMaxColumn, BOOL bRedrawOnScroll)
{
	CPoint point (m_ptCaret.x, m_ptCaret.y);

	if (m_ptCaret.y + m_nLineHeight >= m_rectText.bottom - m_nLineHeight && m_nCurRow < m_szTotalScroll.cy)
	{
		ScrollDown (SB_VERT, bRedrawOnScroll);
	}
	else
	{
		point.y += m_nLineHeight;
	}
	
	int nOffset = HitTest (point, TRUE);

	if (nOffset >= 0)
	{
		if (bSetCaretToMaxColumn && SetCaretToLastMaxColumn (nOffset, bRedrawOnScroll))
		{
			return TRUE;
		}
		else
		{
			return SetCaret (nOffset, TRUE, bRedrawOnScroll);
		}
	}

	OnFailedOperation (g_dwOpDown);
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::PageUp (BOOL bRedrawOnScroll)
{
	if (m_strBuffer.IsEmpty ())
	{
		OnFailedOperation (g_dwOpPgUp);
		return FALSE;
	}

	SCROLLINFO scrollInfo;
	ZeroMemory(&scrollInfo, sizeof(SCROLLINFO));

    scrollInfo.cbSize = sizeof(SCROLLINFO);
	scrollInfo.fMask = SIF_ALL;

	GetScrollInfo (SB_VERT, &scrollInfo);

	CPoint point (m_ptCaret.x, m_ptCaret.y);

	if ((int) scrollInfo.nPage > m_nCurRow && m_nScrollOffsetVert == 0)
	{
		point.y = 0;
	}
	else
	{
		ScrollPageUp (SB_VERT, bRedrawOnScroll);
	}

	int nOffset = HitTest (point, TRUE);

	if (nOffset >= 0)
	{
		if (SetCaretToLastMaxColumn (nOffset, bRedrawOnScroll))
		{
			return TRUE;
		}
		else
		{
			return SetCaret (nOffset, TRUE, bRedrawOnScroll);
		}
	}

	OnFailedOperation (g_dwOpPgUp);
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::PageDown (BOOL bRedrawOnScroll)
{
	int nOffset = -1;
	if (!ScrollPageDown (SB_VERT, bRedrawOnScroll))
	{
		nOffset = max (m_strBuffer.GetLength (), 0);
	}
	else
	{
		CPoint point (m_ptCaret.x, m_ptCaret.y);
		nOffset = HitTest (point, TRUE);
	}

	nOffset = min (nOffset, m_strBuffer.GetLength ());

	if (nOffset >= 0)
	{
		if (SetCaretToLastMaxColumn (nOffset, bRedrawOnScroll))
		{
			return TRUE;
		}
		else
		{
			return SetCaret (nOffset, TRUE, bRedrawOnScroll);
		}
	}
	OnFailedOperation (g_dwOpPgDn);
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::Home (BOOL bRedrawOnScroll)
{
	if (m_strBuffer.IsEmpty ())
	{
		OnFailedOperation (g_dwOpHome);
		return FALSE;
	}

	BOOL bResult = FALSE;
	ScrollHome (SB_HORZ, bRedrawOnScroll);
	int nRowStart = GetCurrRowStart ();
	int nRowTextStart = GetCurrRowTextStart ();

	if (m_nCurrOffset == nRowTextStart)
	{
		bResult = SetCaret (nRowStart, TRUE, bRedrawOnScroll);
	}
	else
	{
		bResult = SetCaret (nRowTextStart, TRUE, bRedrawOnScroll);
	}

	if (bResult)
	{
		m_nLastMaxColumn = GetCurColumn ();
	}
	return bResult;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::End (BOOL bRedrawOnScroll)
{
	int nOffset = m_nCurrOffset;
	while (nOffset < m_strBuffer.GetLength () &&
		m_strBuffer [nOffset] != _T('\n'))
	{
		nOffset++;
	}

	nOffset = min (nOffset, m_strBuffer.GetLength ());

	SetCaret (nOffset, TRUE, bRedrawOnScroll);
	m_nLastMaxColumn = GetCurColumn ();
	return TRUE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::StartOfText (BOOL bRedraw)
{
	ScrollHome (SB_VERT, FALSE);
	ScrollHome (SB_HORZ, FALSE);

	if (bRedraw)
	{
		RedrawWindow ();
	}

	BOOL bResult = SetCaret (0, TRUE, FALSE);
	m_nLastMaxColumn = 0;

	return bResult;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::EndOfText (BOOL bRedrawOnScroll)
{
	if (m_strBuffer.IsEmpty ())
	{
		return TRUE;
	}

	return SetCaret (m_strBuffer.GetLength (), TRUE, bRedrawOnScroll);
}
//****************************************************************************************
BOOL CBCGPEditCtrl::NextWord (BOOL bRedrawOnScroll)
{
	if (m_nCurrOffset >= m_strBuffer.GetLength () - 1)
	{
		OnFailedOperation (g_dwOpNextWord);
		return FALSE;
	}

	int i = m_nCurrOffset;
	BOOL bCaretInsideWord = m_strWordDelimeters.Find (m_strBuffer [i]) < 0;
	BOOL bCaretInsideDelimiters = m_strWordDelimeters.Find (m_strBuffer [i]) >= 0 && 
								  m_strSpecialDelimiters.Find (m_strBuffer [i]) < 0 &&	
								  (m_strBuffer [i] != _T (' '));
	int nSpecialDelimiterIdx = m_strSpecialDelimiters.Find (m_strBuffer [i]);
	BOOL bCaretInsideSpecialDelimiters = (nSpecialDelimiterIdx >= 0);
	TCHAR chSpecialDelimeterToSkip = 0;

	if (bCaretInsideSpecialDelimiters)
	{
		chSpecialDelimeterToSkip = m_strSpecialDelimiters [nSpecialDelimiterIdx];
	}
	
	if (bCaretInsideWord)
	{
		// Move to end of word:
		while (i < m_strBuffer.GetLength () &&
			m_strWordDelimeters.Find (m_strBuffer [i]) < 0)
		{
			i++;
		}
	}
	else if (bCaretInsideDelimiters)
	{
		// Move to end of delimiters string:
		while (i < m_strBuffer.GetLength () &&
			m_strWordDelimeters.Find (m_strBuffer [i]) >= 0 && 
			m_strSpecialDelimiters.Find (m_strBuffer [i]) < 0 &&
			m_strBuffer [i] != _T (' '))
		{
			i++;
		}
	}
	else if (bCaretInsideSpecialDelimiters)
	{
		// Move to end of special delimiters string:
		while (i < m_strBuffer.GetLength () &&
				m_strBuffer [i] == chSpecialDelimeterToSkip)
		{
			i++;
			if (m_strBuffer [i] == g_chEOL)
			{
				break;
			}
		}
	}

	// Skip spaces:
	while (i < m_strBuffer.GetLength () &&
		m_strBuffer [i] == _T (' '))
	{
		i++;
	}

	if (!SetCaret (i, TRUE, bRedrawOnScroll))
	{
		OnFailedOperation (g_dwOpNextWord);
		return FALSE;
	}

	m_nLastMaxColumn = GetCurColumn ();
	return TRUE;
}
//****************************************************************************************
BOOL CBCGPEditCtrl::PrevWord (BOOL bRedrawOnScroll)
{
	if (m_nCurrOffset <= 0)
	{
		OnFailedOperation (g_dwOpPrevWord);
		return FALSE;
	}

	int i = min (m_nCurrOffset - 1, m_strBuffer.GetLength () - 1);

	BOOL bCursorAtEOL = m_strBuffer [i] == g_chEOL;
	// Skip spaces:
	while (i >= 0 && m_strBuffer [i] == _T (' '))
	{
		i--;
		bCursorAtEOL = FALSE;
	}

	if (i == 0)
	{
		SetCaret (0, TRUE, bRedrawOnScroll);
		return TRUE;
	}


	BOOL bCaretInsideWord = m_strWordDelimeters.Find (m_strBuffer [i]) < 0;
	BOOL bCaretInsideDelimiters = m_strWordDelimeters.Find (m_strBuffer [i]) >= 0 && 
								  m_strSpecialDelimiters.Find (m_strBuffer [i]) < 0 &&	
								  (m_strBuffer [i] != _T (' '));
	int nSpecialDelimiterIdx = m_strSpecialDelimiters.Find (m_strBuffer [i]);
	BOOL bCaretInsideSpecialDelimiters = (nSpecialDelimiterIdx >= 0);
	TCHAR chSpecialDelimeterToSkip = 0;

	if (bCaretInsideSpecialDelimiters)
	{
		chSpecialDelimeterToSkip = m_strSpecialDelimiters [nSpecialDelimiterIdx];
	}

	if (bCaretInsideWord)
	{
		// Move to end of word:
		while (i >= 0 &&
			m_strWordDelimeters.Find (m_strBuffer [i]) < 0 && 
			m_strBuffer [i] != g_chEOL)
		{
			i--;
		}
	}
	else if (bCaretInsideDelimiters)
	{
		// Move to end of delimiters string:
		while (i >= 0 &&
			m_strWordDelimeters.Find (m_strBuffer [i]) >= 0 && 
			m_strSpecialDelimiters.Find (m_strBuffer [i]) < 0 &&
			m_strBuffer [i] != _T (' ') && 
			m_strBuffer [i] != g_chEOL)
		{
			i--;
		}
	}
	else if (bCaretInsideSpecialDelimiters)
	{
		// Move to end of special delimiters string:
		while  (i >= 0 &&
				m_strBuffer [i] == chSpecialDelimeterToSkip &&
				m_strBuffer [i] != _T (' ') && m_strBuffer [i] != g_chEOL)
		{
			i--;
		}
	}
	else
	{
		// Skip spaces:
		while (i >= 0 && m_strBuffer [i] == _T (' '))
		{
			i--;
		}
	}

	if (bCursorAtEOL)
	{
		i--;
	}

	if (!SetCaret (i + 1, TRUE, bRedrawOnScroll))
	{
		OnFailedOperation (g_dwOpPrevWord);
		return FALSE;
	}

	m_nLastMaxColumn = GetCurColumn ();

	return TRUE;
}
//****************************************************************************************
BOOL CBCGPEditCtrl::PrevIndent ()
{
	int nCurColumn = GetCurColumn ();

	if (nCurColumn == 0)
	{
		OnFailedOperation (g_dwOpPrevIndent);
		return FALSE;
	}

	int nOffsetToMove = nCurColumn % m_nIndentSize;
	
	if (nOffsetToMove == 0)
	{
		nOffsetToMove = m_nIndentSize;
	}
	
	for (int i = 0; i < nOffsetToMove; i++)
	{
		Left ();
		if (m_strBuffer [m_nCurrOffset] == _T ('\t'))
		{
			i += m_nTabSize;
		}
	}

	return TRUE;
}
//****************************************************************************************
int CBCGPEditCtrl::GetCurrRowStart () const
{
	for (int i = m_nCurrOffset - 1; i >= 0; i--)
	{
		if (m_strBuffer [i] == _T('\n'))
		{
			break;
		}
	}

	return i + 1;
}
//***************************************************************************************
int CBCGPEditCtrl::GetCurrRowEnd () const
{
	for (int i = m_nCurrOffset; i < m_strBuffer.GetLength (); i++)
	{
		if (m_strBuffer [i] == _T('\n'))
		{
			break;
		}
	}

	return i;
}
//***************************************************************************************
int CBCGPEditCtrl::GetCurrRowTextStart () const
{
	int nStart = GetCurrRowStart ();
	return GetRowTextStart (nStart);
}
//***************************************************************************************
int CBCGPEditCtrl::GetNumOfColumnsInRowByOffset (int nOffset, int& nRowEndOffset) const
{
	nRowEndOffset = GetRowEndByOffset (nOffset);
	return GetColumnFromOffset (nRowEndOffset);
}
//***************************************************************************************
int CBCGPEditCtrl::GetRowStart (int nRow) const
{
	if (nRow == 0)
	{
		return 0;
	}

	for (int nIdx = 0, nRowIdx = 0;  nRowIdx != nRow; nRowIdx++, nIdx++)
	{
		nIdx = m_strBuffer.Find (_T('\n'), nIdx);
		if (nIdx == -1)
		{
			break;
		}
	}

	if (nIdx == -1)
	{
		return -1;
	}

	return nIdx++;
}
//***************************************************************************************
int CBCGPEditCtrl::GetRowTextStart (int nRowOffset) const
{
	if (nRowOffset == -1)
	{
		return 0;
	}
	
	if (nRowOffset >= m_strBuffer.GetLength ())
	{
		return m_strBuffer.GetLength () - 1;
	}

	for (int i = nRowOffset; i < m_strBuffer.GetLength (); i++)
	{
		if (m_strBuffer [i] == g_chEOL)
		{
			return i;
		}

		if (m_strNonSelectableChars.Find (m_strBuffer [i]) == -1)
		{
			break;
		}
	}
	
	return i;
}
//***************************************************************************************
int CBCGPEditCtrl::GetRowStartByOffset (int nOffset) const
{
	BOOL bFound = FALSE;
	if (nOffset > m_strBuffer.GetLength ())
	{
		nOffset = m_strBuffer.GetLength ();
	}

	for (int i = nOffset - 1; i >= 0; i--)
	{
		if (m_strBuffer [i] == _T('\n'))
		{
			bFound = TRUE;
			break;
		}
	}

	if (!bFound)
	{
		return 0;
	}

	return i + 1;
}
//***************************************************************************************
int CBCGPEditCtrl::GetRowEndByOffset (int nOffset) const
{
	int nIdx = m_strBuffer.Find (g_chEOL, nOffset);

	if (nIdx == -1)
	{
		nIdx = m_strBuffer.GetLength () - 1;
	}

	return nIdx;
}
//**************************************************************************************
int CBCGPEditCtrl::GetNumOfCharsInText (const CString& strText, TCHAR ch)
{
	int nNumChars = 0;
	for (int iIdx = 0; iIdx != -1; iIdx++)
	{
		iIdx = strText.Find (ch, iIdx);
		if (iIdx != -1)
		{
			nNumChars++;
		}
		else
		{
			break;
		}
	}

	return nNumChars;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::InsertChar (TCHAR nChar, BOOL bRedraw)
{
	ASSERT_VALID (this);
	ASSERT (m_nCurrOffset <= m_strBuffer.GetLength ());

	CString strChar (nChar);

	if (!OnBeforeTextInserted (strChar, m_nCurrOffset))
	{
		return FALSE;
	}

	if (nChar == _T('\n'))
	{
		if (GetOverrideMode ())
		{
			Home ();
			return Down ();
		}
		return InsertNewLine (TRUE);
	}

	int nCurrOffset = m_nCurrOffset;

	int nRowEndOffset = GetRowEndByOffset (m_nCurrOffset);
	int nRowLen = m_bKeepTabs ? GetNumOfColumnsInRowByOffset (m_nCurrOffset, nRowEndOffset) :
					nRowEndOffset - GetRowStartByOffset (m_nCurrOffset);

	if (nRowLen >= m_nMaxScrollWidth - 1)
	{
		HideCaret ();
		InsertNewLine (TRUE);
		if (nRowEndOffset != nCurrOffset)
		{
			SetCaret (nCurrOffset);
		}
		ShowCaret ();
	}
	BOOL bForceInsertMode = !GetOverrideMode () || 
							((m_nCurrOffset == m_strBuffer.GetLength () ||
							  m_strBuffer.GetAt (m_nCurrOffset) == g_chEOL)) && GetOverrideMode ();

	AddUndoAction (nChar, g_dwUATInsertData, m_nCurrOffset, bForceInsertMode);

	BOOL bAtColorBlock = IsOffsetAtColorBlock (m_nCurrOffset);

	if (GetOverrideMode () && !m_strBuffer.IsEmpty () && 
		m_nCurrOffset < m_strBuffer.GetLength () &&
		m_strBuffer.GetAt (m_nCurrOffset) != g_chEOL)
	{
		m_strBuffer.SetAt (m_nCurrOffset, nChar);
	}
	else
	{
		m_strBuffer.Insert (m_nCurrOffset, nChar);
	}	

	OnAfterTextChanged (m_nCurrOffset, CString (nChar), TRUE);

	UpdateScrollBars ();

	if (bRedraw)
	{
		if (IsOffsetAtColorBlock (m_nCurrOffset) || bAtColorBlock)
		{
			RedrawWindow (m_rectText);
		}
		else
		{
			RedrawRestOfLine (m_nCurrOffset);
		}
	}

	Right ();

//	IntelliSense char update ...
	CString strIntelliSense;

	if (!m_bIntelliSenseUpdate &&
		IntelliSenseCharUpdate(m_strBuffer, m_nCurrOffset, nChar, strIntelliSense))
	{
		Sleep(100);
		m_nCurrOffset--;
		OnDelete(TRUE,TRUE);

		if (strIntelliSense.IsEmpty())
		{
			ASSERT(FALSE);
		}
		else
		{
			m_bIntelliSenseUpdate = TRUE;

			try
			{
				InsertText(strIntelliSense.GetBuffer(0), m_nCurrOffset, bRedraw);
			}
			catch (...)
			{
				ASSERT(FALSE);
			}

			m_bIntelliSenseUpdate = FALSE;
		}
	}
	
	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::InsertText (LPCTSTR lpszText, int nInsertFrom, BOOL bRedraw, 
								BOOL bSuppressUndo, BOOL bUpdateLineData, 
								BOOL bForceNextUndo, BOOL bAlwaysAtEnd)
{
	ASSERT_VALID (this);
	ASSERT (lpszText != NULL);

	if (nInsertFrom >= 0)
	{
		nInsertFrom = min (nInsertFrom, m_strBuffer.GetLength ());
	}
	else
	{
		nInsertFrom = m_nCurrOffset;
	}

	CString strText = lpszText;
	
	if (ProcessTextBeforeInsert (strText))
	{
		int nInsertedTextLength = strText.GetLength ();
		int nCurrRowStartOffset = GetRowStartByOffset (m_nCurrOffset);

		if (m_nCurrOffset - nCurrRowStartOffset + nInsertedTextLength >= m_nMaxScrollWidth && 
			m_nCurrOffset != 0 && !m_strBuffer.IsEmpty () && 
			m_nCurrOffset != nCurrRowStartOffset &&  
			m_nCurrOffset < m_strBuffer.GetLength () - 1  && 
			(m_strBuffer [m_nCurrOffset + 1] != g_chEOL || 
			 m_strBuffer [m_nCurrOffset] != g_chEOL) &&
			 !bAlwaysAtEnd)
		{
			OnFailedOperation (0x1000000 | 0x10000000);
			strText.Insert (0, g_chEOL);
		}

		if (m_nCurrOffset >= m_strBuffer.GetLength () - 1 && 
			m_nCurrOffset != nCurrRowStartOffset && !bAlwaysAtEnd)
		{
			OnFailedOperation (0x1000000 | 0x10000000);
			strText.Insert (0, g_chEOL);
		}			

		if (!OnBeforeTextInserted (strText, nInsertFrom))
		{
			return FALSE;
		}

		OnInsertTextToBuffer (nInsertFrom, strText, bUpdateLineData);

		m_strBuffer.Insert (nInsertFrom, strText);
		
		int nRowEndOffset = GetRowEndByOffset (m_nCurrOffset + nInsertedTextLength);
		int nRowStartOffset = GetRowStartByOffset (m_nCurrOffset + nInsertedTextLength);
		int nRowLen = nRowEndOffset - nRowStartOffset;

		if (bSuppressUndo)
		{
			EmptyUndoList ();
		}
		else
		{
			AddUndoAction (strText, g_dwUATInsertData, nInsertFrom, TRUE, 
							bForceNextUndo || nRowLen >= m_nMaxScrollWidth);
		}
		
		if (nRowLen >= m_nMaxScrollWidth)
		{
			InsertNewLine (FALSE, bForceNextUndo, 
								nRowStartOffset + m_nMaxScrollWidth - 1, FALSE);
		}

		UpdateScrollBars ();

		SetCaret (nInsertFrom + strText.GetLength ());

		if (bRedraw)
		{
			RedrawWindow ();
		}

		OnAfterTextChanged (nInsertFrom, strText, TRUE);

		return TRUE;
	}

	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::InsertTextAsBlock (LPCTSTR lpszText, int nInsertFrom, 
									   BOOL bRedraw, BOOL bSuppressUndo, 
									   BOOL bUpdateLineData, BOOL bForceNextUndo)
{
	if (nInsertFrom >= 0)
	{
		nInsertFrom = min (nInsertFrom, m_strBuffer.GetLength ());
	}
	else
	{
		nInsertFrom = m_nCurrOffset;
	}
	
	int nRowStartOffset = GetRowStartByOffset (nInsertFrom);
	int nRowEndOffset   = GetRowEndByOffset (nInsertFrom);

	if (nRowEndOffset - nRowStartOffset < 2)
	{
		return InsertText (lpszText, nInsertFrom, bRedraw, bSuppressUndo, bUpdateLineData, bForceNextUndo);
	}

	CString strText = lpszText;

	if (strText.IsEmpty ())
	{
		return TRUE;
	}

	int nStartColumn = GetColumnFromOffset (nInsertFrom);
	bool bAddedLines = false;
	for (int nTextRowOffset = 0; nTextRowOffset < strText.GetLength ();)
	{
		int nNextRowOffset = strText.Find (g_chEOL, nTextRowOffset);
		if (nNextRowOffset == -1)
		{
			nNextRowOffset = strText.GetLength ();
		}

		CString strSubRow = strText.Mid (nTextRowOffset, nNextRowOffset - nTextRowOffset);

		if (nRowEndOffset >= m_strBuffer.GetLength ())
		{
			m_strBuffer += strSubRow;
			m_nTotalLines++;
			bAddedLines = true;
			continue;
		}

		strSubRow.Remove (g_chEOL);

		int nInsertOffset = GetOffsetOfColumnInRow (nStartColumn, nRowEndOffset);
		nRowEndOffset += strSubRow.GetLength () + 1;

		nInsertOffset = min (nRowEndOffset - 1, nInsertOffset);
		if (OnBeforeTextInserted (strSubRow, nInsertOffset))
		{
			m_strBuffer.Insert (nInsertOffset, strSubRow);
			OnAfterTextChanged (nInsertOffset, strSubRow, TRUE);
		}

		nRowEndOffset = GetRowEndByOffset (nRowEndOffset);

		nTextRowOffset = nNextRowOffset + 1;
	}

	if (bAddedLines)
	{
		UpdateScrollBars ();
	}
	
	if (bRedraw)
	{
		RedrawWindow ();
	}

	return TRUE;
}								
//**************************************************************************************
BOOL CBCGPEditCtrl::ProcessTextBeforeInsert (CString& strText)
{
	strText.Remove (_T('\r'));

	if (!m_bKeepTabs)
	{
		CString strTab (_T (' '), m_nTabSize);
		strText.Replace (_T("\t"), strTab);
	}

	// truncate strings that too long
	for (int i = 0, nCount = 0; i < strText.GetLength (); i++)
	{
		strText [i] == g_chEOL ? nCount = 0 : nCount++;

		if (nCount == m_nMaxScrollWidth)
		{
			strText.Insert (i, g_chEOL); 
			nCount = 0;
		}
	}

	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::InsertNewLine (BOOL bRedraw, BOOL bForceNextUndo, int  nOffset, 
								   BOOL bSetCaret)
{
	if (nOffset == -1)
	{
		nOffset = m_nCurrOffset;
	}

	int nRowNum = RowFromOffset (nOffset);
	int nRowStartOffset = GetRowStartByOffset (nOffset);
	BOOL bLineStart = m_nCurrOffset == nRowStartOffset;

	if (!OnInsertNewLine (bForceNextUndo, nOffset, bSetCaret))
	{
		return FALSE;
	}
		
	if (bLineStart)
	{
		// the new line is inserted at the beginning of the current line
		// the current line with all markesr should be moved down
		UpdateLineRelatedData (nRowNum, 1);
	}
	else
	{
		// the new line is inserted at the middle of the current line
		// the rest of the current line with all markers starting
		// from the next line should be moved down
		UpdateLineRelatedData (nRowNum + 1, 1);
	}
	
	UpdateScrollBars ();

	m_bIsModified = TRUE;

	if (bRedraw)
	{
		RedrawWindow ();
	}

	return TRUE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::OnInsertNewLine (BOOL bForceNextUndo, int nOffset, 
									 BOOL bSetCaret)
{
	if (nOffset == -1)
	{
		nOffset = m_nCurrOffset;
	}
	
	int nCurrRowStart = GetRowStartByOffset (nOffset);
	int nCurrRowTextStart = GetRowTextStart (nCurrRowStart);

	CString strToInsert (g_chEOL);

	if (m_bDefaultIndent && nCurrRowTextStart > nCurrRowStart)
	{
		int nCount = min (nOffset - nCurrRowStart, nCurrRowTextStart - nCurrRowStart);
		CString strRowStart = m_strBuffer.Mid (nCurrRowStart, nCount);
		strToInsert += strRowStart;  
	}

	if (!OnBeforeTextInserted (strToInsert, nOffset))
	{
		return FALSE;
	}
	
	AddUndoAction (strToInsert, g_dwUATInsertData, nOffset, FALSE, bForceNextUndo);
	m_strBuffer.Insert (nOffset, strToInsert);

	OnAfterTextChanged (nOffset, strToInsert, TRUE);

	if (bSetCaret)
	{
		if (m_bDefaultIndent) 
		{
			SetCaret (nOffset + strToInsert.GetLength ());
		}
		else
		{
			Down ();
			Home ();
		}
	}
	
	m_nTotalLines++;

	return TRUE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::InsertTab (BOOL bRedraw)
{
	CString strTab;
	int nInsertOffset = m_nCurrOffset;

	int nRowEndOffset = GetRowEndByOffset (m_nCurrOffset);
	int nRowLen = m_bKeepTabs ? GetNumOfColumnsInRowByOffset (m_nCurrOffset, nRowEndOffset) :
					nRowEndOffset - GetRowStartByOffset (m_nCurrOffset);

	if (nRowLen + m_nIndentSize > m_nMaxScrollWidth - 1)
	{
		OnFailedOperation (g_dwOpInsTab | g_dwOpReasonLength);
		return FALSE;
	}

	if (GetOverrideMode ())
	{
		int nColumn = GetColumnFromOffset (m_nCurrOffset);
		nColumn += m_nIndentSize;
		int nNewOffset = GetOffsetOfColumnInRow (nColumn, m_nCurrOffset, FALSE);
		SetCaret (nNewOffset);
		return TRUE;
	}
		
	if (m_bKeepTabs)
	{
		BuildTabString (strTab);

		if (!OnBeforeTextInserted (strTab, m_nCurrOffset))
		{
			return FALSE;
		}
		m_strBuffer.Insert (m_nCurrOffset, strTab);
		OnAfterTextChanged (m_nCurrOffset, strTab, TRUE);
		UpdateScrollBars ();
		for (int i = 0; i < strTab.GetLength (); i++)
		{
			Right ();
		}
	}
	else
	{
		CString strToInsert (_T(' '), m_nIndentSize);
		if (!OnBeforeTextInserted (strToInsert, m_nCurrOffset))
		{
			return FALSE;
		}
		for (int i = 0; i < m_nIndentSize; i++)
		{
			strTab += _T(' ');
			m_strBuffer.Insert (m_nCurrOffset, _T(' '));
			UpdateScrollBars ();
			Right ();
		}

		OnAfterTextChanged (m_nCurrOffset, strToInsert, TRUE);
	}
	

	AddUndoAction (strTab, g_dwUATInsertData, nInsertOffset, FALSE);

	if (bRedraw)
	{
		RedrawRestOfLine (m_nCurrOffset);
	}

	return TRUE;
}
//**************************************************************************************
void CBCGPEditCtrl::RedrawText (int nStartRow, int nEndRow/* = -1*/)
{
	ASSERT_VALID (this);

	CRect rectArea;
	GetClientRect (rectArea);

	if (nEndRow != -1)
	{
		rectArea.top = (nStartRow - m_nScrollOffsetVert) * m_nLineHeight;

		switch (nEndRow)
		{
		case 0:	// one line
			rectArea.bottom = rectArea.top + m_nLineHeight;
			break;

		case -1: // rest of text
			break;

		default:
			ASSERT (nEndRow >= nStartRow);

			rectArea.bottom = min (rectArea.bottom,
				(rectArea.top + m_nLineHeight) * (nEndRow - nStartRow + 1));
		}
	}

	RedrawWindow (rectArea);
}
//**************************************************************************************
void CBCGPEditCtrl::RedrawTextOffsets (int nStartOffset, int nEndOffset)
{
	int nStart = min (nStartOffset, nEndOffset);
	int nEnd = max (nStartOffset, nEndOffset);
	
	CPoint ptStart, ptEnd;
	OffsetToPoint (nStart, ptStart);
	OffsetToPoint (nEnd, ptEnd);

	ptStart.x = m_rectText.left;
	ptEnd.y += m_nLineHeight;
	ptEnd.x = m_rectText.right;
	CRect rect (ptStart, ptEnd);
	
	RedrawWindow (rect);
}
//***************************************************************************************
void CBCGPEditCtrl::OnFillBackground (CDC* pDC)
{
	ASSERT_VALID (pDC);

	if (m_clrBack == -1)
	{
		::FillRect (pDC->GetSafeHdc (), &m_rectText, ::GetSysColorBrush (COLOR_WINDOW));
	}
	else
	{
		CBrush br (m_clrBack);
		pDC->FillRect (m_rectText, &br);
	}
}
//***************************************************************************************
void CBCGPEditCtrl::OnDrawSideBar (CDC* pDC, CRect rect)
{
	ASSERT_VALID (pDC);

	pDC->FillSolidRect (rect, ::GetSysColor (COLOR_3DLIGHT));

	if (m_lstMarkers.IsEmpty ())
	{
		return;
	}

	// determine start and end offsets of the visible part of buffer
	CPoint ptTopLeft (m_rectText.left, m_rectText.top);
	CPoint ptBottomRight (m_rectText.right - 1, m_rectText.bottom - 1);

	int nStartOffset = /*GetRowStart (m_nScrollOffsetVert);*/HitTest (ptTopLeft);
	int nEndOffset = HitTest (ptBottomRight);

	if (nStartOffset == -1)
	{
		nStartOffset = 0;
	}

	if (nEndOffset == -1)
	{
		nEndOffset = m_strBuffer.GetLength () - 1;
	}

	nEndOffset = min (nEndOffset, m_strBuffer.GetLength ());

	CPoint pt;
	CPoint ptRowColumnTop;
	CPoint ptRowColumnBottom;
	OffsetToPoint (nStartOffset, pt, &ptRowColumnTop);
	OffsetToPoint (nEndOffset, pt, &ptRowColumnBottom);

	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker,
															m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->m_nLine >= ptRowColumnTop.y && 
			pMarkerNext->m_nLine <= ptRowColumnBottom.y + 1)
		{
			CRect rectMarker (rect);
			rectMarker.top = (pMarkerNext->m_nLine - m_nScrollOffsetVert) * m_nLineHeight;
			rectMarker.bottom = rectMarker.top + m_nLineHeight;
			OnDrawMarker (pDC, rectMarker, pMarkerNext);
		}
	}

}
//**************************************************************************************
BOOL CBCGPEditCtrl::OnDelete (BOOL bRedraw, BOOL bForceNextUndo)
{
	if (DeleteSelectedText ())
	{
		m_nLastMaxColumn = GetCurColumn ();
		return TRUE;
	}

	int nBufferLen = m_strBuffer.GetLength ();
	BOOL bDelNewLine = (m_nCurrOffset >= 0 && m_nCurrOffset < nBufferLen - 1
						&& m_strBuffer [m_nCurrOffset] == _T('\n'));

	if (bDelNewLine)
	{
		int nRowEndOffset;
		int nCurLineLen = GetNumOfColumnsInRowByOffset (m_nCurrOffset, nRowEndOffset);
		int nNextLineLen = GetNumOfColumnsInRowByOffset (nRowEndOffset + 1, nRowEndOffset);

		if (nCurLineLen + nNextLineLen > m_nMaxScrollWidth - 2)
		{
			OnFailedOperation (g_dwOpLineConcat | g_dwOpReasonLength);
			return FALSE;
		}
	}
	
	CString strDeletedText = m_strBuffer.Mid (m_nCurrOffset, 1);

	if (!OnBeforeTextDeleted (m_nCurrOffset, strDeletedText))
	{
		return FALSE;
	}

	if (strDeletedText.IsEmpty ())
	{
		OnFailedOperation (g_dwOpDelete | g_dwOpReasonBufEmpty);
		return FALSE;
	}
	AddUndoAction (strDeletedText, g_dwUATDeleteData, m_nCurrOffset, FALSE, bForceNextUndo);

	BOOL bWasAtColorBlock = IsOffsetAtColorBlock (m_nCurrOffset);

	if (bDelNewLine)
	{
		OnDeleteTextFromBuffer (m_nCurrOffset, m_nCurrOffset + 1, strDeletedText);
	}

	m_strBuffer.Delete (m_nCurrOffset, 1);
	OnAfterTextChanged (m_nCurrOffset, strDeletedText, FALSE);

	if (bDelNewLine)
	{
		UpdateScrollBars ();
	}		

	if (bRedraw)
	{
		if (IsOffsetAtColorBlock (m_nCurrOffset) || bWasAtColorBlock)
		{
			RedrawWindow ();
		}
		else if (bDelNewLine)
		{
			RedrawRestOfText (m_nCurrOffset);
		}
		else
		{
			RedrawRestOfLine (m_nCurrOffset);
		}
	}

	m_nLastMaxColumn = GetCurColumn ();

	return TRUE;
}
//**************************************************************************************
void CBCGPEditCtrl::RedrawRestOfLine (int nOffset)
{
	if (nOffset >= 0)
	{
		CPoint pt;
		if (!OffsetToPoint (nOffset, pt))
		{
			ASSERT (FALSE);
		}
		else
		{
			CRect rect (m_rectText.left, pt.y, m_rectText.right, pt.y + m_nLineHeight);
			rect.InflateRect (1, 1);

			InvalidateRect (rect);
			UpdateWindow ();
		}
	}
}
//**************************************************************************************
void CBCGPEditCtrl::RedrawRestOfText (int nOffset)
{
	if (nOffset >= 0)
	{
		CPoint pt;
		if (!OffsetToPoint (nOffset, pt))
		{
			ASSERT (FALSE);
		}
		else
		{
			CRect rect = m_rectText;
			rect.left = 0;
			rect.top = pt.y - m_nScrollOffsetVert * m_nLineHeight;

			InvalidateRect (rect);
			UpdateWindow ();
		}
	}
}
//**************************************************************************************
void CBCGPEditCtrl::OnDrawText (CDC* pDC)
{
	ASSERT_VALID (pDC);

	BOOL bIsPrint = pDC->IsPrinting ();

	CRect rectText;

	if (bIsPrint)
	{
		rectText = m_rectText;
	}
	else
	{
		CRect rectClip; 
		GetClipBox (pDC->m_hDC, rectClip);

		rectText.IntersectRect (rectClip, m_rectText);
		rectText.left = m_rectText.left;
		rectText.right = m_rectText.right;
	}

	if (rectText.IsRectEmpty ())
	{
		return;
	}

	rectText.InflateRect (1, 1);

	CRect rectRow = rectText;

	int nVertOffset = m_nScrollOffsetVert * m_nLineHeight;

	rectRow.OffsetRect (0, -nVertOffset);

	rectRow.DeflateRect (0, 1);

	rectRow.bottom = rectRow.top + m_nLineHeight;

	CString strTabs (_T (' '), m_nTabSize);
	m_nTabLogicalSize = pDC->GetTextExtent (strTabs).cx;

//////////////////////////////////////////////////////////////////////////

	// determine start and end offsets of the visible part of buffer

	CPoint ptTopLeft (rectText.left, rectText.top);
	CPoint ptBottomRight (rectText.right - 1, rectText.bottom - 1);

	int nStartOffset = m_nTopVisibleOffset; 
	int nEndOffset = HitTest (ptBottomRight, FALSE, TRUE);

	if (nStartOffset == -1)
	{
		nStartOffset = 0;
	}

	if (nEndOffset == -1)
	{
		nEndOffset = m_strBuffer.GetLength () - 1;
	}

	nEndOffset = min (nEndOffset, m_strBuffer.GetLength () - 1);

	nStartOffset = GetRowStartByOffset (nStartOffset);
	nEndOffset = GetRowEndByOffset (nEndOffset);

	rectRow.left = rectText.left - m_nScrollOffsetHorz * m_nMaxCharWidth + 1;

	rectRow.top = 0;
	rectRow.bottom = rectRow.top + m_nLineHeight;
	int nTabOrigin = rectRow.left;

	CString strWord; 

	int nOldBkMode = pDC->SetBkMode (OPAQUE);

	COLORREF clrDefaultText = GetDefaultTextColor ();
	COLORREF clrDefaultBack = GetDefaultBackColor ();

	COLORREF clrNextWordFore	= clrDefaultText;
	COLORREF clrNextWordBk		= clrDefaultBack;

	CString strOutString;
	COLORREF clrOutText = clrDefaultText;
	COLORREF clrOutBack = clrDefaultBack;

	COLORREF clrBkSel = GetFocus () == this ? m_clrBackSelActive : 
											  m_clrBackSelInActive;
	COLORREF clrTextSel = GetFocus () == this ? m_clrTextSelActive : 
												m_clrTextSelInActive;

	CRect rectSel;
	
	BCGP_EDIT_COLOR_BLOCK	clrBlock; 
	BOOL bIsOpenBlock = FindOpenBlock (m_nTopVisibleOffset, &clrBlock);

	int nCloseBlockOffset = -1;
	if (bIsOpenBlock)
	{
		nCloseBlockOffset = FindCloseBlock (m_nTopVisibleOffset, &clrBlock);
	}

	COLORREF clrBlockFore = (clrBlock.m_clrForeground == -1) ? GetDefaultTextColor () : clrBlock.m_clrForeground;
	COLORREF clrBlockBack = (clrBlock.m_clrBackground == -1) ? GetDefaultBackColor () : clrBlock.m_clrBackground; 

	CList <BCGP_EDIT_COLOR_AREA, BCGP_EDIT_COLOR_AREA&> colorAreas;

	if (bIsOpenBlock) 
	{
		if (nCloseBlockOffset < nEndOffset)
		{
			BuildColorAreas (colorAreas, nCloseBlockOffset, nEndOffset, NULL, FALSE);
		}
	}
	else
	{
		BuildColorAreas (colorAreas, nStartOffset, nEndOffset, NULL, FALSE);
	}
	
	if (!bIsPrint && m_bBlockSelectionMode && m_iStartSel != -1)
	{
		CPoint ptStart;
		CPoint ptEnd;

		ptStart.x = m_ptStartBlockSel.x - m_nScrollOffsetHorz * m_nMaxCharWidth;
		ptStart.y = m_ptStartBlockSel.y - m_nScrollOffsetVert * m_nLineHeight;

		ptEnd.x = m_ptEndBlockSel.x - m_nScrollOffsetHorz * m_nMaxCharWidth;
		ptEnd.y = m_ptEndBlockSel.y - m_nScrollOffsetVert * m_nLineHeight;

		if (ptEnd.y <= ptStart.y)
		{
			int n = ptEnd.y;
			ptEnd.y = ptStart.y;
			ptStart.y = n;

			ptEnd.y += m_nLineHeight; 
			ptStart.y -= m_nLineHeight;
		}

		rectSel.SetRect (ptStart, ptEnd);
		rectSel.NormalizeRect ();

		pDC->FillSolidRect (rectSel, clrBkSel);
	}

	CPoint ptCharOffset (rectRow.TopLeft ());

	TCHAR* lpszOutBuffer = (TCHAR*) alloca (sizeof (TCHAR) * m_nMaxScrollWidth);
	memset (lpszOutBuffer, 0, sizeof (TCHAR) * m_nMaxScrollWidth);
	int iIdx = 0;

	int nNextDelimiter = -1;
	int nCurrRow = m_nScrollOffsetVert;
	int nCurrColumn = 0;
	BOOL bColoredLine = FALSE;

	for (int i = nStartOffset; i <= nEndOffset; i++)
	{
		COLORREF clrForeground = clrDefaultText;
		COLORREF clrBackground = clrDefaultBack;
		bColoredLine = FALSE;
		
		TCHAR chNext = m_strBuffer [i];

		BOOL bColorFound = FALSE;
		
		BCGP_EDIT_SYM_DEF symDef;
		int nSymExtraLen = 0;
		BOOL bDrawSymbol = m_bEnableSymSupport ? 
								LookUpSymbol (m_strBuffer, i, nEndOffset + 1, chNext, symDef, nSymExtraLen) : FALSE;

		int nCharWidth = 0;
		if (bDrawSymbol)
		{
			IMAGEINFO imgInfo;
			m_pSymbolsImgList->GetImageInfo (symDef.m_nIndex, &imgInfo);
			nCharWidth = imgInfo.rcImage.right - imgInfo.rcImage.left;
		} 
		else if (!m_mapCharWidth.Lookup (chNext, nCharWidth))
		{
			nCharWidth = GetStringExtent (pDC, CString (chNext), 1).cx;
			if (nCharWidth == 0 && chNext == g_chEOL)
			{
				nCharWidth = m_nMaxCharWidth;
			}
			
			m_mapCharWidth.SetAt (chNext, nCharWidth);
		}

		if (chNext == _T ('\t'))
		{
			int nRestOfTab = m_nTabSize - nCurrColumn % m_nTabSize;
			
			nCurrColumn += nRestOfTab; 
			nCharWidth = nRestOfTab * m_nAveCharWidth;
		}
		else
		{
			nCurrColumn++;
		}
		
		// selection has the highest priority, check the character for selection first
		if (!bIsPrint && IsInsideSelection (i, ptCharOffset, rectSel, nCharWidth))
		{
			clrForeground = clrTextSel; 
			clrBackground = clrBkSel; 
			bColorFound = TRUE;
		}
		else if (!bIsPrint && IsColoredLine (nCurrRow, clrForeground, clrBackground))
		{
			bColorFound = TRUE;
			bColoredLine = TRUE;
		}
		else if (bIsOpenBlock && i < nCloseBlockOffset)
		{
			clrForeground = clrBlockFore;
			clrBackground = clrBlockBack;
			bColorFound = TRUE;
		}
		else
		{
			// check all color areas (comments, strings and so on)
			for (POSITION pos = colorAreas.GetHeadPosition (); pos != NULL;)
			{
				BCGP_EDIT_COLOR_AREA colorArea = colorAreas.GetNext (pos);
				if (i >= colorArea.m_nStart && i <= colorArea.m_nEnd)
				{
					clrForeground = colorArea.m_clrForeground; 
					clrBackground = colorArea.m_clrBackground;

					if (clrForeground == -1)
					{
						clrForeground = GetDefaultTextColor ();
					}

					if (clrBackground == -1)
					{
						clrBackground = GetDefaultBackColor ();
					}

					bColorFound = TRUE;
					break;
				}
			}
		}

		if (chNext != g_chEOL && !bDrawSymbol)
		{
			if (nNextDelimiter == -1) 
			{
				CString strNextWord;		
				
				for (int iIdx = i; iIdx <= nEndOffset; iIdx++)
				{
					TCHAR ch = m_strBuffer [iIdx];
					if (m_strWordDelimeters.Find (ch) != -1)
					{
						nNextDelimiter = iIdx;
						break;
					}
				}

				if (nNextDelimiter == -1)
				{
					nNextDelimiter = nEndOffset + 1;
				}

				if (nNextDelimiter != -1)
				{
					strNextWord = 
						m_strBuffer.Mid (i, nNextDelimiter - i);
				}
						
				if (!bColorFound && 
					!OnGetWordColor (strNextWord, clrNextWordFore, clrNextWordBk, i))
				{
					clrNextWordFore = clrDefaultText;
					clrNextWordBk = clrDefaultBack;
				}
			}

			if (i >= nNextDelimiter - 1)
			{
				nNextDelimiter = -1;
			}

			if (!bColorFound)
			{
				clrForeground = clrNextWordFore;
				clrBackground = clrNextWordBk;
			}

			ptCharOffset.x += nCharWidth;
		}
		else if (chNext == g_chEOL)
		{
			pDC->SetTextColor (clrOutText);
			pDC->SetBkColor   (clrOutBack);

			if (iIdx != 0)
			{
				if (m_bKeepTabs)
				{
					rectRow.left += DrawString (pDC, lpszOutBuffer, rectRow, nTabOrigin, clrDefaultBack);
				}
				else
				{
					DrawString (pDC, lpszOutBuffer, rectRow, nTabOrigin, clrDefaultBack);
					rectRow.left += pDC->GetTextExtent (lpszOutBuffer).cx;
				}
				
				iIdx = 0;
				memset (lpszOutBuffer, 0, sizeof (TCHAR) * m_nMaxScrollWidth);
				rectRow.TopLeft ();
			}

			if (i < max (m_iStartSel, m_iEndSel) && !bColoredLine)
			{
				if (clrForeground != clrOutText || clrBackground != clrOutBack)
				{
					pDC->SetTextColor (clrForeground);
					pDC->SetBkColor   (clrBackground);
				}
				
				DrawString (pDC, _T (" "), rectRow, nTabOrigin, clrDefaultBack);
			}

			clrOutText = clrForeground;
			clrOutBack = clrBackground;
					
			if (!bIsPrint)
			{
				DrawColorLine (pDC, nCurrRow, rectRow);
			}

			rectRow.OffsetRect (0, m_nLineHeight);
			rectRow.left = rectText.left - m_nScrollOffsetHorz * m_nMaxCharWidth + 1;
			ptCharOffset = rectRow.TopLeft ();

			nCurrRow++;		
			nCurrColumn = 0;

			continue;
		} 
		
		if (!bColorFound && !bDrawSymbol)
		{
			OnGetCharColor (chNext, i, clrForeground, clrBackground);
		}

		if (clrForeground != clrOutText || clrBackground != clrOutBack || bDrawSymbol)
		{
			pDC->SetTextColor (clrOutText);
			pDC->SetBkColor   (clrOutBack);

			clrOutText = clrForeground;
			clrOutBack = clrBackground;

			if (iIdx != 0)
			{
				if (m_bKeepTabs)
				{
					rectRow.left += DrawString (pDC, lpszOutBuffer, rectRow, nTabOrigin, clrDefaultBack);
				}
				else
				{
					DrawString (pDC, lpszOutBuffer, rectRow, nTabOrigin, clrDefaultBack);
					rectRow.left += pDC->GetTextExtent (lpszOutBuffer).cx;
				}
				
				iIdx = 0;
				memset (lpszOutBuffer, 0, sizeof (TCHAR) * m_nMaxScrollWidth);
				rectRow.TopLeft ();
			}
		}

		if (bDrawSymbol)
		{
			if (clrBackground != clrDefaultBack)
			{
				CRect rect = rectRow;
				rect.right = rect.left + nCharWidth;

				pDC->FillSolidRect (rect, clrBackground);
			}
			if (!OnDrawSymbol (pDC, rectRow, symDef))
			{
				m_pSymbolsImgList->Draw (pDC, symDef.m_nIndex, rectRow.TopLeft (), ILD_TRANSPARENT);
			}

			rectRow.left += nCharWidth;
			i += nSymExtraLen;
			
			continue;
		}

		lpszOutBuffer [iIdx] = chNext;
		if (iIdx + 1 > m_nMaxScrollWidth - 1)
		{
			lpszOutBuffer [iIdx + 1] = _T ('\0');
			continue;
		}
		lpszOutBuffer [iIdx + 1] = _T ('\0');
		iIdx++;
	}

	if (iIdx != 0)
	{
		pDC->SetTextColor (clrOutText);
		pDC->SetBkColor   (clrOutBack);

		DrawString (pDC, lpszOutBuffer, rectRow, nTabOrigin, clrDefaultBack);

		if (i < m_strBuffer.GetLength () && m_strBuffer.GetAt (i) == g_chEOL && 
			i >= min (m_iStartSel, m_iEndSel) && i <= max (m_iStartSel, m_iEndSel))
		{
			pDC->SetTextColor (clrTextSel);
			pDC->SetBkColor   (clrBkSel);

			rectRow.left += pDC->GetTextExtent (lpszOutBuffer).cx;
			DrawString (pDC, _T (" "), rectRow, nTabOrigin, clrDefaultBack);
		}

		if (!bIsPrint)
		{
			DrawColorLine (pDC, nCurrRow, rectRow);
		}
	}

	pDC->SetBkMode (nOldBkMode);
}
//**************************************************************************************
void CBCGPEditCtrl::DrawColorLine (CDC* pDC, int nRow, CRect rectRow)
{
	CObList lstMarkers;
	GetMarkersInRange (nRow, lstMarkers, g_dwBCGPEdit_LineColorMarker);

	if (!lstMarkers.IsEmpty ())
	{
		for (POSITION pos = lstMarkers.GetHeadPosition (); pos != NULL;)
		{
			CBCGPLineColorMarker* pLineColorMarker = 
						DYNAMIC_DOWNCAST (CBCGPLineColorMarker, lstMarkers.GetNext (pos));
			ASSERT_VALID (pLineColorMarker);

			if (pLineColorMarker != NULL && 
				pLineColorMarker->m_bFullLine)
			{
				COLORREF clrBack = pLineColorMarker->m_clrBackground;
				if (clrBack == -1)
				{
					clrBack = GetDefaultBackColor ();
				}
				pDC->FillSolidRect (rectRow, clrBack);
			}
		}
	}
}
//**************************************************************************************
int CBCGPEditCtrl::DrawString (CDC* pDC, LPCTSTR str, CRect rect, int nOrigin, 
							   COLORREF clrBack)
{
	COLORREF crBk = pDC->GetBkColor ();
	
	if (crBk != clrBack && m_nLineVertSpacing > 0)
	{
		CSize size = GetStringExtent (pDC, str, _tcslen (str));
		rect.right = rect.left + size.cx;
		pDC->FillSolidRect (rect, crBk);
	}

	if (m_nLineVertSpacing > 0)
	{
		rect.top += m_nLineVertSpacing / 2;
	}

	if (m_bKeepTabs)  
	{
		return pDC->TabbedTextOut (rect.left, rect.top, str, 1, &m_nTabLogicalSize, nOrigin).cx;
	}
	
	return pDC->DrawText (str, rect, DT_NOCLIP | DT_SINGLELINE | DT_NOPREFIX);
	
}
//**************************************************************************************
/*
CSize CBCGPEditCtrl::GetStringExtent (CDC* pDC, LPCTSTR str)
{
	CSize size;
	if (m_bKeepTabs)  
	{
		size = pDC->GetOutputTabbedTextExtent (str, 1, &m_nTabLogicalSize);	
	}
	else
	{
		size = pDC->GetTextExtent (str);
	}

	return size;
}
*/
//**************************************************************************************
CSize CBCGPEditCtrl::GetStringExtent (CDC* pDC, LPCTSTR lpszString, int nCount)
{
	if (m_bEnableSymSupport)
	{
		ASSERT (m_pSymbolsImgList != NULL);

		int nTotalSymLen = 0;
		TCHAR* pszString = new TCHAR [nCount];
		for (int i = 0, j = 0; i < nCount; i++)
		{
			TCHAR chNext = lpszString [i];
			BCGP_EDIT_SYM_DEF symDef;
			int nSymExtraLen = 0;
			if (LookUpSymbol (lpszString, i, nCount, chNext, symDef, nSymExtraLen))
			{
				IMAGEINFO imgInfo;
				m_pSymbolsImgList->GetImageInfo (symDef.m_nIndex, &imgInfo);
				nTotalSymLen += imgInfo.rcImage.right - imgInfo.rcImage.left;
				i += nSymExtraLen;
			}
			else
			{
				pszString [j++] = chNext;
			}
		}
		
		CSize sizeResult;
		if (m_bKeepTabs)  
		{
			sizeResult = pDC->GetOutputTabbedTextExtent (pszString, j, 1, &m_nTabLogicalSize);	
		}
		else
		{
			sizeResult = pDC->GetTextExtent (pszString, j);
		}

		delete pszString;

		sizeResult.cx += nTotalSymLen;
		return sizeResult;
	}

	if (m_bKeepTabs)  
	{
		return pDC->GetOutputTabbedTextExtent (lpszString, nCount, 1, &m_nTabLogicalSize);	
	}
	
	return pDC->GetTextExtent (lpszString, nCount);
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsInsideSelection (int nOffset, CPoint& ptCharOffset, 
									   CRect rectSel, int nCharWidth) 
{
	if (m_bBlockSelectionMode)
	{
		CPoint ptCharOffsetExt (ptCharOffset.x + nCharWidth, ptCharOffset.y + m_nLineHeight);
		CRect rectChar (ptCharOffset, ptCharOffsetExt);
		return rectSel.IntersectRect (rectSel, rectChar);
	}

	BOOL bInsideBlock = (nOffset >= m_iStartSel && nOffset < m_iEndSel ||
						 nOffset >= m_iEndSel &&  nOffset < m_iStartSel);

	return bInsideBlock;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsColoredLine (int nRow, COLORREF& clrFore, COLORREF& clrBack)
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarker =  (CBCGPEditMarker*) m_lstMarkers.GetNext (pos);
		ASSERT_VALID (pMarker);

		if ((pMarker->m_dwMarkerType & g_dwBCGPEdit_LineColorMarker) == 0)
		{
			continue;
		}

		CBCGPLineColorMarker* pLineMarker = (CBCGPLineColorMarker*) pMarker;
		ASSERT_VALID (pLineMarker);

		if (nRow >= pLineMarker->m_nLine && 
			nRow <= pLineMarker->m_nLine + pLineMarker->m_nLineCount)
		{
			clrFore = pLineMarker->m_clrForeground;
			clrBack = pLineMarker->m_clrBackground; 
			return TRUE;
		}
	} 
	
	return FALSE;
}
//**************************************************************************************
int CBCGPEditCtrl::GetHorzRowExtent (CDC* pDC, int nOffset)
{
	if (m_nScrollOffsetHorz > 0)
	{
		int nNextRowEnd = m_strBuffer.Find (g_chEOL, nOffset);
		if (nNextRowEnd == -1)
		{
			nNextRowEnd = m_strBuffer.GetLength ();
		}
		if (nNextRowEnd != -1 && nOffset < m_strBuffer.GetLength ())
		{
			CString strNextRow = m_strBuffer.Mid (nOffset, nNextRowEnd);

			CString strTabs (_T (' '), m_nTabSize);
			strNextRow.Replace (_T ("\t"), strTabs);

			int nCount = min (m_nScrollOffsetHorz, strNextRow.GetLength ());
			CString strNextRowStart = strNextRow.Mid (0, nCount);
			
			return GetStringExtent (pDC, strNextRowStart, nCount).cx;
		}
	}
	return 0;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::FindOpenBlock (int nStartOffset, BCGP_EDIT_COLOR_BLOCK* pFoundBlock)
{
	LPTSTR  lpcszBuffer = (LPTSTR) (LPCTSTR) m_strBuffer;
	LPTSTR  lpBuffer = (LPTSTR) lpcszBuffer;

	LPTSTR  lpCloseBlock = NULL;
	LPTSTR	lpOpenBlock = NULL;

	// lpBuffer points to the offsset to start the reverse search from;
	lpBuffer = lpcszBuffer + nStartOffset;

	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
		if (colorBlock.m_bWholeText)
		{
			int nCloseBlockLen = colorBlock.m_strClose.GetLength ();
			int nOpenBlockLen  = colorBlock.m_strOpen.GetLength ();

			if (nOpenBlockLen == 0 || nCloseBlockLen == 0)
			{
				continue;
			}

			TCHAR chFirstCloseChar = colorBlock.m_strClose.GetAt (nCloseBlockLen - 1);
			TCHAR chFirstOpenChar = colorBlock.m_strOpen.GetAt (nOpenBlockLen - 1);

			BOOL bCloseBlockFound = FALSE;
			BOOL bOpenBlockFound = FALSE;
			do
			{
				bOpenBlockFound = FALSE;
				lpBuffer--;
				if (*lpBuffer == chFirstCloseChar && !bCloseBlockFound && 
					lpBuffer - nCloseBlockLen >= lpcszBuffer)
				{
					bCloseBlockFound = TRUE;
					for (int i = 0; i < nCloseBlockLen; i++)
					{
						if (*(lpBuffer - i) != colorBlock.m_strClose.GetAt (nCloseBlockLen - i - 1))
						{
							bCloseBlockFound = FALSE;
							break;
						}
					}

					if (bCloseBlockFound)
					{
						lpBuffer -= nCloseBlockLen;
						lpCloseBlock = lpBuffer;
					}
				}
				
				if (*lpBuffer == chFirstOpenChar)
				{
					if (nOpenBlockLen == 2 && lpBuffer > lpcszBuffer)
					{
						if (*(--lpBuffer) == colorBlock.m_strOpen.GetAt (0))		
						{
							// may be first opening symbol is the same as last closing symbol
							// like in C++ /* and */
							// To prevent this situation: */*
							if (lpBuffer > lpcszBuffer && 
								*(lpBuffer - 1) == chFirstOpenChar)
							{
								lpBuffer++;
								continue;
							}

							bOpenBlockFound = TRUE;
							lpOpenBlock = lpBuffer;
						}
						else
						{
							lpBuffer++;
						}
					}
					else if (nOpenBlockLen == 1)
					{
						bOpenBlockFound = TRUE;
						lpOpenBlock = lpBuffer;
					}
					else if (lpBuffer - nOpenBlockLen >= lpcszBuffer)
					{
						bOpenBlockFound = TRUE;
						for (int i = 0; i < nOpenBlockLen; i++)
						{
							if (*(lpBuffer - i) != colorBlock.m_strOpen.GetAt (nOpenBlockLen - i - 1))
							{
								bOpenBlockFound = FALSE;
								break;
							}
						}

						if (bOpenBlockFound)
						{
							lpBuffer -= nOpenBlockLen;
							lpOpenBlock = lpBuffer;
						}
					}

					if (bOpenBlockFound)
					{
						// check whether this open block mark is not commented out
						// at the beginning of the string by another block
		
						int nCurrOffset = (int) (lpBuffer - lpcszBuffer);
						int nRowOffset = GetRowStartByOffset (nCurrOffset);
						CList <BCGP_EDIT_COLOR_AREA, BCGP_EDIT_COLOR_AREA&> colorAreas;

						BuildColorAreas (colorAreas, nRowOffset, nCurrOffset + 2, &colorBlock, FALSE);

						for (POSITION posTmp = colorAreas.GetHeadPosition (); posTmp != NULL;)
						{
							BCGP_EDIT_COLOR_AREA colorArea = colorAreas.GetNext (posTmp);
							if (nCurrOffset > colorArea.m_nStart && 
								nCurrOffset < colorArea.m_nEnd)
							{
								int nCount = colorArea.m_nStart - 1;
								if (nCount < 0)
								{
									nCount = 0;
								}
								lpBuffer = lpcszBuffer + nCount;
								bOpenBlockFound = FALSE;
								lpOpenBlock = NULL;
								break;
							}
						}

						if (bOpenBlockFound)
						{
							break;
						}
					}						
				}
			}
			while (lpBuffer >= lpcszBuffer);

			if (bOpenBlockFound)
			{
				*pFoundBlock = colorBlock;
				break;
			}
			else
			{
				lpBuffer = lpcszBuffer + nStartOffset;
			}
		}
	}

	return (lpOpenBlock != NULL && lpCloseBlock == NULL);
}
//**************************************************************************************
int CBCGPEditCtrl::FindCloseBlock (int nStartOffset, BCGP_EDIT_COLOR_BLOCK* pFoundBlock)
{
	int nResult = -1;
	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
		if (colorBlock.m_bWholeText && colorBlock == *pFoundBlock)
		{
			nResult = m_strBuffer.Find (colorBlock.m_strClose, nStartOffset);
			if (nResult == -1)
			{
				nResult = m_strBuffer.GetLength () - 1;
			}
			else
			{
				nResult += colorBlock.m_strClose.GetLength ();
			}
		}
	}

	return nResult;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsInBlock (int nPos, TCHAR chOpen, TCHAR chClose, 
							   int& nBlockStart, int& nBlockEnd)
{
	BOOL bBlockFound = FALSE;
	CString strBlockChars;
	strBlockChars += chOpen;
	strBlockChars += chClose;

	int nStartOffset = nPos;
	int nLeftBlock = ReverseFindOneOf (m_strBuffer, nStartOffset, strBlockChars);
	if (nLeftBlock - 1 >= 0)
	{
		TCHAR chFound = GetCharAt (nLeftBlock - 1);
		if (chFound == chOpen)
		{
			int nRightBlock = FindOneOf (m_strBuffer, nStartOffset, strBlockChars);
			if (nRightBlock != -1)
			{
				chFound = GetCharAt (nRightBlock);
				if (chFound == chClose)
				{
					bBlockFound = TRUE;
					nBlockStart = nLeftBlock - 1;
					nBlockEnd = nRightBlock;
				}
			}
		}
	}

	return bBlockFound;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsCharBlockSymbol (TCHAR ch)
{
	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
		if (colorBlock.m_strOpen.Find (ch) != -1 || 
			colorBlock.m_strClose.Find (ch) != -1)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsStringHasBlockSymbols (const CString& str)
{
	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
		if (str.FindOneOf (colorBlock.m_strOpen) != -1 || 
			str.FindOneOf (colorBlock.m_strClose) != -1)
		{
			return TRUE;
		}
	}

	return FALSE;

}
//**************************************************************************************
void CBCGPEditCtrl::BuildColorAreas (CList <BCGP_EDIT_COLOR_AREA, BCGP_EDIT_COLOR_AREA&>& colorAreas, 
									 int nStartOffset, int nEndOffset,
									 BCGP_EDIT_COLOR_BLOCK* pColorBlockToExclude,
									 BOOL bRedraw)
{
	colorAreas.RemoveAll ();

	// get only a part of the big buffer
	CString strBuffer = m_strBuffer.Mid (nStartOffset, nEndOffset - nStartOffset);

	// we need to iterate the list of blocks and record the smallest start block position
	// and then look for the end of this position

	int nBufferLen = strBuffer.GetLength ();
	int nNextStartOffset = 0;

	while (nNextStartOffset < nBufferLen)
	{
		BCGP_EDIT_COLOR_AREA colorArea;
		int nIdxMin = strBuffer.GetLength ();
		POSITION posBlockToUse = NULL;

		for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL; 
													 m_lstColorBlocks.GetNext (pos))
		{
			BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetAt (pos);

			if (pColorBlockToExclude != NULL && 
				*pColorBlockToExclude == colorBlock)
			{
				continue;
			}

			if (colorBlock.m_strOpen.IsEmpty ())
			{
				continue;
			}
	
			int nIdx = strBuffer.Find (colorBlock.m_strOpen, nNextStartOffset);
			if (nIdx == -1)
			{
				continue;
			}

			if (nIdx < nIdxMin)
			{
				nIdxMin = nIdx;
				posBlockToUse = pos;
			}
		}

		if (posBlockToUse == NULL)
		{
			return;
		}

		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetAt (posBlockToUse);
		colorArea = colorBlock;
		colorArea.m_nStart = nIdxMin; 
		colorArea.m_nEnd = -1;
		if (!colorBlock.m_strClose.IsEmpty ())
		{
			do 
			{
				colorArea.m_nEnd = strBuffer.Find (colorBlock.m_strClose, nIdxMin + 1);
				if (colorArea.m_nEnd == -1)
				{
					break;
				}
				
				if (m_strContinueStringSymbols.Find (strBuffer.GetAt (colorArea.m_nEnd - 1)) >= 0)
				{
					nIdxMin = nIdxMin + 1;
					colorArea.m_nEnd = strBuffer.Find (colorBlock.m_strClose, nIdxMin + 1);
				}
				else
				{
					break;
				}
			}while (colorArea.m_nEnd != -1);
		}
		
		if (!colorBlock.m_bWholeText)
		{
			int nIdxEOL = strBuffer.Find (g_strEOL, nIdxMin + 1);
			if (colorArea.m_nEnd == -1 || (colorArea.m_nEnd > nIdxEOL && nIdxEOL != -1))
			{
				colorArea.m_nEnd = nIdxEOL;
			}
		}

		if (colorArea.m_nEnd == -1)
		{
			colorArea.m_nEnd = nBufferLen - 1;
		}

		nNextStartOffset = colorArea.m_nEnd + max (1, colorBlock.m_strClose.GetLength ());

		colorArea.m_nStart += nStartOffset;
		if (colorBlock.m_strClose.GetLength () > 0)
		{
			colorArea.m_nEnd += nStartOffset + colorBlock.m_strClose.GetLength () - 1;
		}
		else
		{
			colorArea.m_nEnd += nStartOffset;	
		}
		
		colorAreas.AddTail (colorArea);
	}

	if (bRedraw)
	{
		RedrawWindow ();
	}
}
//**************************************************************************************
BOOL CBCGPEditCtrl::OnGetWordColor (const CString& strWord, 
									COLORREF& clrText, COLORREF& clrBk, 
									int nPos)
{
	BCGP_EDIT_SYNTAX_COLOR clrWord;

	if (strWord.IsEmpty ())
	{
		return FALSE;
	}

	if (!m_mapWordColors.Lookup (strWord, clrWord))
	{
		CString strUpper = strWord; 
		strUpper.MakeUpper ();
		if (!m_mapWordColors.Lookup (strUpper, clrWord))
		{
			return FALSE;
		}

		if (!clrWord.m_bCaseSensitive)
		{
			clrText = clrWord.m_clrForeground;
			clrBk = clrWord.m_clrBackground;

			if (clrText == (COLORREF) -1)
			{
				clrText = GetDefaultTextColor ();
			}

			if (clrBk == (COLORREF) -1)
			{	
				clrBk = GetDefaultBackColor ();
			}

			return TRUE;
		}

		return FALSE;
	}

	if (m_mapWordColors.Lookup (strWord, clrWord))
	{
		clrText = clrWord.m_clrForeground;
		clrBk = clrWord.m_clrBackground;

		if (clrText == (COLORREF) -1)
		{
			clrText = GetDefaultTextColor ();
		}

		if (clrBk == (COLORREF) -1)
		{	
			clrBk = GetDefaultBackColor ();
		}

		return TRUE;
	}

	return FALSE;
}
//************************************************************************************
BOOL CBCGPEditCtrl::Paste (int iPos /*= -1*/)
{
	ASSERT_VALID (this);

	if (m_bReadOnly)
	{
		OnFailedOperation (g_dwOpPaste | g_dwOpReasonReadOnly);
		return FALSE;
	}

	COleDataObject data;
	if (!data.AttachClipboard ())
	{
		TRACE0("Can't open clipboard\n");
		OnFailedOperation (g_dwOpPaste | g_dwOpReasonError);
		return FALSE;
	}

	if (!data.IsDataAvailable (CF_TEXT) && 
		!data.IsDataAvailable (g_cFormat))
	{
		TRACE0("Incorrect clipboard format\n");
		OnFailedOperation (g_dwOpPaste | g_dwOpReasonError);
		return FALSE;
	}

	BOOL bForceNextUndo = m_iStartSel != 0;
	
	DeleteSelectedText (FALSE, FALSE, bForceNextUndo);
	SetLastUndoReason (g_dwUATPaste);
	PasteFromDataObject (&data, iPos, TRUE, FALSE, TRUE, FALSE);

	m_bIsModified = TRUE;

	return TRUE;
}
//************************************************************************************
BOOL CBCGPEditCtrl::PasteFromDataObject (COleDataObject* pDataObject, 
										 int nInsertFrom, BOOL bRedraw, 
										 BOOL bSuppressUndo, BOOL bUpdateLineData,
										 BOOL bForceNextUndo)
{
		// check for internal data first - copy was made by edit control
	HGLOBAL hClipbuffer = NULL;
	bool bBlock = false;
	if (pDataObject->IsDataAvailable (g_cFormat))
	{
		hClipbuffer = pDataObject->GetGlobalData (g_cFormat);
		bBlock = true;
	}
#ifdef _UNICODE
	else if (pDataObject->IsDataAvailable (CF_UNICODETEXT))
	{
		hClipbuffer = pDataObject->GetGlobalData (CF_UNICODETEXT);
	}
#else
	else if (pDataObject->IsDataAvailable (CF_TEXT))
	{
		hClipbuffer = pDataObject->GetGlobalData (CF_TEXT);
	}
#endif

	BOOL bResult = FALSE;
	if (hClipbuffer != NULL)
	{
		LPTSTR lpszBuffer = (LPTSTR) GlobalLock (hClipbuffer);
		ASSERT (lpszBuffer != NULL);

		bResult = bBlock ? InsertTextAsBlock (lpszBuffer, nInsertFrom, bRedraw, bSuppressUndo, bUpdateLineData, bForceNextUndo) :
						InsertText (lpszBuffer, nInsertFrom, bRedraw, bSuppressUndo, bUpdateLineData, bForceNextUndo);

		::GlobalUnlock (hClipbuffer);
	}

	return bResult;
}
//************************************************************************************
BOOL CBCGPEditCtrl::IsPasteEnabled () const
{
	return ::IsClipboardFormatAvailable (CF_TEXT) ||
			::IsClipboardFormatAvailable (g_cFormat);
}
//************************************************************************************
BOOL CBCGPEditCtrl::Copy ()
{
	ASSERT_VALID (this);

	if (!IsCopyEnabled ())
	{
		ASSERT (FALSE);
		OnFailedOperation (g_dwOpCopy);
		return FALSE;
	}

	if (m_iStartSel >= 0)
	{
		if (m_iEndSel < 0)
		{
			ASSERT (FALSE);
			OnFailedOperation (g_dwOpCopy);
			return FALSE;
		}

		CString strBlock;
		PrepareBlock (strBlock);

		return CopyTextToClipboard (strBlock, strBlock.GetLength ());
	}

	if (!m_bEnableWholeTextCopy)
	{
		ASSERT (FALSE);
		OnFailedOperation (g_dwOpCopy);
		return FALSE;
	}

	return CopyTextToClipboard (m_strBuffer, m_strBuffer.GetLength ());	// Copy the whole text
}
//************************************************************************************
BOOL CBCGPEditCtrl::Cut ()
{
	ASSERT_VALID (this);

	if (!IsCutEnabled ())
	{
		ASSERT (FALSE);
		OnFailedOperation (g_dwOpCopy);
		return FALSE;
	}
	SetLastUndoReason (g_dwUATCut);
	return DeleteSelectedText (TRUE, TRUE);
}
//***************************************************************************************		
BOOL CBCGPEditCtrl::Clear ()
{
	ASSERT_VALID (this);
	SetLastUndoReason (g_dwUATDelete);
	return DeleteSelectedText ();
}
//***************************************************************************************		
BOOL CBCGPEditCtrl::DeleteText (int nStartOffset, int nEndOffset, 
								BOOL bRedraw)
{
	int nStart	= min (nStartOffset, nEndOffset);
	int nEnd	= max (nStartOffset, nEndOffset);

	int nDeleteCount = nEnd - nStart;

	if (nDeleteCount == 0)
	{
		return FALSE;
	}

	if (nDeleteCount > m_strBuffer.GetLength ())
	{
		int nDelta = nDeleteCount - m_strBuffer.GetLength ();
		nEnd -= nDelta;
	}


	CString strDeletedText = m_strBuffer.Mid (nStart, nEnd - nStart);

	if (!OnBeforeTextDeleted (nStart, strDeletedText))
	{
		return FALSE;
	}

	AddUndoAction (strDeletedText, g_dwUATDeleteData, nStart, TRUE);

	OnDeleteTextFromBuffer (nStart, nEnd, strDeletedText);

	m_strBuffer.Delete (nStart, nEnd - nStart);

	OnAfterTextChanged (nStart, strDeletedText, FALSE);

	if (bRedraw)
	{
		RedrawWindow ();
	}

	m_nCurrOffset = nStart;
	SetCaret (nStart);
	
	UpdateScrollBars ();

	return TRUE;
}
//***************************************************************************************		
BOOL CBCGPEditCtrl::DeleteSelectedText (BOOL bRedraw, BOOL bCopyToClipboard, 
										BOOL bForceNextUndo)
{
	if (m_iStartSel < 0)
	{
		OnFailedOperation (g_dwOpDelSelText);
		return FALSE;
	}

	if (m_bBlockSelectionMode)
	{
		
	}

	int iStartSel = min (m_iStartSel, m_iEndSel);
	int iEndSel = max (m_iStartSel, m_iEndSel);

	CString strDeletedText = m_strBuffer.Mid (iStartSel, iEndSel - iStartSel);

	if (!OnBeforeTextDeleted (iStartSel, strDeletedText))
	{
		return FALSE;
	}

	if (bCopyToClipboard && 
		!CopyTextToClipboard (strDeletedText, strDeletedText.GetLength ()))
	{
		OnFailedOperation (g_dwOpDelSelText | g_dwOpReasonError);
		return FALSE;
	}

	int nDeleteCount = iEndSel - iStartSel;
	if (nDeleteCount > m_strBuffer.GetLength ())
	{
		int nDelta = nDeleteCount - m_strBuffer.GetLength ();
		iEndSel -= nDelta;
	}
	
	AddUndoAction (strDeletedText, g_dwUATDeleteData, iStartSel, FALSE, bForceNextUndo);

	OnDeleteTextFromBuffer (iStartSel, iEndSel, strDeletedText);
	m_strBuffer.Delete (iStartSel, iEndSel - iStartSel);

	OnAfterTextChanged (iStartSel, strDeletedText, FALSE);

	int nRedrawOffset = iStartSel;
	m_iStartSel = m_iEndSel = -1;
	m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);

	UpdateScrollBars ();

	m_nScrollOffsetVert = RowFromOffset (m_nTopVisibleOffset, TRUE);
	SetScrollPos (SB_VERT, m_nScrollOffsetVert);

	if (bRedraw && nDeleteCount > 0)
	{
		RedrawWindow ();
	}

	m_nCurrOffset = nRedrawOffset;
	SetCaret (nRedrawOffset);

	return TRUE;
}
//***************************************************************************************		
void CBCGPEditCtrl::OnInsertTextToBuffer (int nStartOffset, 
										  const CString& strInsertedText,
										  BOOL bUpdateLineData)
{
	int nNewRowsInserted = GetNumOfCharsInText (strInsertedText, g_chEOL);
	
	if (nNewRowsInserted > 0 && bUpdateLineData)
	{
		int nStartRow = RowFromOffset (nStartOffset);
		int nStartRowOffset = GetRowStartByOffset (nStartOffset);
		if (nStartRowOffset == nStartOffset)
		{
			// inserted at this row from the start - move the whole row
			UpdateLineRelatedData (nStartRow, nNewRowsInserted);
		}
		else
		{
			// update markesr from the next row
			UpdateLineRelatedData (nStartRow + 1, nNewRowsInserted);
		}
	}

	m_nTotalLines += nNewRowsInserted;
}
//***************************************************************************************		
void CBCGPEditCtrl::OnDeleteTextFromBuffer (int nStartOffset, int nEndOffset, 
										  LPCTSTR lpcszDeletedText)
{
	CString strDeletedText;
	if (lpcszDeletedText == NULL)
	{
		strDeletedText = m_strBuffer.Mid (nStartOffset, nEndOffset - nStartOffset);
	}
	else
	{
		strDeletedText = lpcszDeletedText;
	}

	if (strDeletedText.Find (g_strEOL) >= 0)
	{
		int nRowCount = GetNumOfCharsInText (strDeletedText, g_chEOL);
		int nStartRow = RowFromOffset (nStartOffset);
		int nEndRow = RowFromOffset (nEndOffset);

		int nRowStartOffset = GetRowStartByOffset (nStartOffset);
		BOOL bWholeStartRowDeleted = nRowStartOffset == nStartOffset;

		int nRowEndOffset = GetRowEndByOffset (nEndOffset);
		BOOL bWholeEndEowDeleted = nRowEndOffset == nEndOffset;

		RemoveLineDataInRange  (bWholeStartRowDeleted ? nStartRow : nStartRow + 1, 
								bWholeEndEowDeleted ? nEndRow : nEndRow - 1);
		UpdateLineRelatedData (nStartRow + 1, -nRowCount);

		m_nTotalLines -= nRowCount;
	}

	if (nStartOffset < m_nTopVisibleOffset)
	{
		if (nEndOffset < m_nTopVisibleOffset)
		{
			m_nTopVisibleOffset -= (nEndOffset - nStartOffset);
		}
		else 
		{
			m_nTopVisibleOffset -= (m_nTopVisibleOffset - nStartOffset);
		}
		m_nTopVisibleOffset = GetRowStartByOffset (m_nTopVisibleOffset);
	}
}
//***************************************************************************************		
BOOL CBCGPEditCtrl::CopyTextToClipboard (LPCTSTR lpszText, int nLen)
{
	ASSERT_VALID (this);
	ASSERT (lpszText != NULL);

	try
	{
		if (!OpenClipboard ())
		{
			TRACE0("Can't open clipboard\n");
			return FALSE;
		}

		if (!::EmptyClipboard ())
		{
			TRACE0("Can't empty clipboard\n");
			::CloseClipboard ();
			return FALSE;
		}

		if (nLen == -1)
		{
			nLen = _tcslen (lpszText);
		}
		if (m_bBlockSelectionMode && 
			!CopyTextToClipboardInternal (lpszText, g_cFormat, nLen))
		{
			::CloseClipboard ();
			return FALSE;
		}

#ifdef _UNICODE
		if (!CopyTextToClipboardInternal (lpszText, CF_UNICODETEXT, nLen))
		{
			::CloseClipboard ();
			return FALSE;
		}
#else
		if (!CopyTextToClipboardInternal (lpszText, CF_TEXT, nLen))
		{
			::CloseClipboard ();
			return FALSE;
		}
#endif

		::CloseClipboard ();
	}
	catch (...)
	{
		TRACE0("CopyTextToClipboard: out of memory\n");
	}

	return TRUE;
}
//*****************************************************************************
HGLOBAL CBCGPEditCtrl::CopyTextToClipboardInternal (LPCTSTR lpszText, 
													CLIPFORMAT cFormat, 
													int nLen)
{
	try
	{
		HGLOBAL hClipbuffer = ExportBuffer (lpszText, nLen, 
											FALSE, m_bReplaceTabsAndEOLOnCopy);
		if (hClipbuffer != NULL)
		{
			::GlobalUnlock (hClipbuffer);
			::SetClipboardData (cFormat, hClipbuffer);
		}

		return hClipbuffer;
	}
	catch (...)
	{
		TRACE0("CopyTextToClipboardInternal: out of memory\n");
	}
	return NULL;	
}
//*****************************************************************************
HGLOBAL CBCGPEditCtrl::ExportBuffer (LPCTSTR lpszText, int nTextLen, BOOL bReplaceTabs, 
									 BOOL bReplaceEOL)
{
	SIZE_T cbFinalSize = (nTextLen + 1) * 2 * sizeof(TCHAR);

	HGLOBAL hExpBuffer = ::GlobalAlloc (GMEM_DDESHARE, cbFinalSize);

	if (hExpBuffer == NULL)
	{
		TRACE0("ExportBuffer: out of memory\n");
		return NULL;
	}

	LPTSTR lpExpBuffer = (LPTSTR) GlobalLock (hExpBuffer);
	if (lpExpBuffer == NULL)
	{
		TRACE0("ExportBuffer: out of memory\n");
		GlobalFree (hExpBuffer);
		return NULL;
	}

	
	if (!bReplaceTabs && !bReplaceEOL)
	{
		memcpy (lpExpBuffer, lpszText, nTextLen * sizeof(TCHAR));
		lpExpBuffer [nTextLen] = _T ('\0');
		cbFinalSize = (nTextLen + 1) * sizeof(TCHAR);
	}
	else
	{
		int nSpaceCount = 0;
		for (int nTextIdx = 0, nExpIdx = 0; nTextIdx < nTextLen; nTextIdx++, nExpIdx++)
		{
			if (bReplaceTabs && lpszText [nTextIdx] == _T (' '))
			{
				nSpaceCount++;
				if (nSpaceCount == m_nTabSize)
				{
					nExpIdx -= (m_nTabSize - 1);
					lpExpBuffer [nExpIdx] = _T ('\t');
					nSpaceCount = 0;
					continue;
				}
			}
			else
			{
				nSpaceCount = 0;
			}

			if (bReplaceEOL && lpszText [nTextIdx] == g_chEOL)
			{
				lpExpBuffer [nExpIdx] = g_strEOLExport [0];
				lpExpBuffer [++nExpIdx] = g_strEOLExport [1];
				continue;
			}

			lpExpBuffer [nExpIdx] = lpszText [nTextIdx];
		}

		lpExpBuffer [nExpIdx] = _T ('\0');
		cbFinalSize = (nExpIdx + 1) * sizeof(TCHAR);
	}

	GlobalUnlock (hExpBuffer);

	HGLOBAL hGlobalRet = GlobalReAlloc (hExpBuffer, cbFinalSize, 0);
	if (hGlobalRet == NULL)
	{
		return hExpBuffer;
	}
	return hGlobalRet;
}
//*****************************************************************************
void CBCGPEditCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CWnd::OnRButtonDown(nFlags, point);
	SetFocus ();

	if (m_iStartSel < 0 || !IsPointInSelectionRect (point))
	{
		RemoveSelection (FALSE, TRUE);

		int nOffset = HitTest (point, TRUE);
		if (nOffset >= 0)
		{
			SetCaret (nOffset);
		}

		if (GetCapture () == this)
		{
			ReleaseCapture ();
			if (m_nScrollTimer != -1)
			{
				KillTimer (m_nScrollTimer);
				m_nScrollTimer = -1;
			}
		}
	}
}
//*****************************************************************************
void CBCGPEditCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CWnd::OnLButtonDown(nFlags, point);
	if (GetFocus () != this)
	{
		SetFocus ();
	}
	
	BOOL bPerformDragDrop = FALSE;

	int iStartSel = -1;
	int iEndSel = -1;

	iStartSel = min (m_iEndSel, m_iStartSel);
	iEndSel = max (m_iEndSel, m_iStartSel);

	if (IsPointInSelectionRect (point) && nFlags & MK_LBUTTON)
	{
		CString strText;
		PrepareBlock (strText);

		if (OpenClipboard ())
		{
			HGLOBAL hClipbufferText = ::GlobalAlloc (GMEM_DDESHARE, (strText.GetLength () + 1) * sizeof (TCHAR));
			LPTSTR lpszBufferText = (LPTSTR) GlobalLock (hClipbufferText);

			_tcscpy (lpszBufferText, strText);

			::GlobalUnlock (hClipbufferText);
			::CloseClipboard ();

			BOOL bRemoveSelection = FALSE;

			if (hClipbufferText != NULL)
			{
				CPoint ptStart = point;

				m_bDragTextMode = TRUE;
				bPerformDragDrop = TRUE;

				COleDataSource srcItem;
				srcItem.CacheGlobalData (CF_TEXT, hClipbufferText);
				
				DROPEFFECT dropEffect = srcItem.DoDragDrop (DROPEFFECT_COPY | DROPEFFECT_MOVE);

				CPoint ptEnd;
				GetCursorPos (&ptEnd);
				ScreenToClient (&ptEnd);

				m_bDragTextMode = FALSE;

				if (!m_bReadOnly)
				{
					ShowCaret ();
				}

				if (dropEffect != DROPEFFECT_NONE)
				{
					int nLen = iEndSel - iStartSel;

					if (dropEffect == DROPEFFECT_MOVE)
					{
						// m_nDropOffset is -1 when the text is dropped to
						// another application

						if (iStartSel >= 0 && m_nDropOffset <= iStartSel &&
							m_nDropOffset != -1)
						{
							// moving text before selecttion, text inserted, section index should
							// be incremented
							m_iStartSel += nLen;
							m_iEndSel += nLen;
						}

						if (iStartSel >= 0 && iStartSel <= m_nDropOffset && 
							m_nDropOffset != -1)
						{
							// moving text after selection, text deleted, offset should be decremented
							m_nDropOffset -= nLen;
						}
						
						if (!m_bReadOnly)
						{
							// the third TRUE parameter will cause Undo action be played twise
							DeleteSelectedText (FALSE, FALSE);
						}
					}
					
					if (m_nDropOffset != -1)
					{
						if (m_bBlockSelectionMode)
						{
							OffsetToPoint (m_iStartSel, m_ptStartBlockSel);
							OffsetToPoint (m_iEndSel, m_ptEndBlockSel);
						}
						else
						{
							m_iStartSel = m_nDropOffset;
							m_iEndSel = m_nDropOffset + nLen;
							SetCaret (m_iEndSel);
						}
					}
					else if ((ptEnd.y - ptStart.y) == 0 && (ptEnd.x - ptStart.x) == 0)
					{
						bRemoveSelection = TRUE;
					}
					
					RedrawWindow ();			
					m_nDropOffset = -1;
				}

				if ((ptEnd.y - ptStart.y) == 0 && (ptEnd.x - ptStart.x) == 0)
				{
					bRemoveSelection = TRUE;
				}
			}

			if (bRemoveSelection)
			{
				m_iStartSel = m_iEndSel = -1;
				m_bBlockSelectionMode = FALSE; 
				m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
				RedrawWindow ();			
				int nOffset = HitTest (point, TRUE);
				if (nOffset >= 0)
				{
					SetCaret (nOffset);
				}
			}
		}
		return;
	}

	if (point.x >= 0 && point.x < m_rectText.left)
	{
		point.x = m_rectText.left;
	}

	int nOffset = HitTest (point, TRUE);
	if (nOffset >= 0)
	{
		SetCaret (nOffset);
		if ((nFlags & MK_LBUTTON) && !bPerformDragDrop)
		{
			SetCapture ();
			if (::GetAsyncKeyState (VK_MENU) & 0x8000)
			{
#ifdef __ENABLE_BLOCK_SELECTION__						
				m_bBlockSelectionMode = TRUE;
				m_bAltPressedForBlockSel = TRUE;
#endif //__ENABLE_BLOCK_SELECTION__								
			}
		}
	}

	if (m_iStartSel >= 0)
	{
		m_iEndSel = m_iStartSel = -1;
		m_bBlockSelectionMode = FALSE;
		m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
		RedrawWindow ();
	}
}
//************************************************************************************
void CBCGPEditCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (GetCapture () == this)
	{
		ReleaseCapture ();
	}

	if (m_nScrollTimer != -1)
	{
		KillTimer (m_nScrollTimer);
		m_nScrollTimer = -1;
	}

	m_nLastMaxColumn = GetCurColumn ();

	if (m_iEndSel == m_iStartSel && m_iStartSel != -1)
	{
		BOOL bRedraw = m_iEndSel != -1;
		m_iEndSel = m_iStartSel = -1;
		m_bBlockSelectionMode = FALSE;
		m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
		if (bRedraw)
		{
			RedrawWindow ();
		}
	}

	CWnd::OnLButtonUp(nFlags, point);
}
//************************************************************************************
void CBCGPEditCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	int nOffset = HitTest (point);
	
	if (nOffset >= 0 && nOffset < m_strBuffer.GetLength () && !m_bIntelliSenseMode)
	{
		TCHAR ch = m_strBuffer.GetAt (nOffset);

		bool bWordFound = false;
		int nStartSel = -1;
		int nEndSel = -1;
		
		if (ch == _T (' ') || ch == _T ('\t'))
		{
			nStartSel = WhileOneOf (m_strBuffer, nOffset, false, _T (" \t"));
			nEndSel = WhileOneOf (m_strBuffer, nOffset, true, _T (" \t"));

			if (nEndSel - nStartSel > 1)
			{
				m_iStartSel = nStartSel;
				m_iEndSel = nEndSel;
				bWordFound = true;
			}
			
		}

		if (!bWordFound)
		{
			if (m_strNonSelectableChars.Find (ch) != -1)
			{
				int nNextWord = WhileOneOf (m_strBuffer, nOffset, true, m_strNonSelectableChars);
				if (nNextWord == m_strBuffer.GetLength ())
				{
					nNextWord = WhileOneOf (m_strBuffer, nOffset, false, m_strNonSelectableChars);
				}
				nOffset = nNextWord;
			}
			
			FindWordStartFinish (nOffset, m_strBuffer, m_iStartSel, m_iEndSel);
		}

		SetCaret (m_iEndSel);
		RedrawWindow ();		
	}
	
	CWnd::OnLButtonDblClk(nFlags, point);
}
//************************************************************************************
void CBCGPEditCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	int nOldMinSel = min (m_iStartSel, m_iEndSel);
	int nOldMaxSel = max (m_iStartSel, m_iEndSel);
	
	if (nFlags & MK_LBUTTON && !m_bIntelliSenseMode && GetCapture () == this)
	{
		if (!m_rectText.PtInRect (point) && m_nScrollTimer == -1)
		{
			m_nScrollTimer = SetTimer (ID_TEXT_SCROLL, m_nScrollTimeOut, NULL);
		}
		else if (m_nScrollTimer != -1 && m_rectText.PtInRect (point))
		{
			KillTimer (m_nScrollTimer);
			m_nScrollTimer = -1;
		}

		if (point.x < m_rectText.left)
		{
			point.x = m_rectText.left;
		}
		if (point.x > m_rectText.right)
		{
			point.x = m_rectText.right;
		}

		CPoint ptCaret = point;
		int nOffset = HitTest (ptCaret, TRUE, TRUE);

		if (m_bBlockSelectionMode)
		{
			CPoint ptOld = m_ptEndBlockSel;

			if (m_iStartSel < 0)
			{
				m_iStartSel = m_iEndSel = m_nCurrOffset;
				OffsetToPoint (m_iStartSel, m_ptStartBlockSel);
				m_ptStartBlockSel.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
				m_ptStartBlockSel.y += m_nScrollOffsetVert * m_nLineHeight;

				m_ptEndBlockSel = m_ptStartBlockSel;
			}
			else
			{
				if (nOffset >= 0)
				{
					CRect rectOld (m_ptStartBlockSel, m_ptEndBlockSel);

					if (nOffset > -1)
					{
						SetCaret (nOffset, TRUE, FALSE);
					}

					m_iEndSel = nOffset;
					m_ptEndBlockSel = point;

					CPoint pt;
					OffsetToPoint (m_iEndSel, pt);
					m_ptEndBlockSel.y = pt.y + m_nLineHeight;

					if (abs (pt.x - m_ptEndBlockSel.x) > m_nMaxCharWidth)					
					{
						pt.x = m_ptEndBlockSel.x;
					}
					else
					{
						m_ptEndBlockSel.x = pt.x;
					}
					
					SetCaretPos (pt);


					m_ptEndBlockSel.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
					m_ptEndBlockSel.y += m_nScrollOffsetVert * m_nLineHeight;

					if (m_ptEndBlockSel != ptOld)
					{
						RedrawWindow (m_rectText);
					}
				}
			}
		}
		else
		{
			if (m_iStartSel < 0)
			{
				m_iStartSel = m_iEndSel = m_nCurrOffset;
			}
			else
			{
				if (nOffset >= 0)
				{
					m_iEndSel = nOffset;
					SetCaret (nOffset);
					if (m_nLastSelOffset != nOffset)
					{
						int nMinNewSel = min (m_iEndSel, m_iStartSel);
						int nMaxNewSel = max (m_iEndSel, m_iStartSel);
						//RedrawWindow ();
						RedrawTextOffsets (min (nOldMinSel, nMinNewSel), 
										   max (nOldMaxSel, nMaxNewSel));
					}
				}

				m_nLastSelOffset = nOffset;
			}
		}
	}

	if (m_bEnableToolTips && GetCapture () != this)
	{
		CString strText;
		BOOL bIsWordFromPoint = GetWordFromPoint (point, strText);
		if (!bIsWordFromPoint ||
			!(strText == strTipText || 
			  (OnGetTipText (strText) &&
			   strText == strTipText)))
		{
			ToolTipPop ();
		}			

		if (!bIsWordFromPoint)
		{
			m_strLastDisplayedToolTip.Empty ();
		}

	}

	CWnd::OnMouseMove(nFlags, point);
}
//*************************************************************************************
void CBCGPEditCtrl::OnTimer(UINT nIDEvent) 
{
	switch (nIDEvent)
	{
	case ID_TEXT_SCROLL:
		{
			CPoint ptMouse;
			GetCursorPos (&ptMouse);
			ScreenToClient (&ptMouse);
			PostMessage (WM_MOUSEMOVE, MK_LBUTTON, MAKELPARAM (ptMouse.x, ptMouse.y));
		}
		break;
	}
	
	CWnd::OnTimer(nIDEvent);
}
//*************************************************************************************
void CBCGPEditCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	CWnd::OnKillFocus(pNewWnd);

	if (pNewWnd == GetParent () && pNewWnd != NULL &&
		pNewWnd->IsKindOf (RUNTIME_CLASS (CView)))
	{
		return;
	}

	m_nSavedOffset = m_nCurrOffset;
	m_ptSavedBlockCaretOffset = GetCaretPos ();
	m_ptSavedBlockCaretOffset.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
	m_ptSavedBlockCaretOffset.y += m_nScrollOffsetVert * m_nLineHeight;

	if (/*m_bClearSelectionWhenInactive*/FALSE)
	{
		m_iStartSel = m_iEndSel = -1;
		m_bBlockSelectionMode = FALSE;
		m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
	}

	RedrawWindow ();

	if (!m_bIntelliSenseMode && !m_bReadOnly)
	{
		::DestroyCaret ();
	}
}
//*************************************************************************************
void CBCGPEditCtrl::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);
	
	if (!m_bReadOnly)
	{
		::DestroyCaret ();
		CSize sizeCaret = GetCaretSize ();
		::CreateCaret (GetSafeHwnd (), NULL, sizeCaret.cx, sizeCaret.cy);

		if (m_bBlockSelectionMode)
		{
			m_ptSavedBlockCaretOffset.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;
			m_ptSavedBlockCaretOffset.y -= m_nScrollOffsetVert * m_nLineHeight;
			SetCaretPos (m_ptSavedBlockCaretOffset);
		}
		else
		{
			CPoint pt;
			OffsetToPoint (m_nCurrOffset, pt);

			BOOL bInRect = m_rectText.PtInRect (pt);
			SetCaret (m_nCurrOffset, bInRect);
			
			CPoint ptCurrPos = GetCaretPos ();
			if (!m_rectText.PtInRect (pt))
			{
				SetCaretPos (pt);
			}
		}
		
		
		ShowCaret ();
	}

	RedrawWindow ();
}
//*************************************************************************************
void CBCGPEditCtrl::GetSel (int& nStartChar, int& nEndChar) const
{
	nStartChar = m_iStartSel;
	nEndChar = m_iEndSel;
}
//*************************************************************************************
BOOL CBCGPEditCtrl::GetSelectionRect (CRect& rect1, CRect& rect2, CRect& rect3) 
{
	ASSERT_VALID (this);

	rect1.SetRectEmpty ();
	rect2.SetRectEmpty ();
	rect3.SetRectEmpty ();

	if (m_iStartSel < 0 || m_iEndSel < 0)
	{
		return FALSE;
	}

	int iStartSel = min (m_iStartSel, m_iEndSel);
	int iEndSel = max (m_iStartSel, m_iEndSel);

	CPoint ptStart;
	OffsetToPoint (iStartSel, ptStart);

	CPoint ptEnd;
	OffsetToPoint (iEndSel, ptEnd);

	if (ptStart.y == ptEnd.y)
	{
		// Single row.
		rect1 = CRect (ptStart.x, ptStart.y, ptEnd.x, ptStart.y + m_nLineHeight);
	}
	else
	{
		ASSERT (ptStart.y < ptEnd.y);

		rect1 = CRect (ptStart.x, ptStart.y, m_rectText.right, ptStart.y + m_nLineHeight);
		rect2 = CRect (m_rectText.left, ptStart.y + m_nLineHeight,m_rectText.right, ptEnd.y);
		rect3 = CRect (m_rectText.left, ptEnd.y, ptEnd.x, ptEnd.y + m_nLineHeight);
	}

	return TRUE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollText (BOOL bDirection)
{
	bDirection ? ScrollDown (SB_VERT, TRUE) : ScrollUp (SB_VERT, TRUE);

	CRect rectText = m_rectText;
	CPoint pt;
	OffsetToPoint (m_nCurrOffset, pt);
	if (!bDirection)
	{
		// if scrolling up always keep caret one-two lines off the bottom
		rectText.bottom -= m_nLineHeight * 2;
	}
	
	if (rectText.PtInRect (pt))
	{
		SetCaret (m_nCurrOffset, FALSE, FALSE);
		OffsetToPoint (m_nCurrOffset, pt);
		SetCaretPos (pt);
	}
	else
	{
		bDirection ? pt.y += m_nLineHeight : pt.y -= m_nLineHeight;
		int nOffset = HitTest (pt);
		if (!SetCaretToLastMaxColumn (nOffset))
		{
			SetCaret (nOffset, FALSE, FALSE);
			SetCaretPos (pt);
		}
	}
	return TRUE;
}
//***************************************************************************************
void CBCGPEditCtrl::UpdateScrollBars ()
{
	ASSERT_VALID (this);

	if (m_nMaxCharWidth == 0 || m_nLineHeight == 0)
	{
		ASSERT (FALSE);
		return;
	}

	m_szTotalScroll = CSize (m_nMaxScrollWidth, m_nTotalLines);

	int nBufLen = m_strBuffer.GetLength ();
	if (nBufLen > 0 && m_strBuffer [nBufLen - 1] != g_chEOL)
	{
		m_szTotalScroll.cy++;
	}

	SCROLLINFO siHorz;
	siHorz.cbSize = sizeof (SCROLLINFO);
	siHorz.fMask = SIF_RANGE | SIF_PAGE;
	siHorz.nMin = 0;
	siHorz.nMax = m_szTotalScroll.cx;
	siHorz.nPage = m_rectText.Width () / m_nMaxCharWidth;

	SetScrollInfo (SB_HORZ, &siHorz);

	SCROLLINFO siVert;
	siVert.cbSize = sizeof (SCROLLINFO);
	siVert.fMask = SIF_RANGE | SIF_PAGE;
	siVert.nMin = 0;
	siVert.nMax = m_szTotalScroll.cy;
	siVert.nPage = m_rectText.Height () / m_nLineHeight;

	SetScrollInfo (SB_VERT, &siVert);
}
//***************************************************************************************
void CBCGPEditCtrl::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* /*pScrollBar*/) 
{
	if (m_bBlockSelectionMode)
	{
		m_ptSavedBlockCaretOffset = GetCaretPos ();
		m_ptSavedBlockCaretOffset.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
		m_ptSavedBlockCaretOffset.y += m_nScrollOffsetVert * m_nLineHeight;
	}
	

	BOOL bScroll = OnScroll (SB_VERT, nSBCode, nPos);
	CPoint pt;
	OffsetToPoint (m_nCurrOffset, pt);

	if (bScroll)
	{
		ToolTipPop ();
		RedrawWindow ();
	
		if (!m_bBlockSelectionMode)
		{
			if (m_rectText.PtInRect (pt))
			{
				SetCaretPos (pt);
				m_ptCaret = pt;
			}	
			else
			{
				SetCaretPos (CPoint (-10, -10));
			}
		}
		else
		{
			m_ptSavedBlockCaretOffset.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;
			m_ptSavedBlockCaretOffset.y -= m_nScrollOffsetVert * m_nLineHeight;
			SetCaretPos (m_ptSavedBlockCaretOffset);
		}
	}
}
//***************************************************************************************
void CBCGPEditCtrl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* /*pScrollBar*/) 
{
	if (m_bBlockSelectionMode)
	{
		m_ptSavedBlockCaretOffset = GetCaretPos ();
		m_ptSavedBlockCaretOffset.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
		m_ptSavedBlockCaretOffset.y += m_nScrollOffsetVert * m_nLineHeight;
	}

	BOOL bScroll = OnScroll (SB_HORZ, nSBCode, nPos);
	CPoint pt;
	OffsetToPoint (m_nCurrOffset, pt);

	if (bScroll)
	{
		ToolTipPop ();
		RedrawWindow ();
		
		if (!m_bBlockSelectionMode)
		{
			if (m_rectText.PtInRect (pt))
			{
				SetCaretPos (pt);
				m_ptCaret = pt;
			}	
			else
			{
				SetCaretPos (CPoint (-10, -10));
			}
		}
		else
		{
			m_ptSavedBlockCaretOffset.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;
			m_ptSavedBlockCaretOffset.y -= m_nScrollOffsetVert * m_nLineHeight;
			SetCaretPos (m_ptSavedBlockCaretOffset);
		}
	}		
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollDown (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_LINEDOWN);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollUp (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_LINEUP);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollPageDown (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_PAGEDOWN);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollPageUp (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_PAGEUP);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollHome (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_TOP);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::ScrollEnd (int fnBar, BOOL bRedrawOnScroll)
{
	BOOL bScroll = OnScroll (fnBar, SB_BOTTOM);
	if (bScroll && bRedrawOnScroll)
	{
		RedrawWindow ();
	}
	return bScroll;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::OnScroll (int fnBar, UINT nSBCode, int nPos)
{
	if (fnBar == SB_CTL)
	{
		ASSERT (FALSE);
		return FALSE;
	}

	int& nScrollOffset = (fnBar == SB_VERT) ? m_nScrollOffsetVert : m_nScrollOffsetHorz;

	SCROLLINFO scrollInfo;
	ZeroMemory(&scrollInfo, sizeof(SCROLLINFO));

    scrollInfo.cbSize = sizeof(SCROLLINFO);
	scrollInfo.fMask = SIF_ALL;

	GetScrollInfo (fnBar, &scrollInfo);

	int nScrollOffsetOld = nScrollOffset;

	switch (nSBCode)
	{
	case SB_LINEUP:
		nScrollOffset = max (0, nScrollOffset - 1);
		break;

	case SB_LINEDOWN:
		nScrollOffset++;
		break;

	case SB_TOP:
		nScrollOffset = scrollInfo.nMin;
		break;

	case SB_BOTTOM:
		nScrollOffset = scrollInfo.nMax;
		break;

	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		nScrollOffset = nPos;
		break;

	case SB_PAGEDOWN:
		nScrollOffset += scrollInfo.nPage - 1;
		break;

	case SB_PAGEUP:
		nScrollOffset -= scrollInfo.nPage - 1;
		break;

	default:
		return FALSE;
	}

	nScrollOffset = min (max (scrollInfo.nMin, nScrollOffset), scrollInfo.nMax - 1);
	nScrollOffset = max (nScrollOffset, 0);

	if (nScrollOffset == nScrollOffsetOld)
	{
		return FALSE;
	}

	if (fnBar == SB_VERT)
	{
		int nActualScroll = nScrollOffset - nScrollOffsetOld;
		bool bDirection = nActualScroll > 0; 

		if (bDirection)
		{
			int nPageSize = scrollInfo.nPage;
			for (int nOffset = m_nTopVisibleOffset, nRows = 0; nOffset != -1; nRows++)
			{
				nOffset = m_strBuffer.Find (g_strEOL, nOffset + 1);
				if (nRows >= nPageSize)
				{
					break;
				}
			}
		}

		// find the next visible offset
		int nNextOffset = m_nTopVisibleOffset;
		for (int iIdxRow = 0; iIdxRow < abs (nActualScroll); iIdxRow++)
		{
			if (bDirection)
			{
				if (nNextOffset >= m_strBuffer.GetLength ())
				{
					nScrollOffset = nScrollOffsetOld;
					return FALSE;
				}
				if (m_strBuffer.GetAt (nNextOffset) == g_chEOL)
				{
					nNextOffset++;
					continue;
				}
				nNextOffset = m_strBuffer.Find (g_strEOL, nNextOffset + 1);
			}
			else
			{
				nNextOffset = ReverseFindOneOf (m_strBuffer, nNextOffset - 2, g_strEOL);
			}
		
			if (nNextOffset == -1)
			{
				nNextOffset = bDirection ? m_strBuffer.GetLength () : 0;
				break;
			}

			if (bDirection)
			{
				nNextOffset++;
			}
		}

		m_nTopVisibleOffset = nNextOffset;
	}
	SetScrollPos (fnBar, nScrollOffset);	

	return TRUE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::OnMouseWheel(UINT /*nFlags*/, short zDelta, CPoint /*pt*/) 
{
	SCROLLINFO scrollInfo;
	ZeroMemory(&scrollInfo, sizeof(SCROLLINFO));

    scrollInfo.cbSize = sizeof(SCROLLINFO);
	scrollInfo.fMask = SIF_ALL;

	GetScrollInfo (SB_VERT, &scrollInfo);

	if (scrollInfo.nPage > (UINT) m_nTotalLines ||
		scrollInfo.nPos >= scrollInfo.nMax)
	{
		return FALSE;
	}

	int nSteps = abs(zDelta) / WHEEL_DELTA;

	for (int i = 0; i < nSteps; i++)
	{
		OnVScroll (zDelta < 0 ? SB_LINEDOWN : SB_LINEUP, 0, NULL);
	}

	return TRUE;
}
//***************************************************************************************
void CBCGPEditCtrl::SetSel (int nStartSel, int nEndSel, BOOL bNoScroll)
{
	ASSERT_VALID (this);

	if (m_strBuffer.IsEmpty ())
	{
		return;
	}

	if (nStartSel == -1)
	{
		m_iStartSel = m_iEndSel = -1;
		RedrawWindow ();
		return;
	}

	if (nStartSel < 0)
	{
		nStartSel = 0;
	}

	if (nEndSel == -1)
	{
		nEndSel = m_strBuffer.GetLength () - 1;
	}

	if (nEndSel >= m_strBuffer.GetLength ())
	{
		nEndSel = m_strBuffer.GetLength () - 1;
	}

	m_iStartSel = nStartSel;
	m_iEndSel = nEndSel;

	if (!bNoScroll)
	{
		SetCaret (nEndSel, TRUE, FALSE);
	}

	RedrawWindow ();
}
//***************************************************************************************
BOOL CBCGPEditCtrl::MakeSelection (enum CBCGPEditCtrl::BCGP_EDIT_SEL_TYPE selType)
{
	if (m_nCurrOffset == m_strBuffer.GetLength () && 
		(selType == ST_NEXT_WORD ||
		 selType == ST_NEXT_SYMBOL ||
		 selType == ST_NEXT_LINE ||
		 selType == ST_NEXT_PAGE))
	{
		OnFailedOperation (g_dwOpMakeSel);
		return FALSE;
	}


	int nOldOffset = m_nCurrOffset;

	if (m_iStartSel == -1)
	{
		m_iStartSel = nOldOffset;
	}

	BOOL bResult = FALSE;
	BOOL bRedrawRestOfText = FALSE;
	BOOL bCaretHidden = FALSE;

	int nOldVertScroll = m_nScrollOffsetVert;
	int nOldHorzScroll = m_nScrollOffsetHorz;

	switch (selType)
	{
	case ST_PREV_WORD:
		bResult = PrevWord (FALSE);
		bRedrawRestOfText = TRUE;
		break;

	case ST_NEXT_WORD:
		bResult = NextWord (FALSE);
		bRedrawRestOfText = TRUE;
		break;

	case ST_PREV_SYMBOL:
		HideCaret ();
		bResult = Left (FALSE);
		bRedrawRestOfText = TRUE;
		bCaretHidden = TRUE;
		break;

	case ST_NEXT_SYMBOL:
		HideCaret ();
		bResult = Right (FALSE);
		bRedrawRestOfText = TRUE;
		bCaretHidden = TRUE;
		break;
	
	case ST_PREV_LINE:
		bResult = Up (TRUE, FALSE);
		bRedrawRestOfText = TRUE;
		break;

	case ST_NEXT_LINE:
		bResult = Down (TRUE, FALSE);
		bRedrawRestOfText = TRUE;
		break;
	
	case ST_PREV_PAGE:
		bResult = PageUp (FALSE);
		break;

	case ST_NEXT_PAGE:
		bResult = PageDown (FALSE);
		break;

	case ST_HOME:
		bResult = Home (FALSE);
		break;

	case ST_END:
		bResult = End (FALSE);
		break;

	case ST_START_OF_TEXT:
		bResult = StartOfText ();
		break;

	case ST_END_OF_TEXT:
		bResult = EndOfText (FALSE);
		break;

	case ST_ALL_TEXT:
		SetCaret (0, TRUE);
		m_iStartSel = m_strBuffer.GetLength ();
		m_iEndSel = 0;
		RedrawWindow ();
		return TRUE;
	}

	if (bResult)
	{
		m_iEndSel = m_nCurrOffset;
		
		if (bRedrawRestOfText && 
			nOldVertScroll == m_nScrollOffsetVert &&
			nOldHorzScroll == m_nScrollOffsetHorz)
		{
			int nMinOffset = min (nOldOffset, min (m_iEndSel, min (m_iStartSel, m_nCurrOffset)));
			int nMaxOffset = max (nOldOffset, max (m_iEndSel, max (m_iStartSel, m_nCurrOffset)));
			RedrawTextOffsets (nMinOffset, nMaxOffset);

			//RedrawRestOfText (min (nOldOffset, min (m_iEndSel, min (m_iStartSel, m_nCurrOffset))));
		}
		else
		{
			RedrawWindow ();
		}
	}
	

	if (bCaretHidden)
	{
		ShowCaret ();
	}

	if (!bResult)
	{
		OnFailedOperation (g_dwOpMakeSel);
	}

	return bResult;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::RemoveSelection (BOOL bSetCaretToSelStart, BOOL bKeepCurrPos,
									 BOOL bRedraw)
{
	if (m_iStartSel != -1 || m_ptStartBlockSel.x != -1)
	{
		if (m_iStartSel > m_iEndSel)
		{
			bSetCaretToSelStart = !bSetCaretToSelStart;
		}

		int nNewOffset = m_iStartSel;
		m_iStartSel = m_iEndSel = -1;
		m_bBlockSelectionMode = FALSE;
		m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
		m_bIsModified = TRUE;
		if (bRedraw)
		{
			RedrawWindow ();
		}

		if (bSetCaretToSelStart && !bKeepCurrPos)
		{
			SetCaret (nNewOffset, TRUE);
		}
		
		return TRUE;
	}
	
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsIndentEnabled (BOOL bForward)
{
	if (m_bBlockSelectionMode)
	{
		return FALSE;
	}

	if (m_iStartSel != m_iEndSel)
	{
		int nStartOffset = min (m_iStartSel, m_iEndSel);
		int nCurOffset = max (m_iStartSel, m_iEndSel);

		if ((nStartOffset < 0) || (nStartOffset >= m_strBuffer.GetLength()))
		{
			return FALSE;
		}

		if (nCurOffset >= m_strBuffer.GetLength())
		{
			if (bForward)
			{
				return nStartOffset != nCurOffset;
			}

			nCurOffset = m_strBuffer.GetLength();
		}
	
		CString strSubNonSelectableChars;

		int ind = m_strNonSelectableChars.GetLength();
		while (ind-- > 0)
		{
			if (m_strIndentChars.Find(m_strNonSelectableChars.GetAt(ind)) == -1)
			{
				strSubNonSelectableChars += m_strNonSelectableChars.GetAt(ind);
			}
		}

		const int ciStartSel = nStartOffset;
		const int ciEndSel = nCurOffset;

		ReverseFindOneOf(strSubNonSelectableChars, ciStartSel, nCurOffset);

		if (bForward)
		{
			if (nCurOffset >= ciStartSel)
			{
				return TRUE;
			}
		}
		else
		{
			const int nNext = 1;
			const int ciEOLOffset = ((nCurOffset >= ciStartSel)? nCurOffset: -1);

			if (ciEOLOffset > ciStartSel)
			{
				if (ciEOLOffset + nNext != ciEndSel &&
					m_strIndentChars.Find(m_strBuffer.GetAt(ciEOLOffset + nNext)) != -1)
				{
					return TRUE;
				}

				do
				{
					do
					{
						ReverseFindOneOf(strSubNonSelectableChars, ciStartSel, nCurOffset);
					}
					while (nCurOffset >= ciStartSel &&
						   m_strIndentChars.Find(m_strBuffer.GetAt(nCurOffset + nNext)) == -1);

					if (nCurOffset >= ciStartSel &&
						m_strIndentChars.Find(m_strBuffer.GetAt(nCurOffset + nNext)) != -1)
					{
						return TRUE;
					}
					else if (nCurOffset < ciStartSel)
					{
						ReverseFindOneOf(strSubNonSelectableChars, 0, nCurOffset = ciStartSel);

						if (m_strIndentChars.Find(m_strBuffer.GetAt(nCurOffset + nNext)) != -1)
						{
							return TRUE;
						}
					}
				}
				while (nCurOffset >= ciStartSel);
			}
			else if (ciEOLOffset == -1)
			{
				if (ciEndSel == m_strBuffer.GetLength())
				{
					ReverseFindOneOf(strSubNonSelectableChars, 0, nStartOffset);

					if (m_strIndentChars.Find(m_strBuffer.GetAt(nStartOffset + nNext)) != -1)
					{
						return TRUE;
					}
				}
			}
			else if (ciEOLOffset + nNext != ciEndSel &&
					 m_strIndentChars.Find(m_strBuffer.GetAt(nCurOffset + nNext)) != -1)
			{
				return TRUE;
			}
			else
			{
				ReverseFindOneOf(strSubNonSelectableChars, 0, nCurOffset);

				if (m_strIndentChars.Find(m_strBuffer.GetAt(nCurOffset + nNext)) != -1)
				{
					return TRUE;
				}
			}
		}
	}

	return FALSE;	
}

//**************************************************************************************
BOOL CBCGPEditCtrl::IndentSelection (BOOL bForward)
{
	if (!IsIndentEnabled(bForward))
	{
		OnFailedOperation (g_dwOpIndent);
		return FALSE;
	}

	int iStartSel = min (m_iStartSel, m_iEndSel);
	int iEndSel = max (m_iStartSel, m_iEndSel);

	int nEndRowOffset = GetRowStartByOffset (iEndSel);

	if (nEndRowOffset == iEndSel)
	{
		iEndSel--;
	}

	BCGP_EDIT_UNDO_INDENT_DATA* pData = new BCGP_EDIT_UNDO_INDENT_DATA;

	if (IndentText (iStartSel, iEndSel, bForward, m_bBlockSelectionMode, TRUE, 
						pData->m_arInsDelStrings, FALSE))
	{
		// indent text changes the selection - recalculate endsel
		iEndSel = m_iEndSel; 
		int nEndRowOffset = GetRowStartByOffset (iEndSel);

		if (nEndRowOffset == iEndSel)
		{
			iEndSel--;
		}

		pData->m_nStartOffset = m_iStartSel;
		pData->m_nEndOffset = iEndSel;
		pData->m_bForward = bForward;
		
		SetLastUndoReason (bForward ? g_dwUATIndent : g_dwUATUnindent);

		AddUndoAction (pData, g_dwUATIndentData);
		return TRUE;
	}
	else
	{
		delete pData;
	}

	OnFailedOperation (g_dwOpIndent);
	return FALSE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IndentText (int nStartOffset, int nEndOffset, BOOL bForward, 
								BOOL bBlockSelectionMode, BOOL bSelectText, 
								CStringArray& arInsDelText, BOOL bUndoMode)
{
	if (bBlockSelectionMode)
	{
		return FALSE;
	}
	else
	{
		CString strTab;
		BuildTabString (strTab);

		int nTabStringLen = strTab.GetLength ();

		int nFirstSelRow = GetRowStartByOffset	(nStartOffset);
		int nLastSelRow  = max (nEndOffset, GetRowEndByOffset (nEndOffset));

		int i = 0;
		for (int iIdx = nFirstSelRow; iIdx < nLastSelRow; iIdx++, i++)
		{
			if (bForward)
			{
				int nTextStart = GetRowTextStart (iIdx);
				if (!bUndoMode)
				{
					OnBeforeTextInserted (strTab, nTextStart);
					m_strBuffer.Insert (nTextStart, strTab);
					OnAfterTextChanged (nTextStart, strTab, TRUE);
					nLastSelRow += nTabStringLen;
					arInsDelText.SetAtGrow (i, strTab);

					if (m_nTopVisibleOffset > iIdx)
					{
						m_nTopVisibleOffset += nTabStringLen;
					}
				}
				else
				{
					if (i < arInsDelText.GetSize ())
					{
						CString& str = arInsDelText [i];

						OnBeforeTextInserted (str, nTextStart);
						m_strBuffer.Insert (nTextStart, str);
						OnAfterTextChanged (nTextStart, str, TRUE);

						nLastSelRow += str.GetLength ();
						if (m_nTopVisibleOffset > iIdx)
						{
							m_nTopVisibleOffset += str.GetLength ();
						}
					}
				}
			}
			else
			{
				int nTextStart = GetRowTextStart (iIdx);
				int nRowStart = GetRowStartByOffset (nTextStart);

				int nCharsToDelete = m_nIndentSize;

				CString strInsDelText;
				
				if (m_bKeepTabs)
				{
					while (nCharsToDelete > 0 && 
						   nTextStart - nRowStart > 0)
					{
						TCHAR ch = m_strBuffer.GetAt (nTextStart - 1);

						if (ch == _T ('\t'))  
						{
							int nColumn = GetColumnFromOffset (nTextStart - 1);
							nCharsToDelete -= m_nTabSize - nColumn % m_nTabSize;  
						}
						else
						{
							nCharsToDelete--;
						}

						if (nCharsToDelete < 0)
						{
							break;
						}
						OnBeforeTextDeleted (nTextStart - 1, CString (ch));
						m_strBuffer.Delete (nTextStart - 1, 1);
						OnAfterTextChanged (nTextStart - 1, CString (ch), FALSE);
						strInsDelText.Insert (0, CString (ch));

						nLastSelRow--;
						nTextStart = GetRowTextStart (iIdx);
						if (m_nTopVisibleOffset > iIdx)
						{
							m_nTopVisibleOffset--;
						}
					}
				} 
				else
				{
					if (nTextStart - nRowStart < m_nIndentSize)
					{
						nCharsToDelete = nTextStart - nRowStart;
					}
					if (nCharsToDelete > 0)
					{
						strInsDelText = m_strBuffer.Mid (nTextStart - nCharsToDelete, nCharsToDelete);
						OnBeforeTextDeleted (nTextStart - nCharsToDelete, strInsDelText);
						m_strBuffer.Delete (nTextStart - nCharsToDelete, nCharsToDelete);
						OnAfterTextChanged (nTextStart - nCharsToDelete, strInsDelText, FALSE);
						nLastSelRow -= nCharsToDelete;
						if (m_nTopVisibleOffset > iIdx)
						{
							m_nTopVisibleOffset -= nCharsToDelete;
						}
					}
				}
				
				arInsDelText.SetAtGrow (i, strInsDelText);				
			}

			iIdx = m_strBuffer.Find (g_strEOL, iIdx);
			if (iIdx == -1)
			{
				break;
			}
		}

		int nNewCaretPos = -1; 
		if (bSelectText)
		{
			if (m_iEndSel < m_iStartSel)
			{
				nNewCaretPos = nFirstSelRow;
				m_iStartSel = nLastSelRow + 1;
				m_iEndSel = nFirstSelRow;
			}
			else
			{
				nNewCaretPos = nLastSelRow + 1;
				m_iStartSel = nFirstSelRow;
				m_iEndSel = nLastSelRow + 1;
			}
		}

		UpdateScrollBars ();

		if (nFirstSelRow < m_nTopVisibleOffset)
		{
			CPoint pt, ptRowColumn;
			OffsetToPoint (nFirstSelRow, pt, &ptRowColumn);
			OnScroll (SB_VERT, SB_THUMBPOSITION, ptRowColumn.y);
			if (bSelectText)
			{
				SetCaret (nNewCaretPos);
			}
			RedrawWindow ();
		}
		else
		{
			if (bSelectText)
			{
				SetCaret (nNewCaretPos);
			}
			RedrawRestOfText (nFirstSelRow);
		}
	}
	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::SetCaretToLastMaxColumn (int nOffset, BOOL bRedrawOnScroll)
{
	CPoint point;
	CPoint ptRowColumn (-1, -1);
	OffsetToPoint (nOffset, point, &ptRowColumn);

	if (ptRowColumn.x < m_nLastMaxColumn)
	{
		int nRowEndOffset = 0;
		int nRowLen = GetNumOfColumnsInRowByOffset (nOffset, nRowEndOffset);
		if (nRowLen >= m_nLastMaxColumn)
		{
			int nNewOffset = GetOffsetOfColumnInRow (m_nLastMaxColumn, nRowEndOffset);
			if(nNewOffset >= nOffset)
			{
				SetCaret (nNewOffset, TRUE, bRedrawOnScroll);
			}
			else
			{
				SetCaret (nOffset);
			}
		}
		else
		{
			SetCaret (nRowEndOffset, TRUE, bRedrawOnScroll);
		}
		return TRUE;
	}

	return FALSE;
}
//**************************************************************************************
void CBCGPEditCtrl::AddUndoAction (BCGP_EDIT_UNDO_ACTION* pUndoAction)
{
	pUndoAction->m_dwActionType |= GetLastUndoReason ();
	pUndoAction->m_nCursorOffset = m_iEndSel != -1 ? m_iEndSel : m_nCurrOffset;

	if (!OnBeforeAddUndoAction (pUndoAction))
	{
		return;
	}	

	if (!m_bEOL)
	{
		POSITION pos = m_lstUndoActions.GetTailPosition ();
		while (pos != m_posUndoBuffer)
		{
			POSITION posSave = pos;	
			BCGP_EDIT_UNDO_ACTION* pUndoAction = 
				(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.GetPrev (pos);
			m_lstUndoActions.RemoveAt (posSave);
			OnRemoveUndoAction (pUndoAction);
			delete pUndoAction;
		}
	}

	m_lstUndoActions.AddTail (pUndoAction);
	m_bEOL = TRUE;
	m_bBOL = FALSE;

	m_posUndoBuffer = NULL;
	if (m_lstUndoActions.GetCount () > m_nUndoBufferSize)
	{
		BCGP_EDIT_UNDO_ACTION* pUndoAction = 
			(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.RemoveHead ();
		OnRemoveUndoAction (pUndoAction);
		delete pUndoAction;
	}

	m_bIsModified = TRUE;
}
//**************************************************************************************
void CBCGPEditCtrl::AddUndoAction (CString& strText, DWORD dwAction, int nCurrOffset, 
								   BOOL bForceInsertMode, BOOL bForceNextUndo)
{
	BCGP_EDIT_UNDO_ACTION* pUndoAction = new BCGP_EDIT_UNDO_ACTION;
	pUndoAction->m_dwActionType = dwAction;
	pUndoAction->m_strActionText = strText;
	pUndoAction->m_nActionOffset = nCurrOffset;
	pUndoAction->m_bForceNextUndo = bForceNextUndo;

	pUndoAction->m_dwFlags |= (GetOverrideMode () && !bForceInsertMode) ? g_dwUAFOvr : g_dwUAFIns;
	AddUndoAction (pUndoAction);
}
//**************************************************************************************
void CBCGPEditCtrl::AddUndoAction (TCHAR ch, DWORD dwAction, int nCurrOffset, 
								   BOOL bForseInsertMode)
{
	TCHAR chCurrent = _T ('\0');

	if (!m_strBuffer.IsEmpty () &&  nCurrOffset < m_strBuffer.GetLength ())
	{
		chCurrent = m_strBuffer.GetAt (nCurrOffset);
	}

	if (ch == g_chEOL)
	{
		CString str (ch);
		AddUndoAction (str, dwAction, nCurrOffset, TRUE);
		return;
	}

	CString str (GetOverrideMode () ? chCurrent : ch);

	if (!m_bUndoCharMode && dwAction == g_dwUATInsertData && !GetOverrideMode ())
	{
		if ((m_nPrevKey == _T (' ') || m_nPrevKey > VK_HELP) && 
			(m_strWordDelimeters.Find (ch) == -1 && 
			 m_strWordDelimeters.Find ((TCHAR) m_nPrevKey) == -1 ||
			 m_strWordDelimeters.Find (ch) != -1 && 
			 m_strWordDelimeters.Find ((TCHAR) m_nPrevKey) != -1) &&
			 !m_lstUndoActions.IsEmpty () &&
			 m_bEOL && m_posUndoBuffer == NULL)
		{
			BCGP_EDIT_UNDO_ACTION* pUndoAction = m_lstUndoActions.GetTail ();
			if (pUndoAction->m_dwActionType & g_dwUATInsertData)
			{
				pUndoAction->m_strActionText += str;
			}
			else
			{
				AddUndoAction (str, dwAction, nCurrOffset, FALSE);
			}
		}
		else
		{
			AddUndoAction (str, dwAction, nCurrOffset, FALSE);
		}
	}
	else
	{
		AddUndoAction (str, dwAction, nCurrOffset, bForseInsertMode);
	}
}
//**************************************************************************************
void CBCGPEditCtrl::AddUndoAction (BCGP_BASE_UNDO_DATA* lpActionData, DWORD dwAction, 
								   BOOL bDestroyData)
{
	BCGP_EDIT_UNDO_ACTION* pUndoAction = new BCGP_EDIT_UNDO_ACTION;
	pUndoAction->m_lpActionData = lpActionData;
	pUndoAction->m_dwActionType = dwAction;
	pUndoAction->m_bDestroyActionData = bDestroyData;
	
	AddUndoAction (pUndoAction);
}
//**************************************************************************************
BOOL CBCGPEditCtrl::OnUndo (BOOL bSetCaret, BOOL bRedraw)
{
	if (m_lstUndoActions.IsEmpty () || m_bBOL)
	{
		OnFailedOperation (g_dwOpUndo);
		return FALSE;
	}

	if (m_bIntelliSenseMode && m_pIntelliSenseWnd != NULL)
	{
		m_pIntelliSenseWnd->PostMessage (WM_CLOSE);
	}

	if (m_posUndoBuffer == NULL)
	{
		m_posUndoBuffer = m_lstUndoActions.GetTailPosition ();
		m_bEOL = FALSE;
	}
	else if (!m_bIsLastActionUndo)
	{
		m_lstUndoActions.GetPrev (m_posUndoBuffer);
		ASSERT (m_posUndoBuffer != NULL);
	}

	POSITION posCurrent = m_posUndoBuffer;
	BCGP_EDIT_UNDO_ACTION* pUndoAction = 
		(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.GetPrev (m_posUndoBuffer);
	
	ASSERT (pUndoAction != NULL);

	if (m_posUndoBuffer == NULL)
	{
		m_bBOL = TRUE;
	}

	m_bIsLastActionUndo = TRUE;

	BOOL bResult = ProcessUndoRedoAction (pUndoAction, TRUE, bSetCaret);
	if (bResult)
	{
		m_lstUndoActions.SetAt (posCurrent, pUndoAction);
		
		if (m_posUndoBuffer != NULL)
		{
			BCGP_EDIT_UNDO_ACTION* pUndoAction = 
			(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.GetAt (m_posUndoBuffer);
			if (pUndoAction->m_bForceNextUndo)
			{
				bResult = OnUndo (TRUE, FALSE);
			}
		}
	}

	if (m_posUndoBuffer == NULL && m_bBOL)
	{
		m_bIsModified = FALSE;
	}

	if (bRedraw)
	{
		RedrawWindow ();
	}

	return bResult;
}
//**************************************************************************************
void CBCGPEditCtrl::GetUndoActions(CDWordArray& dwaUAT) const
{
	dwaUAT.RemoveAll();

	POSITION nUndoBufferPos = m_posUndoBuffer;
	BOOL bBOL = m_bBOL;
	BOOL bEOL = m_bEOL;
	BOOL bIsLastActionUndo = m_bIsLastActionUndo;
	CList <BCGP_EDIT_UNDO_ACTION*, BCGP_EDIT_UNDO_ACTION*> lstUndoActions;
	BCGP_EDIT_UNDO_ACTION* pNewUndoAction = NULL;
	
	lstUndoActions.AddTail(
		(CList<BCGP_EDIT_UNDO_ACTION*, BCGP_EDIT_UNDO_ACTION*>*) &m_lstUndoActions);

	if (!lstUndoActions.IsEmpty ())
	{
		while (!bBOL)
		{
			if (nUndoBufferPos == NULL)
			{
				nUndoBufferPos = lstUndoActions.GetTailPosition ();
				bEOL = FALSE;
			}
			else if (!bIsLastActionUndo)
			{
				lstUndoActions.GetPrev (nUndoBufferPos);
				ASSERT (nUndoBufferPos != NULL);
			}

			BCGP_EDIT_UNDO_ACTION* pUndoAction = 
				(BCGP_EDIT_UNDO_ACTION*) lstUndoActions.GetPrev (nUndoBufferPos);
			
			if (nUndoBufferPos == NULL)
			{
				bBOL = TRUE;
			}

			bIsLastActionUndo = TRUE;

			if (pUndoAction != NULL &&
				!pUndoAction->m_bForceNextUndo)
			{
				dwaUAT.Add(pUndoAction->m_dwActionType);
			}
		}
	}

	if (pNewUndoAction != NULL)
	{
		delete pNewUndoAction;
	}
}
//**************************************************************************************
BOOL CBCGPEditCtrl::OnRedo (BOOL bSetCaret, BOOL bRedraw)
{
	if (m_lstUndoActions.IsEmpty () || m_bEOL)
	{
		OnFailedOperation (g_dwOpRedo);
		return FALSE;
	}

	if (m_bIntelliSenseMode && m_pIntelliSenseWnd != NULL)
	{
		m_pIntelliSenseWnd->PostMessage (WM_CLOSE);
	}

	if (m_posUndoBuffer == NULL)
	{
		m_posUndoBuffer = m_lstUndoActions.GetHeadPosition ();
		m_bBOL = FALSE;
	}
	else if (m_bIsLastActionUndo)
	{
		m_lstUndoActions.GetNext (m_posUndoBuffer);
		ASSERT (m_posUndoBuffer != NULL);
	}

	POSITION posCurr = m_posUndoBuffer;

	BCGP_EDIT_UNDO_ACTION* pUndoAction = 
		(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.GetNext (m_posUndoBuffer);
	ASSERT (pUndoAction != NULL);
	
	if (m_posUndoBuffer == NULL)
	{
		m_bEOL = TRUE;
	}

	BOOL bForceNextUndo = pUndoAction->m_bForceNextUndo;

	m_bIsLastActionUndo = FALSE;

	BOOL bResult = ProcessUndoRedoAction (pUndoAction, FALSE, bSetCaret);
	if (bResult)
	{
		m_lstUndoActions.SetAt (posCurr, pUndoAction);
		
		if (bForceNextUndo)
		{
			bResult = (pUndoAction->m_dwActionType & g_dwUATDragDrop) ? OnRedo (FALSE, FALSE) :
															  OnRedo (TRUE, FALSE);
		}
	}

	if (bRedraw)
	{
		RedrawWindow ();
	}

	return bResult;
}
//**************************************************************************************
void CBCGPEditCtrl::GetRedoActions(CDWordArray& dwaUAT) const
{
	dwaUAT.RemoveAll();

	if (m_lstUndoActions.IsEmpty ())
	{
		return;
	}

	POSITION nUndoBufferPos = m_posUndoBuffer;
	BOOL bIsLastActionUndo = m_bIsLastActionUndo;
	BOOL bEOL = m_bEOL;

	while (!bEOL)
	{
		if (nUndoBufferPos == NULL)
		{
			nUndoBufferPos = m_lstUndoActions.GetHeadPosition ();
		}
		else if (bIsLastActionUndo)
		{
			m_lstUndoActions.GetNext (nUndoBufferPos);
			ASSERT (nUndoBufferPos != NULL);
		}

		BCGP_EDIT_UNDO_ACTION* pRedoAction = 
			(BCGP_EDIT_UNDO_ACTION*) m_lstUndoActions.GetNext (nUndoBufferPos);
		ASSERT (pRedoAction != NULL);
		
		if (nUndoBufferPos == NULL)
		{
			bEOL = TRUE;
		}

		bIsLastActionUndo = FALSE;

		if (pRedoAction != NULL &&
			!pRedoAction->m_bForceNextUndo)
		{
			dwaUAT.Add(pRedoAction->m_dwActionType);
		}
	}	
}
//**************************************************************************************
BOOL CBCGPEditCtrl::ProcessUndoRedoAction (BCGP_EDIT_UNDO_ACTION* pUndoAction, BOOL bUndo, 
										   BOOL bSetCaret)
{
	ASSERT (pUndoAction != NULL);
	
	int nDataLen = pUndoAction->m_strActionText.GetLength ();

	if (pUndoAction->m_dwActionType & g_dwUATInsertData && bUndo ||
		pUndoAction->m_dwActionType & g_dwUATDeleteData && !bUndo)
	{
		if ((pUndoAction->m_dwFlags & g_dwUAFOvr) && bUndo)
		{
			CString strChar = pUndoAction->m_strActionText [0];
			OnBeforeTextInserted (strChar, pUndoAction->m_nActionOffset);
			TCHAR chTmp = m_strBuffer.GetAt (pUndoAction->m_nActionOffset);
			m_strBuffer.SetAt (pUndoAction->m_nActionOffset, 
									pUndoAction->m_strActionText [0]);
			pUndoAction->m_strActionText = CString (chTmp);
			OnAfterTextChanged (pUndoAction->m_nActionOffset, strChar, TRUE);
		}
		else 
		{
			OnBeforeTextDeleted (pUndoAction->m_nActionOffset, pUndoAction->m_strActionText);
			OnDeleteTextFromBuffer (pUndoAction->m_nActionOffset, 
								pUndoAction->m_nActionOffset + nDataLen, 
								pUndoAction->m_strActionText);
			m_strBuffer.Delete (pUndoAction->m_nActionOffset, nDataLen);
			OnAfterTextChanged (pUndoAction->m_nActionOffset, pUndoAction->m_strActionText, FALSE);
		}
		m_nCurrOffset = pUndoAction->m_nCursorOffset;

		if (bSetCaret)
		{
			SetCaret (m_nCurrOffset);
		}
	}
	else if (pUndoAction->m_dwActionType & g_dwUATInsertData && !bUndo ||
			pUndoAction->m_dwActionType & g_dwUATDeleteData && bUndo)
	{
		if ((pUndoAction->m_dwFlags & g_dwUAFOvr) && !bUndo)
		{
			CString strChar = pUndoAction->m_strActionText [0];
			OnBeforeTextInserted (strChar, pUndoAction->m_nActionOffset);

			TCHAR chTmp = m_strBuffer.GetAt (pUndoAction->m_nActionOffset);
			m_strBuffer.SetAt (pUndoAction->m_nActionOffset, 
									pUndoAction->m_strActionText [0]);
			OnAfterTextChanged (pUndoAction->m_nActionOffset, strChar, TRUE);
			pUndoAction->m_strActionText = CString (chTmp);
		}
		else
		{
			OnBeforeTextInserted (pUndoAction->m_strActionText, pUndoAction->m_nActionOffset);
			OnInsertTextToBuffer (pUndoAction->m_nActionOffset, pUndoAction->m_strActionText);
			m_strBuffer.Insert (pUndoAction->m_nActionOffset, pUndoAction->m_strActionText);
			OnAfterTextChanged (pUndoAction->m_nActionOffset, pUndoAction->m_strActionText, TRUE);
		}
		m_nCurrOffset = pUndoAction->m_nActionOffset + nDataLen;
		if (bSetCaret)
		{
			SetCaret (m_nCurrOffset);
		}
	}
	else if (pUndoAction->m_dwActionType & g_dwUATIndentData)
	{
		BCGP_EDIT_UNDO_INDENT_DATA* pData = 
			(BCGP_EDIT_UNDO_INDENT_DATA*) pUndoAction->m_lpActionData;
		if (pData != NULL)
		{
			BOOL bForward = bUndo && !pData->m_bForward ||
							!bUndo && pData->m_bForward;

			int nStart = min (pData->m_nStartOffset, pData->m_nEndOffset);
			int nEnd = max (pData->m_nStartOffset, pData->m_nEndOffset);

			IndentText (nStart, nEnd, bForward, FALSE, FALSE, pData->m_arInsDelStrings, TRUE);
			int nRowTextStart = GetRowStartByOffset (nStart);
			nRowTextStart = GetRowTextStart (nRowTextStart);
			if (bSetCaret)						
			{
				SetCaret (pUndoAction->m_nCursorOffset);
			}
		}
		else
		{
			ASSERT (FALSE);
		}
	}
	

	m_iStartSel = m_iEndSel = -1;
	m_bBlockSelectionMode = FALSE;
	m_ptStartBlockSel = m_ptEndBlockSel = CPoint (-1, -1);
	
	UpdateScrollBars ();

	m_nScrollOffsetVert = RowFromOffset (m_nTopVisibleOffset, TRUE);
	SetScrollPos (SB_VERT, m_nScrollOffsetVert);
	
	m_bIsModified = TRUE;

	return TRUE;
}
//**************************************************************************************
BOOL CBCGPEditCtrl::CanUndo () const 
{
	return m_posUndoBuffer != NULL || m_bEOL && !m_lstUndoActions.IsEmpty ();
}
//**************************************************************************************
BOOL CBCGPEditCtrl::CanRedo () const 
{
	return m_posUndoBuffer != NULL || m_bBOL && !m_lstUndoActions.IsEmpty ();;
}
//**************************************************************************************
void CBCGPEditCtrl::SetWordColor (CString strWord, COLORREF clrFrgnd, COLORREF clrBkgnd, 
								  BOOL bCaseSensitive)
{
	BCGP_EDIT_SYNTAX_COLOR syntaxColor (clrFrgnd, clrBkgnd, bCaseSensitive);
	if (!bCaseSensitive)
	{
		strWord.MakeUpper ();
	}
	m_mapWordColors.SetAt (strWord, syntaxColor);
}
//**************************************************************************************
BOOL CBCGPEditCtrl::RemoveWordFromColorTable (CString strWord)
{
	return m_mapWordColors.RemoveKey (strWord);
}
//**************************************************************************************
void CBCGPEditCtrl::RemoveAllWordsFromColorTable ()
{
	m_mapWordColors.RemoveAll ();
}
//**************************************************************************************
BOOL CBCGPEditCtrl::IsOffsetAtColorBlock (int nOffset)
{
	int nRangeStart = max (0, nOffset - _MAX_COLOR_BLOCK_STR_LEN);
	int nRangeEnd = min (m_strBuffer.GetLength (), nOffset + _MAX_COLOR_BLOCK_STR_LEN);

	CString strRange = m_strBuffer.Mid (nRangeStart, nRangeEnd - nRangeStart);
	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
		if (!colorBlock.m_strOpen.IsEmpty () &&
			 strRange.Find (colorBlock.m_strOpen) != -1 ||
			!colorBlock.m_strClose.IsEmpty () && 
			strRange.Find (colorBlock.m_strClose) != -1)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//**************************************************************************************
POSITION CBCGPEditCtrl::SetBlockColor (CString strBlockOpen, CString strBlockClose, 
									 BOOL bWholeText,
									COLORREF clrFrgnd, COLORREF clrBkgnd)
{
	if (strBlockOpen.GetLength () > _MAX_COLOR_BLOCK_STR_LEN ||
		strBlockClose.GetLength () > _MAX_COLOR_BLOCK_STR_LEN)
	{
		ASSERT (FALSE);
		TRACE0("Open or close block length must not exceed two characters length!\n");
		return NULL;
	}

	if (bWholeText && 
		(strBlockOpen.GetLength () == 0 || strBlockClose.GetLength () == 0))
	{
		ASSERT (FALSE);
		TRACE0("Open or close block with bWholeText = TRUE must not be empty!\n");
		return NULL;
	}

	if (bWholeText)
	{
		for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
		{
			BCGP_EDIT_COLOR_BLOCK colorBlock = m_lstColorBlocks.GetNext (pos);
			if (colorBlock.m_bWholeText)
			{
				ASSERT (FALSE);
				TRACE0("Can't open the second color block with bWholeText = TRUE!\n");
				return NULL;
			}
		}
	}

	BCGP_EDIT_COLOR_BLOCK colorBlock;
	colorBlock.m_strOpen = strBlockOpen;
	colorBlock.m_strClose = strBlockClose;
	colorBlock.m_clrForeground = clrFrgnd;
	colorBlock.m_clrBackground = clrBkgnd;
	colorBlock.m_bWholeText = bWholeText;

	POSITION posSave = NULL;
	for (POSITION pos = m_lstColorBlocks.GetHeadPosition (); pos != NULL;)
	{
		posSave = pos;
		BCGP_EDIT_COLOR_BLOCK colorBlockNext = m_lstColorBlocks.GetNext (pos);

		if (colorBlockNext.m_strOpen.GetLength () <= colorBlock.m_strOpen.GetLength ())
		{
			return m_lstColorBlocks.InsertBefore (posSave, colorBlock);
		}
	}		

	return m_lstColorBlocks.AddTail (colorBlock);
	
}
//**************************************************************************************
void CBCGPEditCtrl::OnSysColorChange() 
{
	CWnd::OnSysColorChange();

	InitColors ();
	RedrawWindow ();
}
//**************************************************************************************
BOOL CBCGPEditCtrl::RemoveBlockColor (POSITION posBlock)
{
	m_lstColorBlocks.RemoveAt (posBlock);
	return TRUE;
}
//**************************************************************************************
void CBCGPEditCtrl::RemoveAllBlocks ()
{
	m_lstColorBlocks.RemoveAll();
}
//************************************************************************************
int CBCGPEditCtrl::ReverseFindOneOf (const CString& strBuffer, int nPos, const CString& strChars) const
{
	if (nPos < 0 || nPos >= strBuffer.GetLength ())
	{
		return -1;
	}

	for (int i = nPos; i >= 0; i--)
	{
		TCHAR chNext = strBuffer.GetAt (i);
		if (strChars.Find (chNext) != -1)
		{
			return i + 1;
		}
	}

	return -1;
}
//************************************************************************************
void CBCGPEditCtrl::ReverseFindOneOf(const CString& strCharSet, const ciStartPos, int& nCurrOffset) const
{
	do
	{
		nCurrOffset--;
	}
	while (nCurrOffset >= ciStartPos &&
		   strCharSet.Find(m_strBuffer.GetAt(nCurrOffset)) == -1);
}
//************************************************************************************
int CBCGPEditCtrl::FindOneOf (const CString& strBuffer, int nPos, const CString& strChars) const
{
	int nBufLen = strBuffer.GetLength ();
	if (nPos < 0 || nPos >= nBufLen)
	{
		return -1;
	}

	for (int i = nPos; i < nBufLen; i++)
	{
		TCHAR chNext = strBuffer.GetAt (i);
		if (strChars.Find (chNext) != -1)
		{
			return i;
		}
	}

	return -1;
}
//************************************************************************************
int CBCGPEditCtrl::WhileOneOf (const CString& strBuffer, int nOffset, bool bForward, 
							   const CString& strChars)
{
	int nResOffset = nOffset;

	while (nResOffset >= 0 && nResOffset < strBuffer.GetLength ())
	{
		TCHAR ch = strBuffer.GetAt (nResOffset);
		if (strChars.Find (ch) == -1)
		{
			break;
		}

		bForward ? nResOffset++ : nResOffset--;
	}

	if (!bForward) 
	{
		nResOffset++;
	}
		
	return nResOffset;
}
//************************************************************************************
int CBCGPEditCtrl::RowFromOffset (int nOffset, BOOL bCalcAll) const
{
	int nRow = 0; 
	int iIdx = 0;

	if (nOffset >= m_nTopVisibleOffset && !bCalcAll)
	{
		nRow = m_nScrollOffsetVert;
		iIdx = m_nTopVisibleOffset;
	}
	
	for (;  ; nRow++)
	{
		iIdx = m_strBuffer.Find (g_strEOL, iIdx);
		if (iIdx >= nOffset || iIdx < 0)
		{
			break;
		}
		iIdx ++;
	}

	return nRow;
}
//************************************************************************************
int CBCGPEditCtrl::GetOffsetOfColumnInRow (int nColumn, int nRowEndOffset, BOOL bEndOffsetSpecified)
{
	int nStartRowOffset = GetRowStartByOffset (nRowEndOffset);

	if (!m_bKeepTabs)
	{
		return nStartRowOffset + nColumn;
	}

	if (!bEndOffsetSpecified)
	{
		nRowEndOffset = GetRowEndByOffset (nRowEndOffset);
	}

	int nCurrColumn = 0;
	for (int iOffset = nStartRowOffset; iOffset < nRowEndOffset; iOffset++)
	{
		TCHAR chNext = m_strBuffer.GetAt (iOffset);
		if (chNext == _T ('\t'))
		{
			int nRestOfTab = m_nTabSize - nCurrColumn % m_nTabSize;
			nCurrColumn += nRestOfTab; 
		}
		else
		{
			nCurrColumn++;
		}

		if (nCurrColumn > nColumn)
		{
			break;
		}
	}

	return iOffset;
}
//************************************************************************************
BOOL CBCGPEditCtrl::IsPointInSelectionRect (CPoint pt) 
{
	if (m_iStartSel >= 0 && m_iEndSel >= 0)
	{
		CRect rectSel1, rectSel2, rectSel3;
		GetSelectionRect (rectSel1, rectSel2, rectSel3);
		
		if (rectSel1.PtInRect (pt) || rectSel2.PtInRect (pt) || 
			rectSel3.PtInRect (pt))
		{
			return TRUE;
		}
	}

	return FALSE;
}

//*************************************************************************************
// Drag & Drop processing
//*************************************************************************************
//***************************************************************************************
BOOL CBCGPEditCtrlDropTarget::Register (CBCGPEditCtrl* pOwner)
{
	m_pOwner = pOwner;
	return COleDropTarget::Register (pOwner);
}
//***************************************************************************************
DROPEFFECT CBCGPEditCtrlDropTarget::OnDragEnter(CWnd* /*pWnd*/, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	ASSERT (m_pOwner != NULL);

	if (!pDataObject->IsDataAvailable (CF_TEXT) && 
		!pDataObject->IsDataAvailable (g_cFormat))
	{
		return DROPEFFECT_NONE;
	}

	return m_pOwner->OnDragEnter(pDataObject, dwKeyState, point);
}
//***************************************************************************************
void CBCGPEditCtrlDropTarget::OnDragLeave(CWnd* /*pWnd*/) 
{
	ASSERT (m_pOwner != NULL);
	m_pOwner->OnDragLeave ();
}
//***************************************************************************************
DROPEFFECT CBCGPEditCtrlDropTarget::OnDragOver(CWnd* /*pWnd*/, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	ASSERT (m_pOwner != NULL);

	if (!pDataObject->IsDataAvailable (CF_TEXT) && 
		!pDataObject->IsDataAvailable (g_cFormat))
	{
		return DROPEFFECT_NONE;
	}

	return m_pOwner->OnDragOver(pDataObject, dwKeyState, point);
}
//***************************************************************************************
DROPEFFECT CBCGPEditCtrlDropTarget::OnDropEx(CWnd* /*pWnd*/, 
							COleDataObject* pDataObject, 
							DROPEFFECT dropEffect, 
							DROPEFFECT /*dropList*/, CPoint point) 
{
	ASSERT (m_pOwner != NULL);

	if (!pDataObject->IsDataAvailable (CF_TEXT) && 
		!pDataObject->IsDataAvailable (g_cFormat))
	{
		return DROPEFFECT_NONE;
	}

	return m_pOwner->OnDrop(pDataObject, dropEffect, point) ?
			dropEffect : DROPEFFECT_NONE;
}
//***************************************************************************************
BOOL CBCGPEditCtrl::OnDrop (COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	OnDragLeave ();

	BOOL bTextFromat = pDataObject->IsDataAvailable (CF_TEXT);
	BOOL bBlockFormat = pDataObject->IsDataAvailable (g_cFormat);
	if (!bTextFromat && !bBlockFormat)
	{
		return FALSE;
	}

	CRect rectSel (m_ptStartBlockSel, m_ptEndBlockSel);
	rectSel.NormalizeRect ();
	VirtToClient (rectSel);

	int	nOffset = HitTest (point, FALSE, TRUE);
	if (m_iStartSel >= 0 && 
		nOffset >= min (m_iStartSel, m_iEndSel) && 
		nOffset <= max (m_iStartSel, m_iEndSel) && 
		!bBlockFormat ||
		bBlockFormat && m_bBlockSelectionMode && rectSel.PtInRect (point) ||
		nOffset == -1)
	{
		OnFailedOperation (g_dwOpDragDrop);
		return FALSE;
	}

	SetLastUndoReason (g_dwUATDragDrop);
	PasteFromDataObject (pDataObject, nOffset, !m_bDragTextMode, FALSE, TRUE, 
						 dropEffect == DROPEFFECT_MOVE && nOffset >= 0);

	// the drag drop was initiated by another application
	if (!m_bDragTextMode)
	{
		m_nSavedOffset = m_iEndSel;
	}

	m_nDropOffset = nOffset >= 0 ? nOffset : 0;

	return TRUE;
}
//***************************************************************************************
DROPEFFECT CBCGPEditCtrl::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	if (m_bReadOnly)
	{
		return DROPEFFECT_NONE;
	}

	::DestroyCaret ();
	CreateGrayCaret (2, m_nLineHeight);
	ShowCaret ();
	
	return OnDragOver (pDataObject, dwKeyState, point);
}
//***************************************************************************************
void CBCGPEditCtrl::OnDragLeave()	
{
	if (!m_bReadOnly)
	{
		::DestroyCaret ();
		CSize sizeCaret = GetCaretSize ();
		::CreateCaret (GetSafeHwnd (), NULL, sizeCaret.cx, sizeCaret.cy);
	}
}
//***************************************************************************************
DROPEFFECT CBCGPEditCtrl::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	if (!pDataObject->IsDataAvailable (CF_TEXT) && 
		!pDataObject->IsDataAvailable (g_cFormat)
		|| m_bReadOnly)
	{
		return DROPEFFECT_NONE;
	}

	CRect rect = m_rectText;
	rect.left = 0;
	int nCurrOffset = HitTest (point, FALSE, TRUE);
	
	if (!rect.PtInRect (point))
	{
		return DROPEFFECT_NONE; 
	}

	BOOL bScrollRight = FALSE;

	if (m_rectText.bottom - point.y < m_nLineHeight)
	{
		point.y += m_nLineHeight;
	}
	else if (point.y - m_rectText.top < m_nLineHeight)
	{
		point.y -= m_nLineHeight;
	}
	else if (m_rectText.right - point.x < m_nMaxCharWidth) 
	{
		point.x += m_nMaxCharWidth;
		bScrollRight = TRUE;
	}
	else if (point.x - m_rectText.left < m_nMaxCharWidth)
	{
		point.x -= m_nMaxCharWidth;
	}

	int nOffset = HitTest (point, TRUE, TRUE);
	int nColumn = GetColumnFromOffset (nOffset);
	int nCurrColumn = GetColumnFromOffset (nCurrOffset);

	if (bScrollRight && nColumn <= nCurrColumn)
	{
		return dwKeyState & MK_CONTROL ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
	}
	
	if (nOffset >= 0)
	{
		SetCaret (nOffset);
	}

	return dwKeyState & MK_CONTROL ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
}
//************************************************************************************
void CBCGPEditCtrl::FindWordStartFinish (int nCaretOffset, const CString& strBuffer,
										 int& nStartOffset, int& nEndOffset, 
										 BOOL /*bSkipSpaces*/) const
{
	if (nCaretOffset >= strBuffer.GetLength ())
	{
		nCaretOffset = strBuffer.GetLength () - 1;
	}

	if (nCaretOffset < 0)
	{
		nCaretOffset = 0;
	}

	for (int i = nCaretOffset;
	     i >= 0 && m_strNonSelectableChars.Find(strBuffer.GetAt(i)) != -1;
		 i--);
	
	nStartOffset = ReverseFindOneOf (strBuffer, i, m_strWordDelimeters);
	nEndOffset = FindOneOf (strBuffer, i, m_strWordDelimeters);
	
	nStartOffset = max (nStartOffset, 0);
	nEndOffset	= nEndOffset >= 0 ? nEndOffset : strBuffer.GetLength ();
}
//************************************************************************************
BOOL CBCGPEditCtrl::GetWordFromPoint (CPoint pt, CString& strWord)
{
	//pt.x += m_nScrollOffsetHorz;
	//pt.y += m_nScrollOffsetVert;
	
	CPoint ptSave  = pt;
	int nOffset = HitTest (pt, TRUE);
	if (abs (ptSave.x - pt.x) > m_nMaxCharWidth)
	{
		return FALSE;
	}
	if (nOffset == -1 || nOffset >= m_strBuffer.GetLength ())
	{
		return FALSE;
	}

	return GetWordFromOffset (nOffset, strWord);
}
//************************************************************************************
BOOL CBCGPEditCtrl::GetWordFromOffset (int nOffset, CString& strWord)
{
	int nStartOffset = -1;
	int nEndOffset = -1;

	FindWordStartFinish (nOffset, m_strBuffer, nStartOffset, nEndOffset);

	if (nEndOffset <= nStartOffset || nStartOffset == -1)
	{
		return FALSE;
	}

	strWord = m_strBuffer.Mid (nStartOffset, nEndOffset - nStartOffset);
	return TRUE;
}
//************************************************************************************
BOOL CBCGPEditCtrl::PreTranslateMessage(MSG* pMsg) 
{
	if (pMsg->message == WM_SYSKEYUP && pMsg->wParam == VK_MENU && 
		m_bAltPressedForBlockSel)
	{
		m_bAltPressedForBlockSel = FALSE;
		return TRUE;
	}

	switch (pMsg->message)
	{
	case WM_KEYDOWN:
	case WM_SYSKEYDOWN:
	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
	case WM_MBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_RBUTTONUP:
	case WM_MBUTTONUP:
	case WM_NCLBUTTONDOWN:
	case WM_NCRBUTTONDOWN:
	case WM_NCMBUTTONDOWN:
	case WM_NCLBUTTONUP:
	case WM_NCRBUTTONUP:
	case WM_NCMBUTTONUP:
	case WM_MOUSEMOVE:
		if (m_wndToolTip.GetSafeHwnd () != NULL)
		{
			m_wndToolTip.RelayEvent(pMsg);
		}
	}
	
	return CWnd::PreTranslateMessage(pMsg);
}
//****************************************************************************************
BOOL CBCGPEditCtrl::Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext /* = NULL */) 
{
	return CWnd::CreateEx(0, NULL, _T(""), dwStyle, rect, pParentWnd, nID, pContext);
}
//************************************************************************************
void CBCGPEditCtrl::PrepareBlock (CString& strOut)
{
	int iStartSel = min (m_iStartSel, m_iEndSel);
	int iEndSel = max (m_iStartSel, m_iEndSel);

	if (!m_bBlockSelectionMode)
	{
		strOut = m_strBuffer.Mid (iStartSel, iEndSel - iStartSel);
	}
	else
	{
		CRect rect (m_ptStartBlockSel, m_ptEndBlockSel);
		rect.NormalizeRect ();
		CPoint ptRowTopLeft = rect.TopLeft ();
		CPoint ptBottomRight = rect.BottomRight ();

		ptRowTopLeft.x	-= m_nScrollOffsetHorz * m_nMaxCharWidth;
		ptBottomRight.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;

		ptRowTopLeft.y	-= m_nScrollOffsetVert * m_nLineHeight;
		ptBottomRight.y -= m_nScrollOffsetVert * m_nLineHeight;


		if (m_ptEndBlockSel.y <= m_ptStartBlockSel.y)
		{
			ptRowTopLeft.y -= m_nLineHeight;
			ptBottomRight.y += m_nLineHeight;
		}

		
		ptBottomRight.x -= m_nAveCharWidth;
		

		CPoint ptRowEnd = ptBottomRight;
		ptRowEnd.y = ptRowTopLeft.y;

		do
		{
			int nStartColumn = HitTest (ptRowTopLeft);
			int nEndColumn = HitTest (ptRowEnd);

			ptRowTopLeft.y += m_nLineHeight;
			ptRowEnd.y += m_nLineHeight;

			CString strSubRow = m_strBuffer.Mid (nStartColumn, nEndColumn - nStartColumn + 1);
			strOut += strSubRow; 
			if (strSubRow.Find (g_chEOL) == -1)
			{
				strOut += g_strEOL;
			}
		}while (ptRowTopLeft.y < ptBottomRight.y);
	}
}
//************************************************************************************
void CBCGPEditCtrl::RetrieveSelectedBlock (CStringArray& arStrings)
{
	arStrings.RemoveAll ();

	if (m_iStartSel == -1 && m_iEndSel == -1)
	{
		return;
	}

	int iStartSel = min (m_iStartSel, m_iEndSel);
	int iEndSel = max (m_iStartSel, m_iEndSel);


	if (!m_bBlockSelectionMode)
	{
		arStrings.SetAtGrow (0, m_strBuffer.Mid (iStartSel, iEndSel - iStartSel));
	}
	else
	{
		CRect rect (m_ptStartBlockSel, m_ptEndBlockSel);
		rect.NormalizeRect ();
		CPoint ptRowTopLeft = rect.TopLeft ();
		CPoint ptBottomRight = rect.BottomRight ();

		ptRowTopLeft.x	-= m_nScrollOffsetHorz * m_nMaxCharWidth;
		ptBottomRight.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;

		ptRowTopLeft.y	-= m_nScrollOffsetVert * m_nLineHeight;
		ptBottomRight.y -= m_nScrollOffsetVert * m_nLineHeight;


		if (m_ptEndBlockSel.y <= m_ptStartBlockSel.y)
		{
			ptRowTopLeft.y -= m_nLineHeight;
			ptBottomRight.y += m_nLineHeight;
		}
		
		ptBottomRight.x -= m_nAveCharWidth;

		CPoint ptRowEnd = ptBottomRight;
		ptRowEnd.y = ptRowTopLeft.y;

		int nRow = 0;
		do
		{
			int nStartColumn = HitTest (ptRowTopLeft);
			int nEndColumn = HitTest (ptRowEnd);

			ptRowTopLeft.y += m_nLineHeight;
			ptRowEnd.y += m_nLineHeight;

			arStrings.SetAtGrow (nRow, m_strBuffer.Mid (nStartColumn, nEndColumn - nStartColumn + 1));
		}while (ptRowTopLeft.y < ptBottomRight.y);
	}
}
//************************************************************************************
void CBCGPEditCtrl::BuildTabString (CString& str)
{
	if (m_bKeepTabs)
	{
		int nTabs = m_nIndentSize / m_nTabSize;
		int nSpaces = m_nIndentSize % m_nTabSize;

		CString strTabs (_T('\t'), nTabs);
		CString strSpaces (_T(' '), nSpaces);

		str += strTabs;
		str += strSpaces;
	}
	else
	{
		for (int i = 0; i < m_nIndentSize; i++)
		{
			str += _T(' ');
		}
	}
}
//----------------------
// Marker Support
//----------------------
//************************************************************************************
void CBCGPEditCtrl::OnDrawMarker (CDC* pDC, CRect rectMarker, const CBCGPEditMarker* pMarker)
{
	if (pMarker->m_dwMarkerType & g_dwBCGPEdit_BookMark)
	{
		CBrush br (RGB (0, 255, 255));
		CBrush* pBrOld = pDC->SelectObject (&br);

		int nVertSpacing = m_nLineVertSpacing % 2; 

		rectMarker.DeflateRect (0, m_nLineVertSpacing / 2 + nVertSpacing, 0, m_nLineVertSpacing / 2 + nVertSpacing);
		pDC->RoundRect (rectMarker, CPoint (5, 5));
		pDC->SelectObject (pBrOld);
	}		
}
//************************************************************************************
BOOL CBCGPEditCtrl::ToggleMarker (int nRow, DWORD dwMarkerType, LPVOID lpData, BOOL bRedraw)
{
	ASSERT_VALID (this);
	CBCGPEditMarker* pMarker = NULL;

	if (GetMarker (nRow, &pMarker, dwMarkerType))
	{
		DeleteMarker (nRow, dwMarkerType, bRedraw);
		return FALSE;
	}

	pMarker = new CBCGPEditMarker ();

	pMarker->m_dwMarkerType = dwMarkerType;
	pMarker->m_nLine = nRow;
	pMarker->m_pData = lpData;
	SetMarker (pMarker, nRow, bRedraw);	

	return TRUE;
	
}
//************************************************************************************
POSITION CBCGPEditCtrl::SetMarker (CBCGPEditMarker* pMarker, int nRow, BOOL bRedraw)
{
	ASSERT_VALID (this);
	ASSERT_VALID (pMarker);

	if (nRow == -1)
	{
		pMarker->m_nLine = GetCurRow ();
	}

	return InsertMarker (pMarker, bRedraw);
}
//************************************************************************************
POSITION CBCGPEditCtrl::InsertMarker (CBCGPEditMarker* pMarker, BOOL bRedraw)
{
	ASSERT_VALID (this);
	ASSERT_VALID (pMarker);

	if (pMarker->m_dwMarkerType & g_dwBCGPEdit_MarkerReserved)
	{
		ASSERT (FALSE);
		TRACE0 ("The type of this marker is reserved by the library. Use values greatr than 0x0000FFF0\n");
		return NULL;
	}

	POSITION posInserted = NULL;
	
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		POSITION posSave = pos;
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
															m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);

		if (pMarker->m_nLine < pMarkerNext->m_nLine ||
			pMarker->m_nLine == pMarkerNext->m_nLine &&
			pMarker->m_dwMarkerType <= pMarkerNext->m_dwMarkerType)
		{
			posInserted = m_lstMarkers.InsertBefore (posSave, pMarker);
			break;
		}
	}

	if (posInserted == NULL)
	{
		posInserted = m_lstMarkers.AddTail (pMarker);
	}

	if (bRedraw)
	{
		BOOL bFullLine = pMarker->m_dwMarkerType & g_dwBCGPEdit_LineColorMarker;
		RedrawMarkerArea (pMarker->m_nLine, bFullLine);
	}

	return posInserted;
}
//************************************************************************************
POSITION CBCGPEditCtrl::GetMarker (int nRow, CBCGPEditMarker** ppMarker, 
									DWORD dwMarkerType)
{
	ASSERT_VALID (this);
	ASSERT (ppMarker != NULL);

	POSITION posSave = NULL;
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		posSave = pos;
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		
		if (pMarkerNext->m_dwMarkerType & dwMarkerType &&
			pMarkerNext->IsInRange (nRow, nRow))
		{
			*ppMarker = pMarkerNext;
			return posSave;
		}
	}
	return NULL;
}
//************************************************************************************
void CBCGPEditCtrl::GetMarkersInRange (int nRowStart, CObList& lstMarkers, 
									   DWORD dwMarkerType, int nRowEnd) const
{
	ASSERT_VALID (this);

	if (nRowEnd == -1)
	{
		nRowEnd = nRowStart;
	}

	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->m_dwMarkerType & dwMarkerType &&
			pMarkerNext->IsInRange (nRowStart, nRowEnd))
		{
			lstMarkers.AddTail (pMarkerNext);
		}
	}		
}
//************************************************************************************
BOOL CBCGPEditCtrl::SetMarkerData (int nRow, LPVOID lpData, DWORD dwMarkerType, 
								   BOOL bRedraw)
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		
		if (nRow == pMarkerNext->m_nLine && 
			pMarkerNext->m_dwMarkerType & dwMarkerType)
		{
			pMarkerNext->m_pData = lpData;	
			if (bRedraw)
			{
				RedrawMarkerArea (nRow);
			}
			return TRUE;
		}
	}

	return FALSE;
}
//************************************************************************************
BOOL CBCGPEditCtrl::DeleteMarker (int nRow, DWORD dwMarkerType, BOOL bRedraw)
{
	BOOL bDeleted = FALSE;
	POSITION posSave = NULL;
	int nLine = -1;
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		posSave = pos;
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);

		nLine = pMarkerNext->m_nLine;
		
		if (nLine == nRow && pMarkerNext->m_dwMarkerType & dwMarkerType)
		{
			DWORD dwMarkerType = pMarkerNext->m_dwMarkerType;

			if (OnRemoveMarker (pMarkerNext, posSave))
			{
				m_lstMarkers.RemoveAt (posSave);
				delete pMarkerNext;

				bDeleted = TRUE;

				if (bRedraw)
				{
					RedrawMarkerArea (nRow);

					if (dwMarkerType & g_dwBCGPEdit_LineColorMarker)
					{
						RedrawText (nRow, 0);
					}
				}
			}
		}

		if (nLine > nRow)
		{
			break;
		}
	}

	return bDeleted;
}
//************************************************************************************
void CBCGPEditCtrl::DeleteMarker (POSITION posMarker)
{
	ASSERT_VALID (this);
	ASSERT (posMarker != NULL);

	CBCGPEditMarker* pMarker = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
												 m_lstMarkers.GetAt (posMarker));
	if (pMarker != NULL)
	{
		OnRemoveMarker (pMarker, posMarker);
		m_lstMarkers.RemoveAt (posMarker);
		delete pMarker;
	}
}
//************************************************************************************
void CBCGPEditCtrl::DeleteAllMarkers (DWORD dwMarkerType, BOOL bRedraw)
{
	if (dwMarkerType == (DWORD) -1)
	{
		CleanUpMarkers ();
	}
	else
	{
		for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
		{
			POSITION posSave = pos;
			CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
			ASSERT_VALID (pMarkerNext);
			if (pMarkerNext->m_dwMarkerType & dwMarkerType)
			{
				DeleteMarker (posSave); 
			}
		}
	}

	if (bRedraw) 
	{
		RedrawMarkerArea (-1);
		RedrawWindow ();
	}
	return;
}
//************************************************************************************
CBCGPEditMarker* CBCGPEditCtrl::FindMarkerByData (DWORD dwData) const
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->CompareData (dwData))
		{
			return pMarkerNext;
		}
	}

	return NULL;
}
//************************************************************************************
void CBCGPEditCtrl::GetMarkerListByData (CObList& lstMarkers, DWORD dwData) const
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->CompareData (dwData))
		{
			lstMarkers.AddTail (pMarkerNext);
		}
	}
}
//************************************************************************************
void CBCGPEditCtrl::GetMarkerListByType (CObList& lstMarkers, DWORD dwMarkerType) const
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->m_dwMarkerType & dwMarkerType)
		{
			lstMarkers.AddTail (pMarkerNext);
		}
	}
}											
//************************************************************************************
BOOL CBCGPEditCtrl::GoToNextMarker (DWORD dwMarkerType, BOOL bDirection)
{
	if (m_lstMarkers.IsEmpty ())
	{
		return FALSE;
	}

	CObList lstMarkers;
	GetMarkerListByType (lstMarkers, dwMarkerType);

	if (lstMarkers.IsEmpty ())
	{
		return FALSE;
	}

	int nRow = GetCurRow ();

	for (POSITION pos = bDirection ? lstMarkers.GetHeadPosition () : 
									 lstMarkers.GetTailPosition () ; pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = (CBCGPEditMarker*) 
											(bDirection ? lstMarkers.GetNext (pos) : 
											lstMarkers.GetPrev (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->m_nLine > nRow && bDirection ||
			pMarkerNext->m_nLine < nRow && !bDirection)
		{
			return GoToMarker (pMarkerNext);
		}
	}

	CBCGPEditMarker* pMarkerNext = (CBCGPEditMarker*) 
						(bDirection ? lstMarkers.GetHead () : lstMarkers.GetTail ());

	return GoToMarker (pMarkerNext);
}
//************************************************************************************
BOOL CBCGPEditCtrl::HasMarkers (DWORD dwMarkerType)
{
	if (m_lstMarkers.IsEmpty ())
	{
		return FALSE;
	}

	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
													m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);
		if (pMarkerNext->m_dwMarkerType & dwMarkerType)
		{
			return TRUE;
		}
	}
	return FALSE;
}
//************************************************************************************
void CBCGPEditCtrl::RedrawMarkerArea (int nRow, BOOL bFullLine)
{
	CRect rectArea;
	GetClientRect (rectArea);

	rectArea.right = bFullLine ? m_rectText.right : rectArea.left + m_nLeftMarginWidth;
	
	if (nRow != -1)
	{
		rectArea.top = (nRow - m_nScrollOffsetVert) * m_nLineHeight;
		rectArea.bottom = rectArea.top + m_nLineHeight;
	}


	RedrawWindow (rectArea);		
}
//************************************************************************************
BOOL CBCGPEditCtrl::GoToMarker (const CBCGPEditMarker* pMarker)
{
	ASSERT_VALID (this);
	ASSERT_VALID (pMarker);

	int nOffset = GetRowStart (pMarker->m_nLine);
	if (nOffset != -1)
	{
		return SetCaret (nOffset);
	}
	return FALSE;
}
//************************************************************************************
void CBCGPEditCtrl::CleanUpMarkers ()
{
	while (!m_lstMarkers.IsEmpty ())
	{
		POSITION posMarker = m_lstMarkers.GetHeadPosition ();
		CBCGPEditMarker* pMarker =
			DYNAMIC_DOWNCAST (CBCGPEditMarker, m_lstMarkers.RemoveHead ());
		if (pMarker != NULL)
		{
			OnRemoveMarker (pMarker, posMarker);
			delete pMarker;
		}
	}
}
//************************************************************************************
void CBCGPEditCtrl::SetMarkerData (POSITION pos, DWORD dwData)
{
	ASSERT_VALID (this);
	CBCGPEditMarker* pMarker =
			DYNAMIC_DOWNCAST (CBCGPEditMarker, m_lstMarkers.GetAt (pos));
	if (pMarker != NULL)
	{
		pMarker->m_pData = (LPVOID) dwData;
	}
}
//************************************************************************************
POSITION CBCGPEditCtrl::SetLineColorMarker (int nRow, COLORREF clrFore, 
							 COLORREF clrBk, BOOL bFullLine, int nLineCount, 
							 DWORD dwMarkerType)
{
	if (nRow == -1)
	{
		nRow = GetCurRow ();
	}

	if (clrFore == (DWORD) -1)
	{
		clrFore = GetDefaultTextColor ();
	}

	if (clrBk == (DWORD) -1)
	{
		clrBk = GetDefaultBackColor ();
	}

	CBCGPLineColorMarker* pLineColorMarker = new CBCGPLineColorMarker;
	pLineColorMarker->m_nLine = nRow;
	pLineColorMarker->m_nLineCount = nLineCount;
	pLineColorMarker->m_clrForeground = clrFore;
	pLineColorMarker->m_clrBackground = clrBk;
	pLineColorMarker->m_bFullLine = bFullLine;
	pLineColorMarker->m_dwMarkerType = dwMarkerType | g_dwBCGPEdit_LineColorMarker;

	return InsertMarker (pLineColorMarker);
}
//************************************************************************************
void CBCGPEditCtrl::UpdateLineRelatedData (int nStartLine, int nNumLines)
{
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														 m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);

		if (pMarkerNext->m_nLine >= nStartLine)
		{
			int nNewLine = pMarkerNext->m_nLine + nNumLines;
			if (CanUpdateMarker (pMarkerNext))
			{
				pMarkerNext->m_nLine = nNewLine;
			}
		}
	}
}
//************************************************************************************
void CBCGPEditCtrl::RemoveLineDataInRange (int nStartRow, int nEndRow)
{
	POSITION posSave = NULL;
	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		posSave = pos;
		CBCGPEditMarker* pMarkerNext = DYNAMIC_DOWNCAST (CBCGPEditMarker, 
														 m_lstMarkers.GetNext (pos));
		ASSERT_VALID (pMarkerNext);

		CBCGPEditMarkerRange range = pMarkerNext->IsInRange (nStartRow, nEndRow);

		if (range == FULL_IN_RANGE &&
			CanRemoveMarker (pMarkerNext))
		{
			DeleteMarker (posSave);
		}
		else if (range == PARTIAL_IN_RANGE && 
				 CanUpdateMarker (pMarkerNext))
		{
			pMarkerNext->UpdateMarkerForDeletedRange (nStartRow, nEndRow);
		}
	}
}
//************************************************************************************
BOOL CBCGPEditCtrl::LoadXMLSettings (const CString& strFileName)
{
	ASSERT_VALID (this);

	CString strBuffer;
	CString strPath = strFileName;

	if (strFileName.Find (_T("\\")) == -1 &&
		strFileName.Find (_T("/")) == -1 &&
		strFileName.Find (_T(":")) == -1)
	{
		TCHAR lpszFilePath [_MAX_PATH];
		if (::GetModuleFileName (NULL, lpszFilePath, _MAX_PATH) > 0)
		{
			TCHAR path_buffer[_MAX_PATH];   
			TCHAR drive[_MAX_DRIVE];   
			TCHAR dir[_MAX_DIR];
			TCHAR fname[_MAX_FNAME];   
			TCHAR ext[_MAX_EXT];

			_tsplitpath (lpszFilePath, drive, dir, NULL, NULL);
			_tsplitpath (strFileName, NULL, NULL, fname, ext);

			_tmakepath (path_buffer, drive, dir, fname, ext);
			strPath = path_buffer;
		}
	}

	try
	{
		CStdioFile file;
		if (!file.Open (strPath, CFile::modeRead))
		{
			TRACE(_T("File not found: %s"), strFileName);
			return FALSE;
		}

		CString str;

		while (file.ReadString (str))
		{
			strBuffer += str;
		}
	}
	catch (CFileException* pEx)
	{
		pEx->ReportError ();
		pEx->Delete ();

		return FALSE;
	}

	return LoadXMLSettingsFromBuffer (strBuffer);
}
//*************************************************************************************
static BOOL ExcludeTag (CString& strBuffer, 
						LPCTSTR lpszTag,
						CString& strTag, BOOL bIsCharsList = FALSE)
{
	const int iBufLen = strBuffer.GetLength ();

	CString strTagStart = _T('<');
	strTagStart += lpszTag;
	strTagStart += _T('>');

	const int iTagStartLen = strTagStart.GetLength ();

	int iStart = -1;

	int iIndexStart = strBuffer.Find (strTagStart);
	if (iIndexStart < 0)
	{
		return FALSE;
	}

	iStart = iIndexStart + iTagStartLen;

	CString strTagEnd = _T("</");
	strTagEnd += lpszTag;
	strTagEnd += _T('>');

	const int iTagEndLen = strTagEnd.GetLength ();

	int iIndexEnd =  -1;
	int nBalanse = 1;
	for (int i = iStart; i < iBufLen - iTagEndLen + 1; i ++)
	{
		if (strBuffer [i] != '<')
		{
			continue;
		}

		if (i < iBufLen - iTagStartLen &&
			_tcsncmp (strBuffer.Mid (i), strTagStart, iTagStartLen) == 0)
		{
			i += iTagStartLen - 1;
			nBalanse ++;
			continue;
		}

		if (_tcsncmp (strBuffer.Mid (i), strTagEnd, iTagEndLen) == 0)
		{
			nBalanse --;
			if (nBalanse == 0)
			{
				iIndexEnd = i;
				break;
			}

			i += iTagEndLen - 1;
		}
	}

	if (iIndexEnd == -1 || iStart > iIndexEnd)
	{
		return FALSE;
	}

	strTag = strBuffer.Mid (iStart, iIndexEnd - iStart);
	strTag.TrimLeft ();
	strTag.TrimRight ();

	strBuffer.Delete (iIndexStart, iIndexEnd + iTagEndLen - iIndexStart);

	if (bIsCharsList)
	{
		if (strTag.GetLength () > 1 && strTag [0] == _T('\"'))
		{
			strTag = strTag.Mid (1, strTag.GetLength () - 2);
		}

		strTag.Replace (_T("\\t"), _T("\t"));
		strTag.Replace (_T("\\n"), _T("\n"));
		strTag.Replace (_T("\\r"), _T("\r"));
		strTag.Replace (_T("\\b"), _T("\b"));
		strTag.Replace (_T("LT"), _T("<"));
		strTag.Replace (_T("GT"), _T(">"));
		strTag.Replace (_T("AMP"), _T("&"));
	}

	return TRUE;
}
//***************************************************************************************
COLORREF GetColorFromBuffer (const CString strBuffer)
{
	if (strBuffer.GetLength () < 1)
	{
		ASSERT (FALSE);
		return 0;
	}

	CString strColor = strBuffer;

	if (strColor [0] == _T('\"'))
	{
		strColor.MakeLower ();
		strColor = strColor.Mid (1, strColor.GetLength () - 2);

		if (strColor == _T("black"))
			return RGB (0, 0, 0);
		else if (strColor == _T("darkred"))
			return RGB (128, 0, 0);
		else if (strColor == _T("darkgreen"))
			return RGB (0, 128, 0);
		else if (strColor == _T("brown"))
			return RGB (128, 128, 0);
		else if (strColor == _T("darkblue"))
			return RGB (0, 0, 128);
		else if (strColor == _T("darkmagenta"))
			return RGB (128, 0, 128);
		else if (strColor == _T("darkcyan"))
			return RGB (0, 128, 128);
		else if (strColor == _T("gray"))
			return RGB (192, 192, 192);
		else if (strColor == _T("darkgray"))
			return RGB (128, 128, 128);
		else if (strColor == _T("red"))
			return RGB (255, 0, 0);
		else if (strColor == _T("green"))
			return RGB (0, 255, 0);
		else if (strColor == _T("yellow"))
			return RGB (255, 255, 0);
		else if (strColor == _T("blue"))
			return RGB (0, 0, 255);
		else if (strColor == _T("magenta"))
			return RGB (255, 0, 255);
		else if (strColor == _T("cyan"))
			return RGB (0, 255, 255);
		else if (strColor == _T("white"))
			return RGB (255, 255, 255);

		return 0;
	}

	if (strColor.GetLength () != 6)
	{
		// Should be 'rrggbb'!
		ASSERT (FALSE);
		return 0;
	}

	// Swap Red and Blue values:
	CString str;
	str.Format (_T("0x%s%s%s"), strColor.Right (2), strColor.Mid (2, 2), strColor.Left (2));

	COLORREF color;
	_stscanf (str, _T("%x"), &color);

	return color;
}
//****************************************************************************************
void CBCGPEditCtrl::RemoveXMLSettings ()
{
	m_mapWordColors.RemoveAll ();
	m_lstColorBlocks.RemoveAll ();
}
//****************************************************************************************
BOOL CBCGPEditCtrl::LoadXMLSettingsFromBuffer (const CString& strInBuffer)
{
	RemoveXMLSettings ();

	CString strBuffer = strInBuffer;

	//--------------
	// Read options:
	//--------------
	CString strOptions;
	if (ExcludeTag (strBuffer, _T("OPTIONS"), strOptions))
	{
		ExcludeTag (strOptions, _T("WordDelimeters"), m_strWordDelimeters, TRUE);
		ExcludeTag (strOptions, _T("SpecialDelimiters"), m_strSpecialDelimiters, TRUE);
		ExcludeTag (strOptions, _T("IntelliSenseChars"), m_strIntelliSenseChars, TRUE);
	}

	//-----------------
	// Read color data:
	//-----------------
	CString strColorData;
	if (ExcludeTag (strBuffer, _T("COLOR_DATA"), strColorData))
	{
		//---------------------------------
		// Read text and background colors:
		//---------------------------------
		CString strColor;
		if (ExcludeTag (strColorData, _T("EditTextColor"), strColor))
		{
			m_clrText = GetColorFromBuffer (strColor);
		}

		if (ExcludeTag (strColorData, _T("EditBackColor"), strColor))
		{
			m_clrBack = GetColorFromBuffer (strColor);
		}

		if (ExcludeTag (strColorData, _T("SelTextColor"), strColor))
		{
			m_clrTextSelActive = GetColorFromBuffer (strColor);
			m_clrTextSelInActive = m_clrTextSelActive;
		}

		if (ExcludeTag (strColorData, _T("SelBackColor"), strColor))
		{
			m_clrBackSelActive = GetColorFromBuffer (strColor);
			m_clrBackSelInActive = m_clrBackSelActive;
		}

		//----------------
		// Read keywwords:
		//----------------
		CString strKeywords;
		if (ExcludeTag (strColorData, _T("KEYWORDS"), strKeywords))
		{
			BOOL bCaseSensitive = TRUE;

			CString strCaseSensitive;
			if (ExcludeTag (strKeywords, _T("CaseSensitive"), strCaseSensitive))
			{
				strCaseSensitive.MakeUpper ();
				bCaseSensitive = (strCaseSensitive == _T("TRUE"));
			}

			CString strColor;
			if (!ExcludeTag (strKeywords, _T("Color"), strColor))
			{
				ASSERT (FALSE);
			}
			else
			{
				COLORREF color = GetColorFromBuffer (strColor);
				COLORREF colorBack = (COLORREF) -1;

				CString strBackColor;
				if (ExcludeTag (strKeywords, _T("BackColor"), strBackColor))
				{
					colorBack = GetColorFromBuffer (strBackColor);
				}

				while (TRUE)
				{
					CString strKeyword;
					if (!ExcludeTag (strKeywords, _T("Keyword"), strKeyword))
					{
						break;
					}

					SetWordColor (strKeyword, color, colorBack, bCaseSensitive);
				}
			}
		}

		//------------
		// Read words:
		//------------
		CString strWords;
		if (ExcludeTag (strColorData, _T("WORDS"), strWords))
		{
			while (TRUE)
			{
				CString strWordData;
				if (!ExcludeTag (strWords, _T("WORD"), strWordData))
				{
					break;
				}

				CString strVal;
				if (!ExcludeTag (strWordData, _T("Val"), strVal))
				{
					ASSERT (FALSE);
					break;
				}

				CString strColor;
				if (!ExcludeTag (strWordData, _T("Color"), strColor))
				{
					ASSERT (FALSE);
					break;
				}

				COLORREF color = GetColorFromBuffer (strColor);
				COLORREF colorBack = (COLORREF) -1;

				CString strBackColor;
				if (ExcludeTag (strWordData, _T("BackColor"), strBackColor))
				{
					colorBack = GetColorFromBuffer (strBackColor);
				}

				BOOL bCaseSensitive = TRUE;
				CString strCaseSensitive;

				if (ExcludeTag (strWordData, _T("CaseSensitive"), strCaseSensitive))
				{
					strCaseSensitive.MakeUpper ();
					bCaseSensitive = (strCaseSensitive == _T("TRUE"));
				}

				SetWordColor (strVal, color, colorBack, bCaseSensitive);
			}
		}

		//-------------
		// Read blocks:
		//-------------
		CString strBlocks;
		if (ExcludeTag (strColorData, _T("BLOCKS"), strBlocks))
		{
			while (TRUE)
			{
				CString strBlockData;
				if (!ExcludeTag (strBlocks, _T("BLOCK"), strBlockData))
				{
					break;
				}

				CString strStart;
				if (!ExcludeTag (strBlockData, _T("Start"), strStart))
				{
					ASSERT (FALSE);
					break;
				}

				CString strEnd;
				ExcludeTag (strBlockData, _T("End"), strEnd);

				COLORREF color = (COLORREF)-1;

				CString strColor;
				if (ExcludeTag (strBlockData, _T("Color"), strColor))
				{
					color = GetColorFromBuffer (strColor);
				}

				COLORREF colorBack = (COLORREF) -1;

				CString strBackColor;
				if (ExcludeTag (strBlockData, _T("BackColor"), strBackColor))
				{
					colorBack = GetColorFromBuffer (strBackColor);
				}

				CString strIsWholeText;
				BOOL bIsWholeText = FALSE;

				if (ExcludeTag (strBlockData, _T("WholeText"), strIsWholeText))
				{
					strIsWholeText.MakeLower ();
					bIsWholeText = (strIsWholeText == _T("true"));
				}

				SetBlockColor (strStart, strEnd, bIsWholeText, color, colorBack);
			}
		}
	}

	return TRUE;
}
//****************************************************************************************
void CBCGPEditCtrl::ShowContextTip()
{
	if (m_bEnableToolTips)
	{
		if (m_wndToolTip.GetSafeHwnd () != NULL)
		{
			ToolTipPop ();
		}
	}
}
//****************************************************************************************
BOOL CBCGPEditCtrl::OnTTNeedTipText (UINT /*id*/, NMHDR* pNMH, LRESULT* /*pResult*/)
{
	if (!m_bEnableToolTips)
	{
		return FALSE;
	}

	CPoint point;
	
	::GetCursorPos (&point);
	ScreenToClient (&point);

	if (!GetWordFromPoint (point, strTipText))
	{
		ToolTipPop ();
		return FALSE;
	}
	
	LPNMTTDISPINFO	pTTDispInfo	= (LPNMTTDISPINFO) pNMH;
	ASSERT((pTTDispInfo->uFlags & TTF_IDISHWND) == 0);

	if (!OnGetTipText (strTipText))
	{
		return FALSE;
	}

	if (m_bToolTipCleared && strTipText == m_strLastDisplayedToolTip)
	{
		return FALSE;
	}

	pTTDispInfo->lpszText = const_cast<LPTSTR> ((LPCTSTR) strTipText);
	m_bToolTipCleared = FALSE;
	m_strLastDisplayedToolTip = strTipText;
	
	return TRUE;
}
//****************************************************************************************
void CBCGPEditCtrl::EnableToolTips (BOOL bEnable)
{
	if (m_bEnableToolTips == bEnable)
	{
		return;
	}

	m_bEnableToolTips = bEnable;

	if (bEnable)
	{
		m_wndToolTip.Activate (TRUE);
	}
	else
	{
		ToolTipPop ();
		m_wndToolTip.Activate (FALSE);
	}
}
//****************************************************************************************
void CBCGPEditCtrl::ToolTipPop ()
{
	m_wndToolTip.Pop ();
	strTipText.Empty ();
	m_bToolTipCleared = TRUE;
}
//****************************************************************************************
BOOL CBCGPEditCtrl::OpenFile (const CString& strFileName)
{
	CFile file;
	if (!file.Open (strFileName, CFile::modeRead))
	{
		TRACE(_T("File not found: %s"), strFileName);
		return FALSE;
	}

	return OpenFile (file);
}
//***************************************************************************************
BOOL CBCGPEditCtrl::OpenFile (CFile& file)
{
	try
	{
		const DWORD dwFileSize = file.GetLength();
		TCHAR* pszFileBuffer = new TCHAR [dwFileSize + 1];
		
#if _MSC_VER >= 1300
		file.Read ((void*)pszFileBuffer, dwFileSize);
#else
		file.ReadHuge ((void*)pszFileBuffer, dwFileSize);
#endif
		pszFileBuffer [dwFileSize] = _T ('\0');

		SetWindowText (pszFileBuffer);
		delete pszFileBuffer;
	}
	catch (CFileException* pEx)
	{
		pEx->ReportError ();
		pEx->Delete ();

		return FALSE;
	}

	return TRUE;
}

//***************************************************************************************
BOOL CBCGPEditCtrl::SaveFile (const CString& strFileName)
{
	CFile file;
	if (!file.Open (strFileName, CFile::modeWrite | CFile::modeCreate))
	{
		TRACE(_T("Cannot open file: %s"), strFileName);
		return FALSE;
	}

	return SaveFile (file);
}
//****************************************************************************************
BOOL CBCGPEditCtrl::SaveFile (CFile& file)
{
	try
	{

		const DWORD dwBufferSize = m_strBuffer.GetLength();

		if (dwBufferSize > 0)
		{
			HGLOBAL hGlobal = ExportBuffer (m_strBuffer, dwBufferSize, FALSE, TRUE);
			if (hGlobal == NULL)
			{
				TRACE0 ("SaveFile failed - out of memory\n");
				return FALSE;
			}
			LPTSTR lpszBuffer = (LPTSTR) GlobalLock (hGlobal);
			if (lpszBuffer == NULL)
			{
				TRACE0 ("SaveFile failed - memory lock error\n");
				GlobalFree (hGlobal);
				return FALSE;
			}

#if _MSC_VER >= 1300
			file.Write(lpszBuffer, _tcslen (lpszBuffer) * sizeof (TCHAR));
#else
			file.WriteHuge(lpszBuffer, _tcslen (lpszBuffer) * sizeof (TCHAR));
#endif
			GlobalFree (hGlobal);
		}
		else
		{
			file.SetLength(0);
		}
	}
	catch (CFileException* pEx)
	{
		pEx->ReportError ();
		pEx->Delete ();

		return FALSE;
	}
	
	m_bIsModified = FALSE;
	m_posDocSavedUndoPos = m_posUndoBuffer;

	return TRUE;
}
//****************************************************************************************
BOOL CBCGPEditCtrl::FindText(LPCTSTR lpszFind, BOOL bNext /* = TRUE */, BOOL bCase /* = TRUE */, BOOL bWholeWord /* = FALSE */)
{
	if (lpszFind == NULL || lpszFind [0] == 0)
	{
		return FALSE;
	}

	int nCurrOffset;

	if (m_nCurrOffset >= m_strBuffer.GetLength())
	{
		nCurrOffset = m_strBuffer.GetLength() - 1;
	}
	else if (m_nCurrOffset < 0)
	{
		nCurrOffset = 0;
	}
	else
	{
		nCurrOffset = m_nCurrOffset;
	}

	CString strTmpBuffer;

	CString strFind(lpszFind);
	CString *pStrBuffer = NULL;
	
	if (!bCase)
	{
		strTmpBuffer = m_strBuffer;
		strTmpBuffer.MakeLower();
		strFind.MakeLower();

		pStrBuffer = &strTmpBuffer;
	}

	if (!bNext)
	{
		if (!pStrBuffer)
		{
			strTmpBuffer = m_strBuffer;
			pStrBuffer = &strTmpBuffer;
		}

		pStrBuffer->MakeReverse();
		strFind.MakeReverse();

		nCurrOffset = m_strBuffer.GetLength() - nCurrOffset;
	}

	const int nFindLen = strFind.GetLength();
	lpszFind = strFind.GetBuffer(0);

	if (pStrBuffer == NULL)
	{
		pStrBuffer = &m_strBuffer;
	}

	int nPos = 0;
	int nLastOffset = nCurrOffset;
	
	do
	{
		nPos = pStrBuffer->Find(lpszFind, nCurrOffset);

		if (!bNext)
		{
			if (nPos == nLastOffset)
			{
				if (m_iStartSel < m_iEndSel &&
					nPos == m_strBuffer.GetLength() - m_iEndSel)
				{
					nPos = pStrBuffer->Find(lpszFind, ++nCurrOffset);

					if (nPos == -1)
					{
						nCurrOffset += (nFindLen - 1);
					}

					nLastOffset = nCurrOffset;
				}
			}
		}

		if (!bWholeWord || nPos == -1)
		{
			break;
		}
		
		int nPos1 = (nPos > 0)? m_strWordDelimeters.Find(pStrBuffer->GetAt(nPos - 1)): 0;
		int nPos2 = (nPos + nFindLen < m_strBuffer.GetLength() - 1)? m_strWordDelimeters.Find(pStrBuffer->GetAt(nPos + nFindLen)): 0;

		if (nPos1 == -1 ||
			nPos2 == -1)
		{
			nCurrOffset = nPos + nFindLen;
			nPos = -1;

			if (nCurrOffset == m_strBuffer.GetLength())
				break;
		}
	}
	while (nPos == -1);

	if (nPos == -1)
	{
		if (nLastOffset > pStrBuffer->GetLength())
		{
			nLastOffset = pStrBuffer->GetLength();
		}

		if (nLastOffset > 0)
		{
			LPTSTR pszBuffer = pStrBuffer->GetBuffer(nLastOffset + 1);
			TCHAR tCh = pszBuffer[nLastOffset];

			pszBuffer[nLastOffset] = 0;

			if (!bWholeWord)
			{
				nPos = pStrBuffer->Find(lpszFind);
			}
			else
			{
				nCurrOffset = 0;

				do
				{
					nPos = pStrBuffer->Find(lpszFind, nCurrOffset);

					if (nPos == -1)
					{
						break;
					}
					else
					{
						int nPos1 = (nPos > 0)? m_strWordDelimeters.Find(pszBuffer[nPos - 1]): 0;
						int nPos2 = (nPos + nFindLen < nLastOffset - 1)? m_strWordDelimeters.Find(pszBuffer[nPos + nFindLen]): 0;

						if (nPos1 == -1 ||
							nPos2 == -1)
						{
							nCurrOffset = nPos + nFindLen;
							nPos = -1;

							if (nCurrOffset == nLastOffset)
							{
								break;
							}
						}
					}
				}
				while (nPos == -1);
			}
			
			pszBuffer[nLastOffset] = tCh;
			pStrBuffer->ReleaseBuffer();
		}
	}

	if (nPos == -1)
	{
		return FALSE;
	}

	if (!bNext)
	{
		nPos = m_strBuffer.GetLength() - (nPos + nFindLen);
	}
	
	SetSel (nPos, nPos + nFindLen);

	return TRUE;
}
//***********************************************************************************
void CBCGPEditCtrl::InitColors ()
{
	m_clrBackSelActive = ::GetSysColor (COLOR_HIGHLIGHT);
	m_clrTextSelActive = ::GetSysColor (COLOR_HIGHLIGHTTEXT);

	double H;
	double S;
	double L;

	CBCGPDrawManager::RGBtoHSL (m_clrBackSelActive, &H, &S, &L);
	m_clrBackSelInActive = CBCGPDrawManager::HLStoRGB_ONE (
		H,
		min (1., S * 1.1),
		min (1., L * 1.1));

	m_clrTextSelInActive	= CBCGPDrawManager::PixelAlpha (m_clrTextSelActive, 110);
}
//**************************************************************************************
COLORREF CBCGPEditCtrl::GetDefaultTextColor () const
{
	return (m_clrText == -1) ? ::GetSysColor (COLOR_WINDOWTEXT) : m_clrText;
}
//**************************************************************************************
COLORREF CBCGPEditCtrl::GetDefaultBackColor () const
{
	return (m_clrBack == -1) ? ::GetSysColor (COLOR_WINDOW) : m_clrBack;
}
//**************************************************************************************
void CBCGPEditCtrl::VirtToClient (CPoint& pt)
{
	pt.x -= m_nScrollOffsetHorz * m_nMaxCharWidth;
	pt.y -= m_nScrollOffsetVert * m_nLineHeight;
}
//**************************************************************************************
void CBCGPEditCtrl::ClientToVirt (CPoint& pt)
{
	pt.x += m_nScrollOffsetHorz * m_nMaxCharWidth;
	pt.y += m_nScrollOffsetVert * m_nLineHeight;
}
//**************************************************************************************
void CBCGPEditCtrl::VirtToClient (CRect& rect)
{
	CPoint ptTop = rect.TopLeft ();
	VirtToClient (ptTop);
	CPoint ptBottom = rect.BottomRight ();
	VirtToClient (ptBottom);
	rect.SetRect (ptTop, ptBottom);
}
//**************************************************************************************
void CBCGPEditCtrl::ClientToVirt (CRect& rect)
{
	CPoint ptTop = rect.TopLeft ();
	ClientToVirt (ptTop);
	CPoint ptBottom = rect.BottomRight ();
	ClientToVirt (ptBottom);
	rect.SetRect (ptTop, ptBottom);
}
//**************************************************************************************
void CBCGPEditCtrl::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT_VALID(this);
	ASSERT_VALID(pDC);

	if (m_hPrinterFont == NULL)
	{
		// get current screen font object metrics
		CFont* pFont = GetFont();
		LOGFONT lf;
		LOGFONT lfSys;

		if (pFont == NULL)
		{
			pFont = CFont::FromHandle ((HFONT) ::GetStockObject (DEFAULT_GUI_FONT));
			if (pFont == NULL)
			{
				return;
			}
		}

		VERIFY(pFont->GetObject(sizeof(LOGFONT), &lf));
		VERIFY(::GetObject(::GetStockObject(SYSTEM_FONT), sizeof(LOGFONT),
			&lfSys));
		if (lstrcmpi((LPCTSTR)lf.lfFaceName, (LPCTSTR)lfSys.lfFaceName) == 0)
			return;

		// map to printer font metrics
		HDC hDCFrom = ::GetDC(NULL);
		lf.lfHeight = ::MulDiv(lf.lfHeight, pDC->GetDeviceCaps(LOGPIXELSY),
			::GetDeviceCaps(hDCFrom, LOGPIXELSY));
		lf.lfWidth = ::MulDiv(lf.lfWidth, pDC->GetDeviceCaps(LOGPIXELSX),
			::GetDeviceCaps(hDCFrom, LOGPIXELSX));
		::ReleaseDC(NULL, hDCFrom);

		// create it, if it fails we just use the printer's default.
		m_hMirrorFont = ::CreateFontIndirect(&lf);
		m_hPrinterFont = m_hMirrorFont;
	}

	ASSERT_VALID(this);
}
//***************************************************************************************
void CBCGPEditCtrl::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT_VALID(this);

	if (m_hMirrorFont != NULL && m_hPrinterFont == m_hMirrorFont)
	{
		AfxDeleteObject((HGDIOBJ*)&m_hMirrorFont);
		m_hPrinterFont = NULL;
	}
}
//***************************************************************************************
void CBCGPEditCtrl::OnPrint(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT_VALID(this);
	ASSERT_VALID(pDC);
	ASSERT(pInfo != NULL);
	ASSERT(pInfo->m_bContinuePrinting);

	CFont* pOldFont = SelectFont (pDC);
	pDC->SetBkMode(TRANSPARENT);

	UINT nPage = pInfo->m_nCurPage;

	OnChangeFont (pDC);

	CRect rectText = m_rectText;
	m_rectText = pInfo->m_rectDraw;

	m_rectText.DeflateRect (2 * m_nLineHeight, 2 * m_nLineHeight);

	int nRowsInPage = m_nLineHeight == 0 ? 0 : m_rectText.Height () / m_nLineHeight;

	int nScrollOffsetVert = m_nScrollOffsetVert;
	m_nScrollOffsetVert = (nPage - 1) * nRowsInPage;

	int nTopVisibleOffset = m_nTopVisibleOffset;
	m_nTopVisibleOffset = GetRowStart (m_nScrollOffsetVert);

	int nScrollOffsetHorz = m_nScrollOffsetHorz;
	m_nScrollOffsetHorz = 0;

	OnDrawText (pDC);

	if (GetRowStart (m_nScrollOffsetVert + nRowsInPage) < 0)
	{
		pInfo->m_bContinuePrinting = FALSE;
		pInfo->SetMaxPage (nPage);
	}

	m_nScrollOffsetVert = nScrollOffsetVert;
	m_nScrollOffsetHorz = nScrollOffsetHorz;
	m_nTopVisibleOffset = nTopVisibleOffset;

	m_rectText = rectText;

	if (pOldFont != NULL)
	{
		pDC->SelectObject(pOldFont);
	}

	OnChangeFont ();
}
//**************************************************************************************
void CBCGPEditCtrl::SetPrinterFont(CFont* pFont)
{
	ASSERT_VALID(this);
	m_hPrinterFont = (HFONT)pFont->GetSafeHandle();
	ASSERT_VALID(this);
}
//*************************************************************************************
void CBCGPEditCtrl::Serialize(CArchive& ar) 
{
	if (ar.IsStoring())
	{
		SaveFile (*ar.GetFile());
	}
	else
	{
		OpenFile (*ar.GetFile());
	}
}
//**************************************************************************************
void CBCGPEditCtrl::OnAfterTextChanged (int /*nOffsetFrom*/, const CString& /*strText*/, BOOL /*bInserted*/)
{
	GetParent ()->SendMessage (BCGM_ON_EDITCHANGE);
}
//**************************************************************************************
UINT CBCGPEditCtrl::OnGetDlgCode() 
{
	if (m_nDlgCode == (UINT) -1)
	{
		return CWnd::OnGetDlgCode();
	}
	return m_nDlgCode;
}
